/*
 *  Copyright (C) 2004 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <signal.h>

#include "Reseau.h"
#include "Sous_Reseau.h"

static const char version[] = "AutoScan_d 0.93: compilé le " __TIME__ " " __DATE__ "\n";

void     *Thread_Recherche_Ip ();
void     *ScanMachine (int);
void     *Nouveau_Sous_Reseau (int);


void segfault_handler (int sig)
{
	printf ("\n[S] Segmentation fault    Et oui. Encore un bug   pid : %d\n", getpid ());
	Message ("", AUTOSCAN_SEGMENTATION_FAULT);
	Message ("autoscan_d : Segmentation fault", ERREUR);
	exit (0);
}

void Control_C (int sig)
{
	printf ("[S] Demande d'arret\n");
	printf ("[S] Fin\n");
	Message ("", AUTOSCAN_D_ARRETER);
	exit (0);
}


void     *Boucle_Scan ()
{
	int       Machine;

	printf ("[S] Demarrage boucle pid(%d)  scan    En cours de scan (%d) / (%d)\n", getpid (), Nb_Max_Thread - Valeur_Semaphore (&Semaphore_Thread_Scan), Valeur_Semaphore (&Semaphore_Nouvelle_Machine));
	Machine = Nb_Machine_Dans_La_Liste;	// different de 0 si il y eu une restauration
	while (1) {
		sem_wait (&Semaphore_Nouvelle_Machine);
		sem_wait (&Semaphore_Thread_Scan);
		Message_Info ();
		printf ("[S] Boucle scan n %d Ip = [%s]  Mac = [%s] En cours de scan (%d) / (%d)\n", Machine, Liste_Machines[Machine].Adresse_Ip, Liste_Machines[Machine].Adresse_Mac, Nb_Max_Thread - Valeur_Semaphore (&Semaphore_Thread_Scan), Valeur_Semaphore (&Semaphore_Nouvelle_Machine));
		demmarage_Thread (ScanMachine, Machine);
		Machine++;
	}
}

void     *Boucle_demande ()
{
	struct struct_Demande Demande;
	int       Taille_Recuperer, i;
	unsigned long int AdresseIp;
	int       NumSousReseau;
	char      tmp[50];
	struct sockaddr sock;
	int       Socket;


	printf ("[S] Demarrage boucle demande\n");
	while (1) {
		if ((Taille_Recuperer = msgrcv (File_Envoie_Demande, (void *) &Demande, sizeof (Demande) - sizeof (long), 0, 0)) > -1) {
			switch (Demande.type_demande) {
			    case RE_SCAN_MACHINE:
				    printf ("[S] Demande rescanne machine : %s\n", Demande.donnee);
				    i = 0;
				    while ((i < Nb_Machine_Dans_La_Liste) && (strcmp (Liste_Machines[i].Adresse_Ip, Demande.donnee) != 0)) {
					    i++;
				    }
				    if (strcmp (Liste_Machines[i].Adresse_Ip, Demande.donnee) == 0) {
					    printf ("[S] Rescanne la machine : %s    Machine n%d\n", Demande.donnee, i);
					    Liste_Machines[i].Machine_ReScanner = TRUE;
					    demmarage_Thread (ScanMachine, i);
				    }
				    else {
					    sprintf (tmp, "unreachable machine : %s\n", Demande.donnee);
					    Message (tmp, ERREUR);

				    }
				    break;
			    case SCAN_SOUS_RESEAU:
				    printf ("[S] Rescanne un sous reseau : %s\n", Demande.donnee);
				    AdresseIp = inet_addr (Demande.donnee);
				    printf ("[S] %04X\n", AdresseIp);
				    if (AdresseIp != INADDR_NONE) {
					    /* Stocke les nouveaux sous rseau */
					    NumSousReseau = 0;
					    while ((Sous_Reseau[NumSousReseau].Reseau[0] != 0)
						    && (memcmp (&Sous_Reseau[NumSousReseau].Reseau, &AdresseIp, IP_ADDR_LEN - 1) != 0)) {
						    NumSousReseau++;
					    }
					    if (Sous_Reseau[NumSousReseau].Reseau[0] == 0) {
						    memcpy (&Sous_Reseau[NumSousReseau].Reseau, &AdresseIp, IP_ADDR_LEN - 1);
						    demmarage_Thread (Nouveau_Sous_Reseau, NumSousReseau);
					    }
					    else {
						    printf ("[S] Ce sous reseau existe deja\n");
						    Message ("Subnet is already existing", INFO);
					    }
				    }
				    else
					    printf ("[S] Adresse Incorrecte\n");
				    break;
			    case MACHINE_RESTAURER:
				    printf ("[S] Machine restaurer : (%s)    [%s] \n", Demande.Adresse_Ip, Demande.Adresse_Mac);
				    strcpy (Liste_Machines[Nb_Machine_Dans_La_Liste].Adresse_Ip, Demande.Adresse_Ip);
				    strcpy (Liste_Machines[Nb_Machine_Dans_La_Liste].Adresse_Mac, Demande.Adresse_Mac);
				    Liste_Machines[Nb_Machine_Dans_La_Liste].AdresseIp = Demande.AdresseIp;
				    memcpy (Liste_Machines[Nb_Machine_Dans_La_Liste].AdresseMAC, Demande.AdresseMAC, 6);
				    Liste_Machines[Nb_Machine_Dans_La_Liste].Scan_Port_En_Cours = FALSE;
				    Liste_Machines[Nb_Machine_Dans_La_Liste].Machine_Restaurer = TRUE;
				    Liste_Machines[Nb_Machine_Dans_La_Liste].Machine_ReScanner = FALSE;
				    Liste_Machines[Nb_Machine_Dans_La_Liste].Machine_Allumer = FALSE;	// Cette machine est restaurer donc pas dfaut on considre qu'elle est teint
				    printf ("[S] Nouvelle Machine (%d) : (%s)  [%s]  \n", Nb_Machine_Dans_La_Liste, Liste_Machines[Nb_Machine_Dans_La_Liste].Adresse_Ip, Liste_Machines[Nb_Machine_Dans_La_Liste].Adresse_Mac);
				    Nb_Machine_Dans_La_Liste++;

				    //Envoie_Ping();
				    //exit(0);
				    break;
			    case DEMANDE_ARRET_AUTOSCAN_D:
				    printf ("[S] Demande d'arret recu\n");
				    printf ("[S] Fin\n");
				    Message ("", AUTOSCAN_D_ARRETER);
				    exit (0);
				    break;
			    case DEMARRAGE_SANS_SCAN:
				    printf ("[S] Interdiction de scan\n");
				    Message_Info ();
				    printf ("[S] Interface Reseau selectionner : [%s]    Max thread : (%d)\n", Demande.interface, Demande.Nb_Thread_Max);
				    /* Interface au demarrage */
				    Interface = (char *) malloc (strlen (Demande.interface));
				    strcpy (Interface, Demande.interface);
				    strcpy (pwd_Utilisateur, Demande.donnee);
				    Nb_Max_Thread = Demande.Nb_Thread_Max;
				    sem_init (&Semaphore_Nouvelle_Machine, 0, 0);
				    sem_init (&Semaphore_Thread_Scan, 0, Nb_Max_Thread);
				    Autoriser_Scan = FALSE;
				    strcpy (sock.sa_data, Interface);
				    sock.sa_family = AF_INET;
				    if ((Socket = socket (AF_INET, SOCK_PACKET, htons (ETH_P_ALL))) < 0) {
					    perror ("[S] Socket: ");
					    exit (1);
				    }
				    printf ("[S] ********* Information ***********\n");
				    Get_My_MAC (Socket, Interface);
				    Get_My_Ip (Socket, Interface);
				    printf ("[S]  *  pwd de l'utilisateur : [%s]\n", pwd_Utilisateur);
				    printf ("[S] *********************************\n");
				    close (Socket);
				    Creer_PortList ();
				    Creer_SignatureList ();
				    break;
			    case DEMARRAGE_SCAN:
				    printf ("[S] Autorisation de scan accord\n");
				    Message_Info ();
				    printf ("[S] Interface Reseau selectionner : [%s]    Max thread : (%d)\n", Demande.interface, Demande.Nb_Thread_Max);
				    /* Interface au demarrage */
				    Interface = (char *) malloc (strlen (Demande.interface));
				    strcpy (Interface, Demande.interface);
				    strcpy (pwd_Utilisateur, Demande.donnee);
				    Nb_Max_Thread = Demande.Nb_Thread_Max;
				    sem_init (&Semaphore_Nouvelle_Machine, 0, 0);
				    sem_init (&Semaphore_Thread_Scan, 0, Nb_Max_Thread);
				    strcpy (sock.sa_data, Interface);
				    sock.sa_family = AF_INET;
				    if ((Socket = socket (AF_INET, SOCK_PACKET, htons (ETH_P_ALL))) < 0) {
					    perror ("[S] Socket: ");
					    exit (1);
				    }
				    printf ("[S] ********* Information ***********\n");
				    Get_My_MAC (Socket, Interface);
				    Get_My_Ip (Socket, Interface);
				    printf ("[S]  *  pwd de l'utilisateur : [%s]\n", pwd_Utilisateur);
				    printf ("[S] *********************************\n");
				    close (Socket);
				    Creer_PortList ();
				    Creer_SignatureList ();
				    Creer_Liste_Sous_Reseau ();
				    i = 0;
				    while (Sous_Reseau[i].Reseau[0] != 0) {
					    if (!Sous_Reseau[i].Deja_Scanner) {	// probleme rescan 
						    demmarage_Thread (Nouveau_Sous_Reseau, i);
					    }
					    i++;
				    }
				    Autoriser_Scan = TRUE;
				    demmarage_Thread (Thread_Recherche_Ip, NULL);
				    demmarage_Thread (Boucle_Scan, NULL);
				    break;
			    case ARRET_SCAN:
				    Autoriser_Scan = FALSE;
				    break;
			    case RELECTURE_FICHIER_CONFIGURATION:
				    Relecture_Fichiers_Configuration = TRUE;
				    break;
				case REFRESH_MACHINE:
					 printf ("[S] Demande refresh machine : %s\n", Demande.donnee);
					i = 0;
				    while ((i < Nb_Machine_Dans_La_Liste) && (strcmp (Liste_Machines[i].Adresse_Ip, Demande.donnee) != 0)) {
					    i++;
				    }
				    if (strcmp (Liste_Machines[i].Adresse_Ip, Demande.donnee) == 0) {
					    printf ("[S] Refresh la machine : %s    Machine n%d\n", Demande.donnee, i);
						Refresh (i);
				    }
				    else {
					    sprintf (tmp, "unreachable machine : %s\n", Demande.donnee);
					    Message (tmp, ERREUR);
				    }
					break;
			    default:
				    printf ("[S] Type de la demande Inconnu\n");
			}
		}
		else {
			printf ("[S] Erreur taille de demanden");
			exit (-1);
		}
	}
}


int main (int argc, char *argv[])
{
	int       i;

	Nb_Machine_Dans_La_Liste = 0;
	Autoriser_Scan = FALSE;
	Relecture_Fichiers_Configuration = FALSE;
	printf (version);
	/* Elevation des privileges */
	setuid (0);
	seteuid (0);
	if ((getuid () != 0) || (geteuid () != 0)) {
		printf ("[S] Root Only\n");
		exit (0);
	}
	if ((File_Reception_Nouvelle_Machine = msgget (KEY_FILE_RECEPTION_NOUVELLE_MACHINE, 0)) == -1) {
		perror ("[S] msgget");
		printf ("Lancer 'AutoScan'\n");
		exit (1);
	}
	if ((File_Envoie_Demande = msgget (KEY_FILE_DEMANDE, 0)) == -1) {
		perror ("[S] msgget");
		exit (1);
	}
	if ((File_Envoie_Message = msgget (KEY_FILE_MESSAGE, 0)) == -1) {
		perror ("[S] msgget");
		exit (1);
	}
	if ((File_Envoie_Message_Temps_Reel = msgget (KEY_FILE_MESSAGE_TEMP_REEL, 0)) == -1) {
		perror ("[S] msgget");
		exit (1);
	}
	printf ("[S] Attachement des files des messages : OK\n");

	signal (SIGSEGV, segfault_handler);
	signal (SIGINT, Control_C);
	Initialise_Structure_Sous_Reseau ();
	demmarage_Thread (Boucle_demande, NULL);
	while (1) {
		pause ();
	}
	return 0;
}
