/*
 *  Copyright (C) 2004 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "callbacks.h"
#include <gtk/gtk.h>
#include <string.h>

#include <libgtkhtml/graphics/htmlpainter.h>
#include <libgtkhtml/layout/htmlbox.h>
#include <libgtkhtml/view/htmlview.h>


HtmlDocument *document;
HtmlParser *parser;
GtkWidget *view;

typedef struct _FetchContext FetchContext;

struct _FetchContext {
	HtmlStream *stream;
	gchar    *url;
};

enum {
	TITLE_COLUMN,
	FILENAME_COLUMN,
	NUM_COLUMNS
};



static void load_file (const gchar * filename)
{
	FILE     *file;

	/* FIXME: loading of bigger files */
	gchar     buffer[300000];
	gint      i;
	gdouble   elapsed_time;
	gchar    *path;
	GTimer   *timer;


	if (!filename)
		return;

	path = g_strdup_printf ("%s", filename);
	timer = g_timer_new ();

	memset (buffer, 0, sizeof (buffer));

	html_view_set_document (HTML_VIEW (view), NULL);
	html_document_clear (document);
	html_view_set_document (HTML_VIEW (view), document);

	file = fopen (path, "r");
	printf ("Load : %s\n", path);
	g_free (path);

	if (!file)
		return;

	printf ("document: %s\n", document);
	if (html_document_open_stream (document, "text/html")) {
		while ((i = fread (&buffer, 1, 10, file))) {
			html_document_write_stream (document, buffer, i);
		}

		html_document_close_stream (document);
	}

	elapsed_time = g_timer_elapsed (timer, NULL);

	g_print ("Parsing time is %f secs\n", elapsed_time);
}



static void link_clicked (HtmlDocument * doc, const gchar * url)
{
	g_print ("link clicked: %s!\n", url);
	load_file (url);
}





static gboolean url_requested_timeout (FetchContext * context)
{
	gchar    *file = NULL;
	gchar    *path;

	if (g_path_is_absolute (context->url))
		path = g_strdup (context->url);
	else
		path = g_strdup_printf ("%s", context->url);


	if (g_file_test (path, G_FILE_TEST_EXISTS)) {
		gint      i;
		guchar    buf[4096];
		FILE     *f = fopen (path, "r");

		while ((i = fread (buf, 1, 4096, f)) != 0) {
			//g_print ("i: %d\n", i);
			html_stream_write (context->stream, buf, i);

			if (gtk_events_pending ())
				gtk_main_iteration ();

		}

		html_stream_close (context->stream);
		fclose (f);

	}
	else {
		g_print ("eeek, wrong!\n");
		html_stream_close (context->stream);
	}

	g_free (file);
	g_free (context->url);
	g_free (context);

	return FALSE;
}


static gboolean url_requested (HtmlDocument * doc, const gchar * url, HtmlStream * stream)
{
	FetchContext *context = g_new (FetchContext, 1);

	context->stream = stream;
	context->url = g_strdup (url);

	printf ("context is: %s\n", url);

	gtk_timeout_add (200, (GtkFunction) url_requested_timeout, context);

	return TRUE;
}





void Init_Documentation (char *Fichier)
{
	GtkWidget *tree_view;
	GtkWidget *vbox, *hbox;
	GtkWidget *button, *sw;

	if (!Mode_Documentation_Demarrage)
		gtk_widget_destroy (vpaned);
	else
		gtk_widget_destroy (frame_Documentation);

	document = html_document_new ();
	g_signal_connect (G_OBJECT (document), "request_url", G_CALLBACK (url_requested), NULL);
	g_signal_connect (G_OBJECT (document), "link_clicked", G_CALLBACK (link_clicked), NULL);
	view = html_view_new ();
	sw = gtk_scrolled_window_new (gtk_layout_get_hadjustment (GTK_LAYOUT (view)), gtk_layout_get_vadjustment (GTK_LAYOUT (view)));
	gtk_container_add (GTK_CONTAINER (sw), view);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	frame_Documentation = gtk_frame_new (NULL);
	gtk_container_add (GTK_CONTAINER (frame_Documentation), sw);
	gtk_widget_set_size_request (frame_Documentation, 380, 394);
	gtk_paned_pack2 (GTK_PANED (hpaned), frame_Documentation, FALSE, FALSE);
	load_file (Fichier);
	html_view_set_document (HTML_VIEW (view), document);
	gtk_widget_show (sw);
	gtk_widget_show (view);
	gtk_widget_show (frame_Documentation);
}
