/*
 *  Copyright (C) 2004 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Reseau.h"
#include "callbacks.h"
#include "Machine.h"


void Initialise_Structure_Machine (int Machine)
{
	printf ("[I] Machine %d initialise\n", Machine);
	Liste_Machines[Machine].Scan_Port = FALSE;
	Liste_Machines[Machine].Serveur = FALSE;
	Liste_Machines[Machine].Debug_Scan = NULL;
	Liste_Machines[Machine].Nb_Port_Ouvert = 0;
	Liste_Machines[Machine].TypeMateriel = INCONNU;
	Liste_Machines[Machine].color.red = -63;
	Liste_Machines[Machine].color.green = -60;
	Liste_Machines[Machine].color.blue = -59;
	Initialise_Structure_SMB (Machine);


}

void Ajoute_Nouvelle_Machine (int NumMachine, char *Adresse_Ip, char *Adresse_Mac, unsigned long int AdresseIp, char *AdresseMac, gboolean NouvelleMachine)
{
	strcpy (Liste_Machines[NumMachine].Adresse_Ip, Adresse_Ip);
	strcpy (Liste_Machines[NumMachine].Adresse_Mac, Adresse_Mac);
	Liste_Machines[NumMachine].AdresseIp = AdresseIp;
	memcpy (Liste_Machines[NumMachine].AdresseMAC, AdresseMac, 6);
	printf ("[I] Nouvelle Machine (%d) : (%s)  [%s]  %04X\n", NumMachine, Liste_Machines[NumMachine].Adresse_Ip, Liste_Machines[NumMachine].Adresse_Mac, Liste_Machines[NumMachine].AdresseIp);
	if (NouvelleMachine) {
		if (NumMachine != Nb_Machine_Trouve) {
			printf ("[I] Erreur : Nb de machines trouve\n");
			exit (-1);
		}
		Nb_Machine_Trouve++;
	}
}

void Affiche_Nouvelle_Machine (int Machine, gboolean Rescan, gboolean Allumer)
{
	gchar    *titre[1], *text[1];
	int       Num_Domaine_Serveur, Num_Domaine_Station;
	GtkCTreeNode *Node_Parent;
	GdkColor  couleur_Texte;


	printf ("[I] Affichage machine : %d\n", Machine);
	Liste_Machines[Machine].Machine_Allumer = Allumer;

	/* Supprime la machine de la liste */
	if (!Rescan) {
		printf ("[I] La machine existe deja dans la liste  : Effacement \n");
		gtk_ctree_remove_node (GTK_CTREE (ctree_Liste_Machine), Liste_Machines[Machine].ctree);
	}


	if (Liste_Machines[Machine].Smb.HostName[0] == 0)
		titre[0] = g_strdup_printf (Liste_Machines[Machine].Adresse_Ip);
	else
		titre[0] = g_strdup_printf (Liste_Machines[Machine].Smb.HostName);

	Node_Parent = NULL;
	/* Recherche du parent */
	if (Node_Parent == NULL)
		Node_Parent = Ajoute_Domaine_Serveur (Machine);
	if (Node_Parent == NULL)
		Node_Parent = Ajoute_Domaine_Station (Machine);
	if (Node_Parent == NULL)
		Node_Parent = Ajoute_Imprimante (Machine);
	if (Node_Parent == NULL)
		Node_Parent = Ajoute_Routeur_Switch (Machine);
	if (Node_Parent == NULL)
		Node_Parent = Ajoute_Routeur (Machine);
	if (Node_Parent == NULL)
		Node_Parent = Ajoute_Switch (Machine);
	if (Node_Parent == NULL)
		Node_Parent = Ajoute_Point_Access_Wifi (Machine);
	if ((Node_Parent == NULL) && (Liste_Machines[Machine].Nb_Port_Ouvert > 0) && (Liste_Machines[Machine].Serveur) && (Liste_Machines[Machine].TypeMateriel == STATION)) {
		Creer_Node_Serveur ();
		Node_Parent = Node_Serveur;	// c'est un serveur mais il n'est pas dans un domaine
	}
	if ((Node_Parent == NULL) && (Liste_Machines[Machine].Nb_Port_Ouvert > 0) && (Liste_Machines[Machine].TypeMateriel == STATION)) {
		Creer_Node_Station ();
		Node_Parent = Node_Station;	// c'est un station mais il n'est pas dans un domaine
	}

	if (Liste_Machines[Machine].Serveur)
		printf ("[I] Serveur ");
	else
		printf ("[I] pas serveur ");
	printf ("[I] nb port ouvert : [%d]  type materiel : [%d] \n", Liste_Machines[Machine].Nb_Port_Ouvert, Liste_Machines[Machine].TypeMateriel);

	if (Node_Parent == NULL) {
		/* Créer le noeud "Inconnu " */
		printf("[I]  nb total port (%d)   nb total firewalle (%d)\n",Liste_Machines[Machine].Nb_Port_Total_Scanner,Liste_Machines[Machine].Nb_Port_Firewaller);
		if (Liste_Machines[Machine].Nb_Port_Total_Scanner==Liste_Machines[Machine].Nb_Port_Firewaller) {	// Machine firewallé
			if (Node_Firewall == NULL) {
				text[0] = g_strdup_printf ("Firewall");
				Node_Firewall = gtk_ctree_insert_node (GTK_CTREE (ctree_Liste_Machine), NULL, NULL, text, 5, Pixmap_Firewall, Masque_Firewall, Pixmap_Firewall, Masque_Firewall, FALSE, FALSE);
				gtk_ctree_node_set_row_data (GTK_CTREE (ctree_Liste_Machine), Node_Firewall, (gpointer *) FIREWALL);
				g_free (text[0]);
			}
			Liste_Machines[Machine].ctree = gtk_ctree_insert_node (GTK_CTREE (ctree_Liste_Machine), Node_Firewall, NULL, titre, 5, Pixmap_Firewall, Masque_Firewall, Pixmap_Firewall, Masque_Firewall, FALSE, FALSE);
			Liste_Machines[Machine].ctree_Parent = Node_Firewall;
			gtk_ctree_node_set_row_data (GTK_CTREE (ctree_Liste_Machine), Liste_Machines[Machine].ctree, (gpointer *) Machine);
		}
		else {
			if (Node_Inconnu == NULL) {
				text[0] = g_strdup_printf ("Unknown equipment");
				Node_Inconnu = gtk_ctree_insert_node (GTK_CTREE (ctree_Liste_Machine), NULL, NULL, text, 5, Pixmap_Inconnu, Masque_Inconnu, Pixmap_Inconnu, Masque_Inconnu, FALSE, FALSE);
				gtk_ctree_node_set_row_data (GTK_CTREE (ctree_Liste_Machine), Node_Inconnu, (gpointer *) INCONNU);
				couleur_Texte.red = -1 * 255;
				couleur_Texte.green = 0;
				couleur_Texte.blue = 0;
				gtk_ctree_node_set_background (GTK_CTREE (ctree_Liste_Machine), Node_Inconnu, &couleur_Texte);
				g_free (text[0]);
				Init_Documentation ("/usr/share/doc/AutoScan/Os1.html");
				Mode_Documentation = TRUE;
				Mode_Documentation_Demarrage = FALSE;
				Message ("Unknown Equipment", GNOME_MESSAGE_BOX_WARNING);
				MachineSelectionner_Avant_Documentation = MachineSelectionner;

			}
			Liste_Machines[Machine].ctree = gtk_ctree_insert_node (GTK_CTREE (ctree_Liste_Machine), Node_Inconnu, NULL, titre, 5, Pixmap_Inconnu, Masque_Inconnu, Pixmap_Inconnu, Masque_Inconnu, FALSE, FALSE);
			Liste_Machines[Machine].ctree_Parent = Node_Inconnu;
			gtk_ctree_node_set_row_data (GTK_CTREE (ctree_Liste_Machine), Liste_Machines[Machine].ctree, (gpointer *) Machine);
		}
	}
	else {
		if (Allumer) {
			couleur_Texte.red = 0;
			couleur_Texte.green = 0;
			couleur_Texte.blue = 0;
			Liste_Machines[Machine].ctree = gtk_ctree_insert_node (GTK_CTREE (ctree_Liste_Machine), Node_Parent, NULL, titre, 5, Liste_Machines[Machine].Image, Liste_Machines[Machine].Masque, Liste_Machines[Machine].Image, Liste_Machines[Machine].Masque, FALSE, FALSE);
			gtk_ctree_node_set_foreground (GTK_CTREE (ctree_Liste_Machine), Liste_Machines[Machine].ctree, &couleur_Texte);
		}
		else {
			couleur_Texte.red = -65 * 255;
			couleur_Texte.green = -65 * 255;
			couleur_Texte.blue = -65 * 255;
			Liste_Machines[Machine].ctree = gtk_ctree_insert_node (GTK_CTREE (ctree_Liste_Machine), Node_Parent, NULL, titre, 5, Liste_Machines[Machine].Image_Eteint, Liste_Machines[Machine].Masque_Eteint, Liste_Machines[Machine].Image_Eteint, Liste_Machines[Machine].Masque_Eteint, FALSE, FALSE);
			gtk_ctree_node_set_foreground (GTK_CTREE (ctree_Liste_Machine), Liste_Machines[Machine].ctree, &couleur_Texte);
		}
		Liste_Machines[Machine].ctree_Parent = Node_Parent;
		gtk_ctree_node_set_row_data (GTK_CTREE (ctree_Liste_Machine), Liste_Machines[Machine].ctree, (gpointer *) Machine);
	}
	/* Modification de la couleur de fond */
	gtk_ctree_node_set_background (GTK_CTREE (ctree_Liste_Machine), Liste_Machines[Machine].ctree, &Liste_Machines[Machine].color);
	g_free (titre[0]);
}
