/*
 *  Copyright (C) 2004 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <bonobo.h>
#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"


#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)




GtkWidget *create_about (void)
{
	const gchar *authors[] = { "Developer :","  Thierry Lagarde <tlagarde@users.sourceforge.net> \n       Nantes, France\n\n","Logo :"," Rhandros Dembicki <rhandros@yellowicon.com>\n\n",
		"Icons :"," Crystal GT : http://www.linuxcult.com/forum/crystal.php",
		NULL
	};
	const gchar *documenters[] = { NULL };
	gchar    *translators = NULL;
	GdkPixbuf *about_logo_pixbuf;

	about_logo_pixbuf = create_pixbuf ("/usr/share/pixmaps/AutoScan/Logo.png");
	about = gnome_about_new ("AutoScan", "beta 0.93", "Copyright (c) Lagarde thierry", "AutoScan is a utility for network exploration.", authors, documenters, translators, about_logo_pixbuf);
	gtk_container_set_border_width (GTK_CONTAINER (about), 5);
	gtk_window_set_destroy_with_parent (GTK_WINDOW (about), TRUE);
	about_icon_pixbuf = create_pixbuf ("/usr/share/pixmaps/AutoScan/AutoScan.png");
	if (about_icon_pixbuf) {
		gtk_window_set_icon (GTK_WINDOW (about), about_icon_pixbuf);
		gdk_pixbuf_unref (about_icon_pixbuf);
	}

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (about, about, "about");

	return about;
}

static GnomeUIInfo rechercher_menu_uiinfo[] = {
	{
			GNOME_APP_UI_ITEM, "Host",
			NULL,
			(gpointer) on_une_machine_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/machine.png",
		0, (GdkModifierType) 0, NULL},
	{
			GNOME_APP_UI_ITEM, "Proxy",
			NULL,
			(gpointer) on_le_proxy_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/proxy_1.png",
		0, (GdkModifierType) 0, NULL},
	{
			GNOME_APP_UI_ITEM, "DNS server",
			NULL,
			(gpointer) on_serveur_dns_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/dns.png",
		0, (GdkModifierType) 0, NULL},
	{
			GNOME_APP_UI_ITEM, "DHCP server",
			NULL,
			(gpointer) on_serveur_dhcp_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/dhcp.png",
		0, (GdkModifierType) 0, NULL},
	GNOMEUIINFO_END
};

static GnomeUIInfo r__seau_menu_uiinfo[] = {
	{
			GNOME_APP_UI_ITEM, "Save",
			NULL,
			(gpointer) on_enregistrer_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/filesave.png",
		0, (GdkModifierType) 0, NULL},
	GNOMEUIINFO_SEPARATOR,
	{
			GNOME_APP_UI_ITEM, "Add a subnet",
			NULL,
			(gpointer) on_force_un_sous_r__seau_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/network.png",
		0, (GdkModifierType) 0, NULL},
	{
			GNOME_APP_UI_SUBTREE, "Search",
			NULL,
			rechercher_menu_uiinfo, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/find.png",
		0, (GdkModifierType) 0, NULL},
	GNOMEUIINFO_SEPARATOR,
	{
			GNOME_APP_UI_ITEM, "Quit",
			NULL,
			(gpointer) on_quitter_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/exit.png",
		0, (GdkModifierType) 0, NULL},
	GNOMEUIINFO_END
};

static GnomeUIInfo configuration1_menu_uiinfo[] = {
	{
			GNOME_APP_UI_ITEM, "Ports list",
			NULL,
			(gpointer) on_liste_ports_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/List.png",
		0, (GdkModifierType) 0, NULL},
	{
			GNOME_APP_UI_ITEM, "Scripts list",
			NULL,
			(gpointer) on_ajoute_un_script_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/script.png",
		0, (GdkModifierType) 0, NULL},
	{
			GNOME_APP_UI_ITEM, "OS fingerprinting list",
			NULL,
			(gpointer) on_liste_des_signatures_activate, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, "gnome-stock-book-red",
		0, (GdkModifierType) 0, NULL},
	GNOMEUIINFO_END
};

static GnomeUIInfo aide1_menu_uiinfo[] = {
	GNOMEUIINFO_MENU_ABOUT_ITEM (on____propos_activate, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo menubar_uiinfo[] = {
	{
			GNOME_APP_UI_SUBTREE, "Network",
			NULL,
			r__seau_menu_uiinfo, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
		0, (GdkModifierType) 0, NULL},
	{
			GNOME_APP_UI_SUBTREE, "Settings",
			NULL,
			configuration1_menu_uiinfo, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
		0, (GdkModifierType) 0, NULL},
	GNOMEUIINFO_MENU_HELP_TREE (aide1_menu_uiinfo),
	GNOMEUIINFO_END
};

static GnomeUIInfo menu_Liste_Modes_uiinfo[] = {
	GNOMEUIINFO_END
};


/*
static void
select_icon (GnomeIconList *gil, gint n, GdkEvent *event, gpointer data)
{
	printf ("Icon %d selected", n);

	if (event)
		printf (" with event type %d\n", (int) event->type);
	else
		printf ("\n");
}*/
void Init_Info_Mode ()
{
	vpaned = gtk_vpaned_new ();
	gtk_widget_show (vpaned);
	gtk_paned_pack2 (GTK_PANED (hpaned), vpaned, FALSE, FALSE);
	gtk_paned_set_position (GTK_PANED (vpaned), 179);
	scrolledwindow_Info = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Info);
	gtk_paned_pack1 (GTK_PANED (vpaned), scrolledwindow_Info, FALSE, FALSE);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_Info), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	ctree_Info = gtk_ctree_new (1, 0);
	gtk_widget_show (ctree_Info);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Info), ctree_Info);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Info), 0, 80);
	gtk_clist_column_titles_hide (GTK_CLIST (ctree_Info));
	fixed_Modes = gtk_fixed_new ();
	gtk_widget_show (fixed_Modes);
	gtk_paned_pack2 (GTK_PANED (vpaned), fixed_Modes, FALSE, FALSE);
	viewport_Modes = gtk_viewport_new (NULL, NULL);
	gtk_widget_show (viewport_Modes);
	gtk_fixed_put (GTK_FIXED (fixed_Modes), viewport_Modes, 0, 30);
	gtk_widget_set_size_request (viewport_Modes, 380, 308);
	scrolledwindow_IconList = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_IconList);
	gtk_container_add (GTK_CONTAINER (viewport_Modes), scrolledwindow_IconList);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_IconList), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	//iconlist = gnome_icon_list_new_flags(78, NULL, GNOME_ICON_LIST_IS_EDITABLE );
	//gnome_icon_list_set_selection_mode (GNOME_ICON_LIST (iconlist), GTK_SELECTION_BROWSE);
	//iconlist = gnome_icon_list_new (78, NULL, 0);
	//gtk_widget_show (iconlist);


	iconlist = gnome_icon_list_new (78, NULL, 0);

	//gtk_container_add (GTK_CONTAINER (scrolledwindow_IconList), iconlist);
	gnome_icon_list_set_selection_mode (GNOME_ICON_LIST (iconlist), GTK_SELECTION_MULTIPLE);

	enum {
		TARGET_STRING,
		TARGET_URL
	};

	static GtkTargetEntry targetentries[] = {
		{"text/plain", 0, TARGET_STRING}
	};


	static GtkTargetEntry target_table[] = {
		{"STRING", 0, TARGET_STRING},
		{"text/plain", 0, TARGET_STRING},
		{"text/uri-list", 0, TARGET_URL}
	};

	static guint n_targets = sizeof (target_table) / sizeof (target_table[0]);


	gtk_drag_dest_set (iconlist, GTK_DEST_DEFAULT_ALL, targetentries, 1, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
	gtk_drag_source_set (iconlist, GDK_BUTTON1_MASK | GDK_BUTTON2_MASK, target_table, n_targets, GDK_ACTION_COPY);



	gtk_widget_show (iconlist);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_IconList), iconlist);
	optionmenu_Liste_Modes = gtk_option_menu_new ();
	gtk_widget_show (optionmenu_Liste_Modes);
	gtk_fixed_put (GTK_FIXED (fixed_Modes), optionmenu_Liste_Modes, 0, 0);
	gtk_widget_set_size_request (optionmenu_Liste_Modes, 248, 29);
	menu_Liste_Modes = gtk_menu_new ();
	gnome_app_fill_menu (GTK_MENU_SHELL (menu_Liste_Modes), menu_Liste_Modes_uiinfo, NULL, FALSE, 0);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu_Liste_Modes), menu_Liste_Modes);
	button_Liste_Modes_Favorite = gtk_button_new ();
	//gtk_widget_show (button_Liste_Modes_Favorite);
	gtk_fixed_put (GTK_FIXED (fixed_Modes), button_Liste_Modes_Favorite, 312, 0);
	gtk_widget_set_size_request (button_Liste_Modes_Favorite, 29, 29);
	gtk_tooltips_set_tip (tooltips, button_Liste_Modes_Favorite, "favorite", NULL);
	image_Liste_Modes_Favorite = create_pixmap (window, "/usr/share/pixmaps/AutoScan/bookmark_add.png");
	gtk_widget_show (image_Liste_Modes_Favorite);
	gtk_container_add (GTK_CONTAINER (button_Liste_Modes_Favorite), image_Liste_Modes_Favorite);
	button_Liste_Modes_Refresh = gtk_button_new ();
	gtk_widget_show (button_Liste_Modes_Refresh);
	gtk_fixed_put (GTK_FIXED (fixed_Modes), button_Liste_Modes_Refresh, 344, 0);
	gtk_widget_set_size_request (button_Liste_Modes_Refresh, 29, 29);
	gtk_tooltips_set_tip (tooltips, button_Liste_Modes_Refresh, "Refresh", NULL);
	image_Liste_Port_Refresh = create_pixmap (window, "/usr/share/pixmaps/AutoScan/reload.png");
	gtk_widget_show (image_Liste_Port_Refresh);
	gtk_container_add (GTK_CONTAINER (button_Liste_Modes_Refresh), image_Liste_Port_Refresh);
	Creer_Menu_Modes ();
	gtk_ctree_set_line_style (GTK_CTREE (ctree_Info), GTK_CTREE_LINES_NONE);
	gtk_clist_set_row_height (GTK_CLIST (ctree_Info), 17);


	g_signal_connect ((gpointer) iconlist, "select_icon", G_CALLBACK (on_iconlist_select_icon), NULL);
	g_signal_connect ((gpointer) iconlist, "event", G_CALLBACK (on_iconlist_event), NULL);
	g_signal_connect ((gpointer) iconlist, "focus_icon", G_CALLBACK (on_iconlist_focus_icon), NULL);
	g_signal_connect ((gpointer) iconlist, "move_cursor", G_CALLBACK (on_iconlist_move_cursor), NULL);
	g_signal_connect ((gpointer) iconlist, "unselect_icon", G_CALLBACK (on_iconlist_unselect_icon), NULL);
	g_signal_connect ((gpointer) iconlist, "toggle_cursor_selection", G_CALLBACK (on_iconlist_toggle_cursor_selection), NULL);
	g_signal_connect ((gpointer) iconlist, "text_changed", G_CALLBACK (on_iconlist_text_changed), NULL);
	g_signal_connect ((gpointer) iconlist, "drag_begin", G_CALLBACK (on_iconlist_drag_begin), NULL);
	g_signal_connect ((gpointer) iconlist, "drag_data_get", G_CALLBACK (on_iconlist_drag_data_get), NULL);
	g_signal_connect ((gpointer) iconlist, "drag_data_received", G_CALLBACK (on_iconlist_drag_data_received), NULL);
	g_signal_connect ((gpointer) iconlist, "drag_end", G_CALLBACK (on_iconlist_drag_end), NULL);
	g_signal_connect ((gpointer) optionmenu_Liste_Modes, "changed", G_CALLBACK (on_optionmenu_Liste_Modes_changed), NULL);
	g_signal_connect ((gpointer) button_Liste_Modes_Favorite, "clicked", G_CALLBACK (on_button_Liste_Modes_Favorite_clicked), NULL);
	g_signal_connect ((gpointer) button_Liste_Modes_Refresh, "clicked", G_CALLBACK (on_button_Liste_Modes_Refresh_clicked), NULL);

/*	GLADE_HOOKUP_OBJECT (window, vpaned, "vpaned");printf("AA17\n");
	GLADE_HOOKUP_OBJECT (window, scrolledwindow_Info, "scrolledwindow_Info");printf("AA17\n");
	GLADE_HOOKUP_OBJECT (window, ctree_Info, "ctree_Info");printf("AA17\n");
	GLADE_HOOKUP_OBJECT (window, fixed_Modes, "fixed_Modes");printf("AA17\n");
	GLADE_HOOKUP_OBJECT (window, viewport_Modes, "viewport_Modes");printf("AA17\n");
	GLADE_HOOKUP_OBJECT (window, scrolledwindow_IconList, "scrolledwindow_IconList");printf("AA17\n");
	GLADE_HOOKUP_OBJECT (window, iconlist, "iconlist");printf("AA17\n");
	GLADE_HOOKUP_OBJECT (window, optionmenu_Liste_Modes, "optionmenu_Liste_Modes");printf("AA17\n");
	GLADE_HOOKUP_OBJECT (window, menu_Liste_Modes, "menu_Liste_Modes");printf("AA17\n");
	GLADE_HOOKUP_OBJECT (window, button_Liste_Modes_Favorite, "button_Liste_Modes_Favorite");printf("AA17\n");
	GLADE_HOOKUP_OBJECT (window, image_Liste_Modes_Favorite, "image_Liste_Modes_Favorite");printf("AA17\n");
	GLADE_HOOKUP_OBJECT (window, button_Liste_Modes_Refresh, "button_Liste_Modes_Refresh");printf("AA17\n");
	GLADE_HOOKUP_OBJECT (window, image_Liste_Port_Refresh, "image_Liste_Port_Refresh");printf("AA17\n");*/
}


GtkWidget *create_window (void)
{
	tooltips = gtk_tooltips_new ();

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (window), "AutoScan Beta 0.93");
	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
	window_icon_pixbuf = create_pixbuf ("/usr/share/pixmaps/AutoScan/AutoScan.png");
	if (window_icon_pixbuf) {
		gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
		gdk_pixbuf_unref (window_icon_pixbuf);
	}

	fixed_Root = gtk_fixed_new ();
	gtk_widget_show (fixed_Root);
	gtk_container_add (GTK_CONTAINER (window), fixed_Root);
	gtk_widget_set_size_request (fixed_Root, 584, 584);

	menubar = gtk_menu_bar_new ();
	gtk_widget_show (menubar);
	gtk_fixed_put (GTK_FIXED (fixed_Root), menubar, 0, 0);
	gtk_widget_set_size_request (menubar, 584, 32);
	gnome_app_fill_menu (GTK_MENU_SHELL (menubar), menubar_uiinfo, NULL, FALSE, 0);

	viewport = gtk_viewport_new (NULL, NULL);
	gtk_widget_show (viewport);
	gtk_fixed_put (GTK_FIXED (fixed_Root), viewport, 0, 30);
	gtk_widget_set_size_request (viewport, 584, 528);

	hpaned = gtk_hpaned_new ();
	gtk_widget_show (hpaned);
	gtk_container_add (GTK_CONTAINER (viewport), hpaned);
	gtk_paned_set_position (GTK_PANED (hpaned), 195);

	scrolledwindow_Liste_Machine = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Liste_Machine);
	gtk_paned_pack1 (GTK_PANED (hpaned), scrolledwindow_Liste_Machine, FALSE, FALSE);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_Liste_Machine), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	ctree_Liste_Machine = gtk_ctree_new (1, 0);
	gtk_widget_show (ctree_Liste_Machine);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Liste_Machine), ctree_Liste_Machine);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Liste_Machine), 0, 80);
	gtk_clist_column_titles_hide (GTK_CLIST (ctree_Liste_Machine));


	Init_Info_Mode ();

	viewport_Info = gtk_viewport_new (NULL, NULL);
	gtk_widget_show (viewport_Info);
	gtk_fixed_put (GTK_FIXED (fixed_Root), viewport_Info, 0, 559);
	gtk_widget_set_size_request (viewport_Info, 584, 26);

	fixed_Info = gtk_fixed_new ();
	gtk_widget_show (fixed_Info);
	gtk_container_add (GTK_CONTAINER (viewport_Info), fixed_Info);

	progressbar_Info = gtk_progress_bar_new ();
	gtk_widget_show (progressbar_Info);
	gtk_fixed_put (GTK_FIXED (fixed_Info), progressbar_Info, 0, 0);
	gtk_widget_set_size_request (progressbar_Info, 192, 24);

	vseparator_Info = gtk_vseparator_new ();
	gtk_widget_show (vseparator_Info);
	gtk_fixed_put (GTK_FIXED (fixed_Info), vseparator_Info, 192, 0);
	gtk_widget_set_size_request (vseparator_Info, 16, 24);

	vseparator_Info2 = gtk_vseparator_new ();
	gtk_widget_show (vseparator_Info2);
	gtk_fixed_put (GTK_FIXED (fixed_Info), vseparator_Info2, 464, 0);
	gtk_widget_set_size_request (vseparator_Info2, 16, 24);

	label_Info = gtk_label_new ("");
	gtk_widget_show (label_Info);
	gtk_fixed_put (GTK_FIXED (fixed_Info), label_Info, 205, 2);
	gtk_widget_set_size_request (label_Info, 262, 16);


	href_autoscan = gnome_href_new ("http://autoscan.free.fr/update.html", _("AutoScan.free.fr"));
	gtk_widget_show (href_autoscan);
	gtk_fixed_put (GTK_FIXED (fixed_Info), href_autoscan, 477, 0);
	gtk_widget_set_size_request (href_autoscan, 105, 25);

	g_signal_connect ((gpointer) window, "destroy", G_CALLBACK (on_window_destroy), NULL);
	g_signal_connect ((gpointer) ctree_Liste_Machine, "tree_select_row", G_CALLBACK (on_ctree_Liste_Machine_tree_select_row), NULL);
	g_signal_connect ((gpointer) ctree_Liste_Machine, "event", G_CALLBACK (on_ctree_Liste_Machine_event), NULL);



	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window, window, "window");
	GLADE_HOOKUP_OBJECT (window, fixed_Root, "fixed_Root");
	GLADE_HOOKUP_OBJECT (window, menubar, "menubar");
	GLADE_HOOKUP_OBJECT (window, menubar_uiinfo[0].widget, "r__seau");
	GLADE_HOOKUP_OBJECT (window, r__seau_menu_uiinfo[0].widget, "enregistrer1");
	GLADE_HOOKUP_OBJECT (window, r__seau_menu_uiinfo[1].widget, "s__parateur6");
	GLADE_HOOKUP_OBJECT (window, r__seau_menu_uiinfo[2].widget, "force_un_sous_r__seau");
	GLADE_HOOKUP_OBJECT (window, r__seau_menu_uiinfo[3].widget, "rechercher");
	GLADE_HOOKUP_OBJECT (window, rechercher_menu_uiinfo[0].widget, "une_machine");
	GLADE_HOOKUP_OBJECT (window, rechercher_menu_uiinfo[1].widget, "le_proxy");
	GLADE_HOOKUP_OBJECT (window, rechercher_menu_uiinfo[2].widget, "serveur_dns");
	GLADE_HOOKUP_OBJECT (window, rechercher_menu_uiinfo[3].widget, "serveur_dhcp");
	GLADE_HOOKUP_OBJECT (window, r__seau_menu_uiinfo[4].widget, "s__parateur9");
	GLADE_HOOKUP_OBJECT (window, r__seau_menu_uiinfo[5].widget, "quitter4");
	GLADE_HOOKUP_OBJECT (window, menubar_uiinfo[1].widget, "configuration1");
	GLADE_HOOKUP_OBJECT (window, configuration1_menu_uiinfo[0].widget, "liste_ports");
	GLADE_HOOKUP_OBJECT (window, configuration1_menu_uiinfo[1].widget, "ajoute_un_script");
	GLADE_HOOKUP_OBJECT (window, configuration1_menu_uiinfo[2].widget, "liste_des_signatures");
	GLADE_HOOKUP_OBJECT (window, configuration1_menu_uiinfo[3].widget, "extensions");
	GLADE_HOOKUP_OBJECT (window, menubar_uiinfo[2].widget, "aide1");
	GLADE_HOOKUP_OBJECT (window, aide1_menu_uiinfo[0].widget, "___propos");
	GLADE_HOOKUP_OBJECT (window, viewport, "viewport");
	GLADE_HOOKUP_OBJECT (window, hpaned, "hpaned");
	GLADE_HOOKUP_OBJECT (window, scrolledwindow_Liste_Machine, "scrolledwindow_Liste_Machine");
	GLADE_HOOKUP_OBJECT (window, ctree_Liste_Machine, "ctree_Liste_Machine");
	GLADE_HOOKUP_OBJECT (window, viewport_Info, "viewport_Info");
	GLADE_HOOKUP_OBJECT (window, fixed_Info, "fixed_Info");
	GLADE_HOOKUP_OBJECT (window, progressbar_Info, "progressbar_Info");
	GLADE_HOOKUP_OBJECT (window, vseparator_Info, "vseparator_Info");
	GLADE_HOOKUP_OBJECT (window, label_Info, "label_Info");
	GLADE_HOOKUP_OBJECT_NO_REF (window, tooltips, "tooltips");
	return window;
}

static GnomeUIInfo menu_Liste_Machine_uiinfo[] = {
	{
			GNOME_APP_UI_ITEM, "add a subnet",
			NULL,
			(gpointer) on_force_un_sous_r__seau_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/network.png",
		0, (GdkModifierType) 0, NULL},
	{
			GNOME_APP_UI_ITEM, "update host",
			NULL,
			(gpointer) on_re_scanne_la_machine_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/reload.png",
		0, (GdkModifierType) 0, NULL},
	{
			GNOME_APP_UI_ITEM, "Net send",
			NULL,
			(gpointer) on_net_send_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/mail_send.png",
		0, (GdkModifierType) 0, NULL},
	{
			GNOME_APP_UI_ITEM, "add a OS fingerprinting",
			NULL,
			(gpointer) on_ajouter_signature_activate, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, "gnome-stock-book-red",
		0, (GdkModifierType) 0, NULL},
	GNOMEUIINFO_SEPARATOR,
	{
			GNOME_APP_UI_ITEM, "Quit",
			NULL,
			(gpointer) on_quitter_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/exit.png",
		0, (GdkModifierType) 0, NULL},
	GNOMEUIINFO_END
};

GtkWidget *create_menu_Liste_Machine (void)
{


	menu_Liste_Machine = gtk_menu_new ();
	gnome_app_fill_menu (GTK_MENU_SHELL (menu_Liste_Machine), menu_Liste_Machine_uiinfo, NULL, FALSE, 0);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (menu_Liste_Machine, menu_Liste_Machine, "menu_Liste_Machine");
	GLADE_HOOKUP_OBJECT (menu_Liste_Machine, menu_Liste_Machine_uiinfo[0].widget, "force_un_sous_r__seau");
	GLADE_HOOKUP_OBJECT (menu_Liste_Machine, menu_Liste_Machine_uiinfo[1].widget, "re_scanne_la_machine");
	GLADE_HOOKUP_OBJECT (menu_Liste_Machine, menu_Liste_Machine_uiinfo[2].widget, "net_send");
	GLADE_HOOKUP_OBJECT (menu_Liste_Machine, menu_Liste_Machine_uiinfo[3].widget, "ajouter_signature");
	GLADE_HOOKUP_OBJECT (menu_Liste_Machine, menu_Liste_Machine_uiinfo[4].widget, "s__parateur12");
	GLADE_HOOKUP_OBJECT (menu_Liste_Machine, menu_Liste_Machine_uiinfo[5].widget, "quitter1");

	return menu_Liste_Machine;
}

static GnomeUIInfo menu_Port_uiinfo[] = {
	{
			GNOME_APP_UI_ITEM, "Update host",
			NULL,
			(gpointer) on_re_scanne_la_machine_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/reload.png",
		0, (GdkModifierType) 0, NULL},
	GNOMEUIINFO_SEPARATOR,
	{
			GNOME_APP_UI_ITEM, "Telnet",
			NULL,
			(gpointer) on_telnet_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/script.png",
		0, (GdkModifierType) 0, NULL},
	GNOMEUIINFO_SEPARATOR,
	{
			GNOME_APP_UI_ITEM, "Quit",
			NULL,
			(gpointer) on_quitter_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/exit.png",
		0, (GdkModifierType) 0, NULL},
	GNOMEUIINFO_END
};

GtkWidget *create_menu_Port (void)
{


	menu_Port = gtk_menu_new ();
	gnome_app_fill_menu (GTK_MENU_SHELL (menu_Port), menu_Port_uiinfo, NULL, FALSE, 0);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (menu_Port, menu_Port, "menu_Port");
	GLADE_HOOKUP_OBJECT (menu_Port, menu_Port_uiinfo[0].widget, "re_scanne_la_machine");
	GLADE_HOOKUP_OBJECT (menu_Port, menu_Port_uiinfo[1].widget, "s__parateur10");
	GLADE_HOOKUP_OBJECT (menu_Port, menu_Port_uiinfo[2].widget, "telnet");
	GLADE_HOOKUP_OBJECT (menu_Port, menu_Port_uiinfo[3].widget, "s__parateur11");
	GLADE_HOOKUP_OBJECT (menu_Port, menu_Port_uiinfo[4].widget, "quitter3");

	return menu_Port;
}

static GnomeUIInfo menu_Applet_uiinfo[] = {
	{
			GNOME_APP_UI_ITEM, "Show/Hide",
			NULL,
			(gpointer) on_Affiche_Cache_activate, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, "gtk-redo",
		0, (GdkModifierType) 0, NULL},
	GNOMEUIINFO_MENU_ABOUT_ITEM (on____propos_activate, NULL),
	{
			GNOME_APP_UI_ITEM, "Quit",
			NULL,
			(gpointer) on_quitter_activate, NULL, NULL,
			GNOME_APP_PIXMAP_FILENAME, "/usr/share/pixmaps/AutoScan/exit.png",
		0, (GdkModifierType) 0, NULL},
	GNOMEUIINFO_END
};

GtkWidget *create_menu_Applet (void)
{
	menu_Applet = gtk_menu_new ();
	gnome_app_fill_menu (GTK_MENU_SHELL (menu_Applet), menu_Applet_uiinfo, NULL, FALSE, 0);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (menu_Applet, menu_Applet, "menu_Applet");
	GLADE_HOOKUP_OBJECT (menu_Applet, menu_Applet_uiinfo[0].widget, "Affiche_Cache");
	GLADE_HOOKUP_OBJECT (menu_Applet, menu_Applet_uiinfo[1].widget, "___propos");
	GLADE_HOOKUP_OBJECT (menu_Applet, menu_Applet_uiinfo[2].widget, "quitter2");

	return menu_Applet;
}

GtkWidget *create_window_Tester_Smtp (void)
{
	window_Tester_Smtp = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_Tester_Smtp, 355, 363);
	gtk_window_set_title (GTK_WINDOW (window_Tester_Smtp), "SMTP");
	gtk_window_set_position (GTK_WINDOW (window_Tester_Smtp), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window_Tester_Smtp), FALSE);

	fixed_Tester_Smtp = gtk_fixed_new ();
	gtk_widget_show (fixed_Tester_Smtp);
	gtk_container_add (GTK_CONTAINER (window_Tester_Smtp), fixed_Tester_Smtp);

	entry_Tester_Smtp_Emetteur = gtk_entry_new ();
	gtk_widget_show (entry_Tester_Smtp_Emetteur);
	gtk_fixed_put (GTK_FIXED (fixed_Tester_Smtp), entry_Tester_Smtp_Emetteur, 64, 8);
	gtk_widget_set_size_request (entry_Tester_Smtp_Emetteur, 152, 16);
	gtk_entry_set_text (GTK_ENTRY (entry_Tester_Smtp_Emetteur), "User@Tester.org");

	entry_Tester_Smtp_Recepteur = gtk_entry_new ();
	gtk_widget_show (entry_Tester_Smtp_Recepteur);
	gtk_fixed_put (GTK_FIXED (fixed_Tester_Smtp), entry_Tester_Smtp_Recepteur, 64, 32);
	gtk_widget_set_size_request (entry_Tester_Smtp_Recepteur, 152, 16);

	scrolledwindow_Tester_Smtp_Message = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Tester_Smtp_Message);
	gtk_fixed_put (GTK_FIXED (fixed_Tester_Smtp), scrolledwindow_Tester_Smtp_Message, 12, 92);
	gtk_widget_set_size_request (scrolledwindow_Tester_Smtp_Message, 326, 264);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_Tester_Smtp_Message), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	/*textview_Tester_Smtp_Message = gtk_text_view_new ();
	 * gtk_widget_show (textview_Tester_Smtp_Message);
	 * gtk_container_add (GTK_CONTAINER (scrolledwindow_Tester_Smtp_Message), textview_Tester_Smtp_Message); */

	textview_Tester_Smtp_Message = gtk_text_new (NULL, NULL);
	gtk_widget_show (textview_Tester_Smtp_Message);
	gtk_text_set_editable (textview_Tester_Smtp_Message, TRUE);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Tester_Smtp_Message), textview_Tester_Smtp_Message);


	entry_Tester_Smtp_Objet = gtk_entry_new ();
	gtk_widget_show (entry_Tester_Smtp_Objet);
	gtk_fixed_put (GTK_FIXED (fixed_Tester_Smtp), entry_Tester_Smtp_Objet, 64, 57);
	gtk_widget_set_size_request (entry_Tester_Smtp_Objet, 152, 16);

	label_Tester_Smtp_Recepteur = gtk_label_new ("From :");
	gtk_widget_show (label_Tester_Smtp_Recepteur);
	gtk_fixed_put (GTK_FIXED (fixed_Tester_Smtp), label_Tester_Smtp_Recepteur, 8, 32);
	gtk_widget_set_size_request (label_Tester_Smtp_Recepteur, 48, 16);

	label_Tester_Smtp_Emetteur = gtk_label_new ("To : ");
	gtk_widget_show (label_Tester_Smtp_Emetteur);
	gtk_fixed_put (GTK_FIXED (fixed_Tester_Smtp), label_Tester_Smtp_Emetteur, 0, 8);
	gtk_widget_set_size_request (label_Tester_Smtp_Emetteur, 72, 16);

	label_Tester_Smtp_Objet = gtk_label_new ("Subject :");
	gtk_widget_show (label_Tester_Smtp_Objet);
	gtk_fixed_put (GTK_FIXED (fixed_Tester_Smtp), label_Tester_Smtp_Objet, 8, 56);
	gtk_widget_set_size_request (label_Tester_Smtp_Objet, 48, 16);

	button_Tester_Smtp_Annuler = gtk_button_new ();
	gtk_widget_show (button_Tester_Smtp_Annuler);
	gtk_fixed_put (GTK_FIXED (fixed_Tester_Smtp), button_Tester_Smtp_Annuler, 240, 48);
	gtk_widget_set_size_request (button_Tester_Smtp_Annuler, 104, 32);

	alignment_Tester_Smtp_Annuler = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Tester_Smtp_Annuler);
	gtk_container_add (GTK_CONTAINER (button_Tester_Smtp_Annuler), alignment_Tester_Smtp_Annuler);

	hbox_Tester_Smtp_Annuler = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Tester_Smtp_Annuler);
	gtk_container_add (GTK_CONTAINER (alignment_Tester_Smtp_Annuler), hbox_Tester_Smtp_Annuler);

	image_Tester_Smtp_Annuler = create_pixmap (window_Tester_Smtp, "/usr/share/pixmaps/AutoScan/button_cancel.png");
	gtk_widget_show (image_Tester_Smtp_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Tester_Smtp_Annuler), image_Tester_Smtp_Annuler, FALSE, FALSE, 0);

	label_Tester_Smtp_Annuler = gtk_label_new_with_mnemonic ("Cancel");
	gtk_widget_show (label_Tester_Smtp_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Tester_Smtp_Annuler), label_Tester_Smtp_Annuler, FALSE, FALSE, 0);

	button_Tester_Smtp_Envoyer = gtk_button_new ();
	gtk_widget_show (button_Tester_Smtp_Envoyer);
	gtk_fixed_put (GTK_FIXED (fixed_Tester_Smtp), button_Tester_Smtp_Envoyer, 240, 8);
	gtk_widget_set_size_request (button_Tester_Smtp_Envoyer, 104, 32);

	alignment_Tester_Smtp_Envoyer = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Tester_Smtp_Envoyer);
	gtk_container_add (GTK_CONTAINER (button_Tester_Smtp_Envoyer), alignment_Tester_Smtp_Envoyer);

	hbox_Tester_Smtp_Envoyer = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Tester_Smtp_Envoyer);
	gtk_container_add (GTK_CONTAINER (alignment_Tester_Smtp_Envoyer), hbox_Tester_Smtp_Envoyer);

	image_Tester_Smtp_Envoyer = create_pixmap (window_Tester_Smtp, "/usr/share/pixmaps/AutoScan/mail_send.png");
	gtk_widget_show (image_Tester_Smtp_Envoyer);
	gtk_box_pack_start (GTK_BOX (hbox_Tester_Smtp_Envoyer), image_Tester_Smtp_Envoyer, FALSE, FALSE, 0);

	label_Tester_Smtp_Envoyer = gtk_label_new_with_mnemonic ("Send");
	gtk_widget_show (label_Tester_Smtp_Envoyer);
	gtk_box_pack_start (GTK_BOX (hbox_Tester_Smtp_Envoyer), label_Tester_Smtp_Envoyer, FALSE, FALSE, 0);

	g_signal_connect ((gpointer) window_Tester_Smtp, "destroy", G_CALLBACK (on_window_Tester_Smtp_destroy), NULL);
	g_signal_connect ((gpointer) button_Tester_Smtp_Annuler, "clicked", G_CALLBACK (on_button_Tester_Smtp_Annuler_clicked), NULL);
	g_signal_connect ((gpointer) button_Tester_Smtp_Envoyer, "clicked", G_CALLBACK (on_button_Tester_Smtp_Envoyer_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_Tester_Smtp, window_Tester_Smtp, "window_Tester_Smtp");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, fixed_Tester_Smtp, "fixed_Tester_Smtp");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, entry_Tester_Smtp_Emetteur, "entry_Tester_Smtp_Emetteur");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, entry_Tester_Smtp_Recepteur, "entry_Tester_Smtp_Recepteur");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, scrolledwindow_Tester_Smtp_Message, "scrolledwindow_Tester_Smtp_Message");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, textview_Tester_Smtp_Message, "textview_Tester_Smtp_Message");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, entry_Tester_Smtp_Objet, "entry_Tester_Smtp_Objet");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, label_Tester_Smtp_Recepteur, "label_Tester_Smtp_Recepteur");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, label_Tester_Smtp_Emetteur, "label_Tester_Smtp_Emetteur");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, label_Tester_Smtp_Objet, "label_Tester_Smtp_Objet");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, button_Tester_Smtp_Annuler, "button_Tester_Smtp_Annuler");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, alignment_Tester_Smtp_Annuler, "alignment_Tester_Smtp_Annuler");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, hbox_Tester_Smtp_Annuler, "hbox_Tester_Smtp_Annuler");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, image_Tester_Smtp_Annuler, "image_Tester_Smtp_Annuler");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, label_Tester_Smtp_Annuler, "label_Tester_Smtp_Annuler");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, button_Tester_Smtp_Envoyer, "button_Tester_Smtp_Envoyer");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, alignment_Tester_Smtp_Envoyer, "alignment_Tester_Smtp_Envoyer");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, hbox_Tester_Smtp_Envoyer, "hbox_Tester_Smtp_Envoyer");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, image_Tester_Smtp_Envoyer, "image_Tester_Smtp_Envoyer");
	GLADE_HOOKUP_OBJECT (window_Tester_Smtp, label_Tester_Smtp_Envoyer, "label_Tester_Smtp_Envoyer");

	return window_Tester_Smtp;
}

GtkWidget *create_window_NetSend (void)
{
	window_NetSend = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_NetSend, 396, 278);
	gtk_window_set_title (GTK_WINDOW (window_NetSend), "Net Send");
	gtk_window_set_position (GTK_WINDOW (window_NetSend), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window_NetSend), FALSE);
	window_NetSend_icon_pixbuf = create_pixbuf ("/usr/share/pixmaps/AutoScan/AutoScan.png");
	if (window_NetSend_icon_pixbuf) {
		gtk_window_set_icon (GTK_WINDOW (window_NetSend), window_NetSend_icon_pixbuf);
		gdk_pixbuf_unref (window_NetSend_icon_pixbuf);
	}

	fixed_Net_Send = gtk_fixed_new ();
	gtk_widget_show (fixed_Net_Send);
	gtk_container_add (GTK_CONTAINER (window_NetSend), fixed_Net_Send);

	label_Net_Send_Message = gtk_label_new ("Message :");
	gtk_widget_show (label_Net_Send_Message);
	gtk_fixed_put (GTK_FIXED (fixed_Net_Send), label_Net_Send_Message, 16, 8);
	gtk_widget_set_size_request (label_Net_Send_Message, 72, 16);

	scrolledwindow_Net_Send = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Net_Send);
	gtk_fixed_put (GTK_FIXED (fixed_Net_Send), scrolledwindow_Net_Send, 16, 32);
	gtk_widget_set_size_request (scrolledwindow_Net_Send, 376, 184);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_Net_Send), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	textview_Net_Send_Message = gtk_text_new (NULL, NULL);
	gtk_widget_show (textview_Net_Send_Message);
	gtk_text_set_editable (textview_Net_Send_Message, TRUE);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Net_Send), textview_Net_Send_Message);

	button_Net_Send_Annuler = gtk_button_new ();
	gtk_widget_show (button_Net_Send_Annuler);
	gtk_fixed_put (GTK_FIXED (fixed_Net_Send), button_Net_Send_Annuler, 272, 232);
	gtk_widget_set_size_request (button_Net_Send_Annuler, 104, 32);

	alignment_Net_Send_Annuler = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Net_Send_Annuler);
	gtk_container_add (GTK_CONTAINER (button_Net_Send_Annuler), alignment_Net_Send_Annuler);

	hbox_Net_Send_Annuler = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Net_Send_Annuler);
	gtk_container_add (GTK_CONTAINER (alignment_Net_Send_Annuler), hbox_Net_Send_Annuler);

	image_Net_Send_Annuler = create_pixmap (window_NetSend, "/usr/share/pixmaps/AutoScan/button_cancel.png");
	gtk_widget_show (image_Net_Send_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Net_Send_Annuler), image_Net_Send_Annuler, FALSE, FALSE, 0);

	label_Net_Send_Annuler = gtk_label_new_with_mnemonic ("Cancel");
	gtk_widget_show (label_Net_Send_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Net_Send_Annuler), label_Net_Send_Annuler, FALSE, FALSE, 0);

	button_Net_Send_Envoyer = gtk_button_new ();
	gtk_widget_show (button_Net_Send_Envoyer);
	gtk_fixed_put (GTK_FIXED (fixed_Net_Send), button_Net_Send_Envoyer, 152, 232);
	gtk_widget_set_size_request (button_Net_Send_Envoyer, 104, 32);

	alignment_Net_Send_Envoyer = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Net_Send_Envoyer);
	gtk_container_add (GTK_CONTAINER (button_Net_Send_Envoyer), alignment_Net_Send_Envoyer);

	hbox_Net_Send_Envoyer = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Net_Send_Envoyer);
	gtk_container_add (GTK_CONTAINER (alignment_Net_Send_Envoyer), hbox_Net_Send_Envoyer);

	image_Net_Send_Envoyer = create_pixmap (window_NetSend, "/usr/share/pixmaps/AutoScan/mail_send.png");
	gtk_widget_show (image_Net_Send_Envoyer);
	gtk_box_pack_start (GTK_BOX (hbox_Net_Send_Envoyer), image_Net_Send_Envoyer, FALSE, FALSE, 0);

	label_Net_Send_Envoyer = gtk_label_new_with_mnemonic ("Send");
	gtk_widget_show (label_Net_Send_Envoyer);
	gtk_box_pack_start (GTK_BOX (hbox_Net_Send_Envoyer), label_Net_Send_Envoyer, FALSE, FALSE, 0);

	g_signal_connect ((gpointer) window_NetSend, "destroy", G_CALLBACK (on_window_NetSend_destroy), NULL);
	g_signal_connect ((gpointer) button_Net_Send_Annuler, "clicked", G_CALLBACK (on_button_Net_Send_Annuler_clicked), NULL);
	g_signal_connect ((gpointer) button_Net_Send_Envoyer, "clicked", G_CALLBACK (on_button_Net_Send_Envoyer_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_NetSend, window_NetSend, "window_NetSend");
	GLADE_HOOKUP_OBJECT (window_NetSend, fixed_Net_Send, "fixed_Net_Send");
	GLADE_HOOKUP_OBJECT (window_NetSend, label_Net_Send_Message, "label_Net_Send_Message");
	GLADE_HOOKUP_OBJECT (window_NetSend, scrolledwindow_Net_Send, "scrolledwindow_Net_Send");
	GLADE_HOOKUP_OBJECT (window_NetSend, textview_Net_Send_Message, "textview_Net_Send_Message");
	GLADE_HOOKUP_OBJECT (window_NetSend, button_Net_Send_Annuler, "button_Net_Send_Annuler");
	GLADE_HOOKUP_OBJECT (window_NetSend, alignment_Net_Send_Annuler, "alignment_Net_Send_Annuler");
	GLADE_HOOKUP_OBJECT (window_NetSend, hbox_Net_Send_Annuler, "hbox_Net_Send_Annuler");
	GLADE_HOOKUP_OBJECT (window_NetSend, image_Net_Send_Annuler, "image_Net_Send_Annuler");
	GLADE_HOOKUP_OBJECT (window_NetSend, label_Net_Send_Annuler, "label_Net_Send_Annuler");
	GLADE_HOOKUP_OBJECT (window_NetSend, button_Net_Send_Envoyer, "button_Net_Send_Envoyer");
	GLADE_HOOKUP_OBJECT (window_NetSend, alignment_Net_Send_Envoyer, "alignment_Net_Send_Envoyer");
	GLADE_HOOKUP_OBJECT (window_NetSend, hbox_Net_Send_Envoyer, "hbox_Net_Send_Envoyer");
	GLADE_HOOKUP_OBJECT (window_NetSend, image_Net_Send_Envoyer, "image_Net_Send_Envoyer");
	GLADE_HOOKUP_OBJECT (window_NetSend, label_Net_Send_Envoyer, "label_Net_Send_Envoyer");

	return window_NetSend;
}

GtkWidget *create_window_Pop3 (void)
{
	window_Pop3 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_Pop3, 562, 466);
	gtk_window_set_title (GTK_WINDOW (window_Pop3), "Pop3");
	gtk_window_set_position (GTK_WINDOW (window_Pop3), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window_Pop3), FALSE);

	fixed_Pop3 = gtk_fixed_new ();
	gtk_widget_show (fixed_Pop3);
	gtk_container_add (GTK_CONTAINER (window_Pop3), fixed_Pop3);

	entry_Pop3_User = gtk_entry_new ();
	gtk_widget_show (entry_Pop3_User);
	gtk_fixed_put (GTK_FIXED (fixed_Pop3), entry_Pop3_User, 96, 16);
	gtk_widget_set_size_request (entry_Pop3_User, 152, 17);

	entry_Pop3_Password = gtk_entry_new ();
	gtk_widget_show (entry_Pop3_Password);
	gtk_fixed_put (GTK_FIXED (fixed_Pop3), entry_Pop3_Password, 96, 48);
	gtk_widget_set_size_request (entry_Pop3_Password, 152, 16);

	scrolledwindow_Pop3 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Pop3);
	gtk_fixed_put (GTK_FIXED (fixed_Pop3), scrolledwindow_Pop3, 16, 94);
	gtk_widget_set_size_request (scrolledwindow_Pop3, 528, 370);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_Pop3), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	textview_Pop3 = gtk_text_new (NULL, NULL);
	gtk_widget_show (textview_Pop3);
	gtk_text_set_editable (textview_Pop3, FALSE);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Pop3), textview_Pop3);

	label_Pop3_User = gtk_label_new ("User : ");
	gtk_widget_show (label_Pop3_User);
	gtk_fixed_put (GTK_FIXED (fixed_Pop3), label_Pop3_User, 32, 16);
	gtk_widget_set_size_request (label_Pop3_User, 48, 16);

	label_Pop3_Password = gtk_label_new ("Password :");
	gtk_widget_show (label_Pop3_Password);
	gtk_fixed_put (GTK_FIXED (fixed_Pop3), label_Pop3_Password, 8, 48);
	gtk_widget_set_size_request (label_Pop3_Password, 72, 16);

	button_Pop3_Quitter = gtk_button_new ();
	gtk_widget_show (button_Pop3_Quitter);
	gtk_fixed_put (GTK_FIXED (fixed_Pop3), button_Pop3_Quitter, 432, 48);
	gtk_widget_set_size_request (button_Pop3_Quitter, 104, 32);

	alignment_Pop3_Quitter = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Pop3_Quitter);
	gtk_container_add (GTK_CONTAINER (button_Pop3_Quitter), alignment_Pop3_Quitter);

	hbox_Pop3_Quitter = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Pop3_Quitter);
	gtk_container_add (GTK_CONTAINER (alignment_Pop3_Quitter), hbox_Pop3_Quitter);

	image_Pop3_Quitter = create_pixmap (window_Pop3, "/usr/share/pixmaps/AutoScan/button_cancel.png");
	gtk_widget_show (image_Pop3_Quitter);
	gtk_box_pack_start (GTK_BOX (hbox_Pop3_Quitter), image_Pop3_Quitter, FALSE, FALSE, 0);

	label_Pop3_Quitter = gtk_label_new_with_mnemonic ("Cancel");
	gtk_widget_show (label_Pop3_Quitter);
	gtk_box_pack_start (GTK_BOX (hbox_Pop3_Quitter), label_Pop3_Quitter, FALSE, FALSE, 0);

	button_Pop3_Rafraichir = gtk_button_new ();
	gtk_widget_show (button_Pop3_Rafraichir);
	gtk_fixed_put (GTK_FIXED (fixed_Pop3), button_Pop3_Rafraichir, 432, 8);
	gtk_widget_set_size_request (button_Pop3_Rafraichir, 104, 32);

	alignment_Pop3_Rafraichir = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Pop3_Rafraichir);
	gtk_container_add (GTK_CONTAINER (button_Pop3_Rafraichir), alignment_Pop3_Rafraichir);

	hbox_Pop3_Rafraichir = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Pop3_Rafraichir);
	gtk_container_add (GTK_CONTAINER (alignment_Pop3_Rafraichir), hbox_Pop3_Rafraichir);

	image_Pop3_Rafraichir = create_pixmap (window_Pop3, "/usr/share/pixmaps/AutoScan/button_ok.png");
	gtk_widget_show (image_Pop3_Rafraichir);
	gtk_box_pack_start (GTK_BOX (hbox_Pop3_Rafraichir), image_Pop3_Rafraichir, FALSE, FALSE, 0);

	label_Pop3_Rafraichir = gtk_label_new_with_mnemonic ("Ok");
	gtk_widget_show (label_Pop3_Rafraichir);
	gtk_box_pack_start (GTK_BOX (hbox_Pop3_Rafraichir), label_Pop3_Rafraichir, FALSE, FALSE, 0);

	g_signal_connect ((gpointer) window_Pop3, "destroy", G_CALLBACK (on_window_Pop3_destroy), NULL);
	g_signal_connect ((gpointer) button_Pop3_Quitter, "clicked", G_CALLBACK (on_button_Pop3_Quitter_clicked), NULL);
	g_signal_connect ((gpointer) button_Pop3_Rafraichir, "clicked", G_CALLBACK (on_button_Pop3_Rafraichir_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_Pop3, window_Pop3, "window_Pop3");
	GLADE_HOOKUP_OBJECT (window_Pop3, fixed_Pop3, "fixed_Pop3");
	GLADE_HOOKUP_OBJECT (window_Pop3, entry_Pop3_User, "entry_Pop3_User");
	GLADE_HOOKUP_OBJECT (window_Pop3, entry_Pop3_Password, "entry_Pop3_Password");
	GLADE_HOOKUP_OBJECT (window_Pop3, scrolledwindow_Pop3, "scrolledwindow_Pop3");
	GLADE_HOOKUP_OBJECT (window_Pop3, textview_Pop3, "textview_Pop3");
	GLADE_HOOKUP_OBJECT (window_Pop3, label_Pop3_User, "label_Pop3_User");
	GLADE_HOOKUP_OBJECT (window_Pop3, label_Pop3_Password, "label_Pop3_Password");
	GLADE_HOOKUP_OBJECT (window_Pop3, button_Pop3_Quitter, "button_Pop3_Quitter");
	GLADE_HOOKUP_OBJECT (window_Pop3, alignment_Pop3_Quitter, "alignment_Pop3_Quitter");
	GLADE_HOOKUP_OBJECT (window_Pop3, hbox_Pop3_Quitter, "hbox_Pop3_Quitter");
	GLADE_HOOKUP_OBJECT (window_Pop3, image_Pop3_Quitter, "image_Pop3_Quitter");
	GLADE_HOOKUP_OBJECT (window_Pop3, label_Pop3_Quitter, "label_Pop3_Quitter");
	GLADE_HOOKUP_OBJECT (window_Pop3, button_Pop3_Rafraichir, "button_Pop3_Rafraichir");
	GLADE_HOOKUP_OBJECT (window_Pop3, alignment_Pop3_Rafraichir, "alignment_Pop3_Rafraichir");
	GLADE_HOOKUP_OBJECT (window_Pop3, hbox_Pop3_Rafraichir, "hbox_Pop3_Rafraichir");
	GLADE_HOOKUP_OBJECT (window_Pop3, image_Pop3_Rafraichir, "image_Pop3_Rafraichir");
	GLADE_HOOKUP_OBJECT (window_Pop3, label_Pop3_Rafraichir, "label_Pop3_Rafraichir");

	return window_Pop3;
}

GtkWidget *create_window_Liste_Port (void)
{
	GList    *combo_Liste_Port_Machine_Type_items = NULL;

	tooltips = gtk_tooltips_new ();

	window_Liste_Port = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_Liste_Port, 633, 609);
	gtk_window_set_title (GTK_WINDOW (window_Liste_Port), _("Ports list"));
	gtk_window_set_position (GTK_WINDOW (window_Liste_Port), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window_Liste_Port), FALSE);
	window_Liste_Port_icon_pixbuf = create_pixbuf ("/usr/share/pixmaps/AutoScan/AutoScan.png");
	if (window_Liste_Port_icon_pixbuf) {
		gtk_window_set_icon (GTK_WINDOW (window_Liste_Port), window_Liste_Port_icon_pixbuf);
		gdk_pixbuf_unref (window_Liste_Port_icon_pixbuf);
	}

	fixed_Liste_Port = gtk_fixed_new ();
	gtk_widget_show (fixed_Liste_Port);
	gtk_container_add (GTK_CONTAINER (window_Liste_Port), fixed_Liste_Port);

	scrolledwindow_Liste_Port = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Liste_Port);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port), scrolledwindow_Liste_Port, 24, 24);
	gtk_widget_set_size_request (scrolledwindow_Liste_Port, 592, 256);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_Liste_Port), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	ctree_Liste_Port = gtk_ctree_new (3, 0);
	gtk_widget_show (ctree_Liste_Port);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Liste_Port), ctree_Liste_Port);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Liste_Port), 0, 52);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Liste_Port), 1, 132);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Liste_Port), 2, 80);
	gtk_clist_column_titles_show (GTK_CLIST (ctree_Liste_Port));

	label_Liste_Port_Port = gtk_label_new (_("Port"));
	gtk_widget_show (label_Liste_Port_Port);
	gtk_clist_set_column_widget (GTK_CLIST (ctree_Liste_Port), 0, label_Liste_Port_Port);

	label_Liste_Port_Service = gtk_label_new (_("Service"));
	gtk_widget_show (label_Liste_Port_Service);
	gtk_clist_set_column_widget (GTK_CLIST (ctree_Liste_Port), 1, label_Liste_Port_Service);

	label_Liste_Port_Description = gtk_label_new (_("Description"));
	gtk_widget_show (label_Liste_Port_Description);
	gtk_clist_set_column_widget (GTK_CLIST (ctree_Liste_Port), 2, label_Liste_Port_Description);

	frame_Liste_Port_Modif = gtk_frame_new (NULL);
	gtk_widget_show (frame_Liste_Port_Modif);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port), frame_Liste_Port_Modif, 24, 288);
	gtk_widget_set_size_request (frame_Liste_Port_Modif, 487, 317);

	fixed_Liste_Port_Modif = gtk_fixed_new ();
	gtk_widget_show (fixed_Liste_Port_Modif);
	gtk_container_add (GTK_CONTAINER (frame_Liste_Port_Modif), fixed_Liste_Port_Modif);

	entry_Liste_Port_Service = gtk_entry_new ();
	gtk_widget_show (entry_Liste_Port_Service);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), entry_Liste_Port_Service, 88, 40);
	gtk_widget_set_size_request (entry_Liste_Port_Service, 224, 21);

	entry_Liste_Port_Description = gtk_entry_new ();
	gtk_widget_show (entry_Liste_Port_Description);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), entry_Liste_Port_Description, 88, 72);
	gtk_widget_set_size_request (entry_Liste_Port_Description, 336, 21);

	entry_Liste_Port_Programme = gtk_entry_new ();
	gtk_widget_show (entry_Liste_Port_Programme);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), entry_Liste_Port_Programme, 88, 104);
	gtk_widget_set_size_request (entry_Liste_Port_Programme, 336, 21);
	gtk_tooltips_set_tip (tooltips, entry_Liste_Port_Programme, _("Parameters : _IP_ : IP adress, _PORT_: Port, _MAC_ : Adress Mac"), NULL);
	gtk_entry_set_text (GTK_ENTRY (entry_Liste_Port_Programme), _("galeon http://_IP_:_PORT_"));

	spinbutton_Liste_Port_Port_adj = gtk_adjustment_new (1, 0, 65535, 1, 10, 10);
	spinbutton_Liste_Port_Port = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_Liste_Port_Port_adj), 1, 0);
	gtk_widget_show (spinbutton_Liste_Port_Port);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), spinbutton_Liste_Port_Port, 88, 8);
	gtk_widget_set_size_request (spinbutton_Liste_Port_Port, 69, 21);

	combo_Liste_Port_Machine_Type = gtk_combo_new ();
	g_object_set_data (G_OBJECT (GTK_COMBO (combo_Liste_Port_Machine_Type)->popwin), "GladeParentKey", combo_Liste_Port_Machine_Type);
	gtk_widget_show (combo_Liste_Port_Machine_Type);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), combo_Liste_Port_Machine_Type, 214, 8);
	gtk_widget_set_size_request (combo_Liste_Port_Machine_Type, 88, 25);
	combo_Liste_Port_Machine_Type_items = g_list_append (combo_Liste_Port_Machine_Type_items, (gpointer) _("Server"));
	combo_Liste_Port_Machine_Type_items = g_list_append (combo_Liste_Port_Machine_Type_items, (gpointer) _("Other"));
	gtk_combo_set_popdown_strings (GTK_COMBO (combo_Liste_Port_Machine_Type), combo_Liste_Port_Machine_Type_items);
	g_list_free (combo_Liste_Port_Machine_Type_items);

	combo_entry_Liste_Port_Machine_Type = GTK_COMBO (combo_Liste_Port_Machine_Type)->entry;
	gtk_widget_show (combo_entry_Liste_Port_Machine_Type);

	label_Liste_Port_Machine_Type = gtk_label_new (_("Type : "));
	gtk_widget_show (label_Liste_Port_Machine_Type);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), label_Liste_Port_Machine_Type, 168, 13);
	gtk_widget_set_size_request (label_Liste_Port_Machine_Type, 51, 16);

	colorpicker_Liste_Port = gnome_color_picker_new ();
	gtk_widget_show (colorpicker_Liste_Port);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), colorpicker_Liste_Port, 200, 136);
	gtk_widget_set_size_request (colorpicker_Liste_Port, 40, 32);

	label_Liste_Port_Port = gtk_label_new (_("Port :"));
	gtk_widget_show (label_Liste_Port_Port);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), label_Liste_Port_Port, 40, 16);
	gtk_widget_set_size_request (label_Liste_Port_Port, 40, 16);

	label_Liste_Port_Service = gtk_label_new (_("Service :"));
	gtk_widget_show (label_Liste_Port_Service);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), label_Liste_Port_Service, 24, 40);
	gtk_widget_set_size_request (label_Liste_Port_Service, 56, 16);

	label_Liste_Port_Programme = gtk_label_new (_("Program :"));
	gtk_widget_show (label_Liste_Port_Programme);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), label_Liste_Port_Programme, 24, 104);
	gtk_widget_set_size_request (label_Liste_Port_Programme, 64, 16);

	label_Liste_Port_Description = gtk_label_new (_("Description :"));
	gtk_widget_show (label_Liste_Port_Description);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), label_Liste_Port_Description, 8, 72);
	gtk_widget_set_size_request (label_Liste_Port_Description, 80, 16);

	frame_Liste_Port_Etat = gtk_frame_new (NULL);
	//gtk_widget_show (frame_Liste_Port_Etat);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), frame_Liste_Port_Etat, 16, 168);
	gtk_widget_set_size_request (frame_Liste_Port_Etat, 310, 126);

	fixed_Liste_Port_Etat = gtk_fixed_new ();
	gtk_widget_show (fixed_Liste_Port_Etat);
	gtk_container_add (GTK_CONTAINER (frame_Liste_Port_Etat), fixed_Liste_Port_Etat);

	frame_Liste_Port_Etat_Action = gtk_frame_new (NULL);
	gtk_widget_show (frame_Liste_Port_Etat_Action);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Etat), frame_Liste_Port_Etat_Action, 24, 24);
	gtk_widget_set_size_request (frame_Liste_Port_Etat_Action, 274, 70);

	fixed_Liste_Port_Etat_Action = gtk_fixed_new ();
	gtk_widget_show (fixed_Liste_Port_Etat_Action);
	gtk_container_add (GTK_CONTAINER (frame_Liste_Port_Etat_Action), fixed_Liste_Port_Etat_Action);

	entry_Liste_Port_Etat_Action_Email = gtk_entry_new ();
	gtk_widget_show (entry_Liste_Port_Etat_Action_Email);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Etat_Action), entry_Liste_Port_Etat_Action_Email, 75, 0);
	gtk_widget_set_size_request (entry_Liste_Port_Etat_Action_Email, 188, 21);

	checkbutton_Liste_Port_Etat_Action_Message = gtk_check_button_new_with_mnemonic (_("Message"));
	gtk_widget_show (checkbutton_Liste_Port_Etat_Action_Message);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Etat_Action), checkbutton_Liste_Port_Etat_Action_Message, 16, 24);
	gtk_widget_set_size_request (checkbutton_Liste_Port_Etat_Action_Message, 98, 23);

	checkbutton_Liste_Port_Etat_Action_Email = gtk_check_button_new_with_mnemonic (_("Email"));
	gtk_widget_show (checkbutton_Liste_Port_Etat_Action_Email);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Etat_Action), checkbutton_Liste_Port_Etat_Action_Email, 16, 0);
	gtk_widget_set_size_request (checkbutton_Liste_Port_Etat_Action_Email, 136, 24);

	label_Liste_Port_Etat = gtk_label_new (_("Actions"));
	gtk_widget_show (label_Liste_Port_Etat);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Liste_Port_Etat_Action), label_Liste_Port_Etat);

	checkbutton_Liste_Port_Etat_Close = gtk_check_button_new_with_mnemonic (_("Poll even when it is closed"));
	gtk_widget_show (checkbutton_Liste_Port_Etat_Close);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Etat), checkbutton_Liste_Port_Etat_Close, 8, 0);
	gtk_widget_set_size_request (checkbutton_Liste_Port_Etat_Close, 168, 24);

	checkbutton_Liste_Port_Etat = gtk_check_button_new_with_mnemonic (_("Detect the state change of the port"));
	gtk_widget_show (checkbutton_Liste_Port_Etat);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Liste_Port_Etat), checkbutton_Liste_Port_Etat);

	label_Liste_Port_Open = gtk_label_new (_("Open"));
	gtk_widget_show (label_Liste_Port_Open);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), label_Liste_Port_Open, 352, 256);
	gtk_widget_set_size_request (label_Liste_Port_Open, 41, 16);

	label_Liste_Port_Close = gtk_label_new (_("Close"));
	gtk_widget_show (label_Liste_Port_Close);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), label_Liste_Port_Close, 424, 256);
	gtk_widget_set_size_request (label_Liste_Port_Close, 41, 16);

	checkbutton_Liste_Port_Marque = gtk_check_button_new_with_mnemonic (_("To mark if the port is open"));
	gtk_widget_show (checkbutton_Liste_Port_Marque);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), checkbutton_Liste_Port_Marque, 22, 140);
	gtk_widget_set_size_request (checkbutton_Liste_Port_Marque, 168, 24);

	iconentry_Liste_Port_Ouvert = gnome_icon_entry_new (NULL, NULL);
	gtk_widget_show (iconentry_Liste_Port_Ouvert);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), iconentry_Liste_Port_Ouvert, 344, 184);
	gtk_widget_set_size_request (iconentry_Liste_Port_Ouvert, 60, 60);

	iconentry_Liste_Port_Fermer = gnome_icon_entry_new (NULL, NULL);
	gtk_widget_show (iconentry_Liste_Port_Fermer);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port_Modif), iconentry_Liste_Port_Fermer, 416, 184);
	gtk_widget_set_size_request (iconentry_Liste_Port_Fermer, 60, 60);

	label_Liste_Port_Modif = gtk_label_new (_("Change, add, delete port info"));
	gtk_widget_show (label_Liste_Port_Modif);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Liste_Port_Modif), label_Liste_Port_Modif);

	button_Liste_Port_Ajouter = gtk_button_new ();
	gtk_widget_show (button_Liste_Port_Ajouter);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port), button_Liste_Port_Ajouter, 520, 328);
	gtk_widget_set_size_request (button_Liste_Port_Ajouter, 104, 36);

	alignment_Liste_Port_Ajouter = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Liste_Port_Ajouter);
	gtk_container_add (GTK_CONTAINER (button_Liste_Port_Ajouter), alignment_Liste_Port_Ajouter);

	hbox_Liste_Port_Ajouter = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Liste_Port_Ajouter);
	gtk_container_add (GTK_CONTAINER (alignment_Liste_Port_Ajouter), hbox_Liste_Port_Ajouter);

	image_Liste_Port_Ajouter = create_pixmap (window_Liste_Port, "/usr/share/pixmaps/AutoScan/filenew.png");
	gtk_widget_show (image_Liste_Port_Ajouter);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Port_Ajouter), image_Liste_Port_Ajouter, FALSE, FALSE, 0);

	label_Liste_Port_Ajouter = gtk_label_new_with_mnemonic (_("Add"));
	gtk_widget_show (label_Liste_Port_Ajouter);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Port_Ajouter), label_Liste_Port_Ajouter, FALSE, FALSE, 0);

	button_Liste_Port_Enlever = gtk_button_new ();
	gtk_widget_show (button_Liste_Port_Enlever);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port), button_Liste_Port_Enlever, 520, 368);
	gtk_widget_set_size_request (button_Liste_Port_Enlever, 104, 36);

	alignment_Liste_Port_Enlever = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Liste_Port_Enlever);
	gtk_container_add (GTK_CONTAINER (button_Liste_Port_Enlever), alignment_Liste_Port_Enlever);

	hbox_Liste_Port_Enlever = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Liste_Port_Enlever);
	gtk_container_add (GTK_CONTAINER (alignment_Liste_Port_Enlever), hbox_Liste_Port_Enlever);

	image_Liste_Port_Enlever = create_pixmap (window_Liste_Port, "/usr/share/pixmaps/AutoScan/editcut.png");
	gtk_widget_show (image_Liste_Port_Enlever);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Port_Enlever), image_Liste_Port_Enlever, FALSE, FALSE, 0);

	label_Liste_Port_Enlever = gtk_label_new_with_mnemonic (_("Delete"));
	gtk_widget_show (label_Liste_Port_Enlever);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Port_Enlever), label_Liste_Port_Enlever, FALSE, FALSE, 0);

	button_Liste_Port_Annuler = gtk_button_new ();
	gtk_widget_show (button_Liste_Port_Annuler);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port), button_Liste_Port_Annuler, 520, 464);
	gtk_widget_set_size_request (button_Liste_Port_Annuler, 104, 36);

	alignment_Liste_Port_Annuler = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Liste_Port_Annuler);
	gtk_container_add (GTK_CONTAINER (button_Liste_Port_Annuler), alignment_Liste_Port_Annuler);

	hbox_Liste_Port_Annuler = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Liste_Port_Annuler);
	gtk_container_add (GTK_CONTAINER (alignment_Liste_Port_Annuler), hbox_Liste_Port_Annuler);

	image_Liste_Port_Annuler = create_pixmap (window_Liste_Port, "/usr/share/pixmaps/AutoScan/button_cancel.png");
	gtk_widget_show (image_Liste_Port_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Port_Annuler), image_Liste_Port_Annuler, FALSE, FALSE, 0);

	label_Liste_Port_Annuler = gtk_label_new_with_mnemonic (_("Cancel"));
	gtk_widget_show (label_Liste_Port_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Port_Annuler), label_Liste_Port_Annuler, FALSE, FALSE, 0);

	button_Liste_Port_Ok = gtk_button_new ();
	gtk_widget_show (button_Liste_Port_Ok);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port), button_Liste_Port_Ok, 520, 424);
	gtk_widget_set_size_request (button_Liste_Port_Ok, 104, 36);

	alignment_Liste_Port_Enregistrer = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Liste_Port_Enregistrer);
	gtk_container_add (GTK_CONTAINER (button_Liste_Port_Ok), alignment_Liste_Port_Enregistrer);

	hbox_Liste_Port_Enregistrer = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Liste_Port_Enregistrer);
	gtk_container_add (GTK_CONTAINER (alignment_Liste_Port_Enregistrer), hbox_Liste_Port_Enregistrer);

	image_Liste_Port_Enregistrer = create_pixmap (window_Liste_Port, "/usr/share/pixmaps/AutoScan/button_ok.png");
	gtk_widget_show (image_Liste_Port_Enregistrer);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Port_Enregistrer), image_Liste_Port_Enregistrer, FALSE, FALSE, 0);

	label_Liste_Port_Enregistrer = gtk_label_new_with_mnemonic (_("Ok"));
	gtk_widget_show (label_Liste_Port_Enregistrer);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Port_Enregistrer), label_Liste_Port_Enregistrer, FALSE, FALSE, 0);

	button_Liste_Port_Sauvegarder = gtk_button_new ();
	gtk_widget_show (button_Liste_Port_Sauvegarder);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Port), button_Liste_Port_Sauvegarder, 520, 528);
	gtk_widget_set_size_request (button_Liste_Port_Sauvegarder, 104, 36);

	alignment_Liste_Port_Valider = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Liste_Port_Valider);
	gtk_container_add (GTK_CONTAINER (button_Liste_Port_Sauvegarder), alignment_Liste_Port_Valider);

	hbox_Liste_Port_Valider = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Liste_Port_Valider);
	gtk_container_add (GTK_CONTAINER (alignment_Liste_Port_Valider), hbox_Liste_Port_Valider);

	image_Liste_Port_Valider = create_pixmap (window_Liste_Port, "/usr/share/pixmaps/AutoScan/filesave.png");
	gtk_widget_show (image_Liste_Port_Valider);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Port_Valider), image_Liste_Port_Valider, FALSE, FALSE, 0);

	label_Liste_Port_Valider = gtk_label_new_with_mnemonic (_("Save"));
	gtk_widget_show (label_Liste_Port_Valider);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Port_Valider), label_Liste_Port_Valider, FALSE, FALSE, 0);

	g_signal_connect ((gpointer) window_Liste_Port, "destroy", G_CALLBACK (on_window_Liste_Port_destroy), NULL);
	g_signal_connect ((gpointer) ctree_Liste_Port, "tree_select_row", G_CALLBACK (on_ctree_Liste_Port_tree_select_row), NULL);
	g_signal_connect ((gpointer) entry_Liste_Port_Service, "changed", G_CALLBACK (Liste_Port_changed), NULL);
	g_signal_connect ((gpointer) entry_Liste_Port_Description, "changed", G_CALLBACK (Liste_Port_changed), NULL);
	g_signal_connect ((gpointer) entry_Liste_Port_Programme, "changed", G_CALLBACK (Liste_Port_changed), NULL);
	g_signal_connect ((gpointer) spinbutton_Liste_Port_Port, "changed", G_CALLBACK (Liste_Port_changed), NULL);
	g_signal_connect ((gpointer) combo_entry_Liste_Port_Machine_Type, "changed", G_CALLBACK (Liste_Port_changed), NULL);
	g_signal_connect ((gpointer) colorpicker_Liste_Port, "color_set", G_CALLBACK (Liste_Port_changed), NULL);
	g_signal_connect ((gpointer) entry_Liste_Port_Etat_Action_Email, "changed", G_CALLBACK (Liste_Port_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Port_Etat_Action_Message, "clicked", G_CALLBACK (Liste_Port_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Port_Etat_Action_Email, "clicked", G_CALLBACK (Liste_Port_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Port_Etat_Action_Email, "clicked", G_CALLBACK (on_checkbutton_Liste_Port_Etat_Action_Email_clicked), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Port_Etat_Close, "clicked", G_CALLBACK (Liste_Port_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Port_Etat, "clicked", G_CALLBACK (Liste_Port_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Port_Etat, "clicked", G_CALLBACK (on_checkbutton_Liste_Port_Etat_clicked), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Port_Marque, "clicked", G_CALLBACK (Liste_Port_changed), NULL);
	g_signal_connect ((gpointer) iconentry_Liste_Port_Ouvert, "changed", G_CALLBACK (Liste_Port_changed), NULL);
	g_signal_connect ((gpointer) iconentry_Liste_Port_Fermer, "changed", G_CALLBACK (Liste_Port_changed), NULL);
	g_signal_connect ((gpointer) button_Liste_Port_Ajouter, "clicked", G_CALLBACK (on_button_Liste_Port_Ajouter_clicked), NULL);
	g_signal_connect ((gpointer) button_Liste_Port_Enlever, "clicked", G_CALLBACK (on_button_Liste_Port_Enlever_clicked), NULL);
	g_signal_connect ((gpointer) button_Liste_Port_Annuler, "clicked", G_CALLBACK (on_button_Liste_Port_Annuler_clicked), NULL);
	g_signal_connect ((gpointer) button_Liste_Port_Ok, "clicked", G_CALLBACK (on_button_Liste_Port_Ok_clicked), NULL);
	g_signal_connect ((gpointer) button_Liste_Port_Sauvegarder, "clicked", G_CALLBACK (on_button_Liste_Port_Sauvegarder_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_Liste_Port, window_Liste_Port, "window_Liste_Port");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, fixed_Liste_Port, "fixed_Liste_Port");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, scrolledwindow_Liste_Port, "scrolledwindow_Liste_Port");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, ctree_Liste_Port, "ctree_Liste_Port");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Port, "label_Liste_Port_Port");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Service, "label_Liste_Port_Service");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Description, "label_Liste_Port_Description");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, frame_Liste_Port_Modif, "frame_Liste_Port_Modif");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, fixed_Liste_Port_Modif, "fixed_Liste_Port_Modif");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, entry_Liste_Port_Service, "entry_Liste_Port_Service");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, entry_Liste_Port_Description, "entry_Liste_Port_Description");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, entry_Liste_Port_Programme, "entry_Liste_Port_Programme");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, spinbutton_Liste_Port_Port, "spinbutton_Liste_Port_Port");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, combo_Liste_Port_Machine_Type, "combo_Liste_Port_Machine_Type");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, combo_entry_Liste_Port_Machine_Type, "combo_entry_Liste_Port_Machine_Type");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Machine_Type, "label_Liste_Port_Machine_Type");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, colorpicker_Liste_Port, "colorpicker_Liste_Port");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Port, "label_Liste_Port_Port");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Service, "label_Liste_Port_Service");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Programme, "label_Liste_Port_Programme");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Description, "label_Liste_Port_Description");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, frame_Liste_Port_Etat, "frame_Liste_Port_Etat");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, fixed_Liste_Port_Etat, "fixed_Liste_Port_Etat");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, frame_Liste_Port_Etat_Action, "frame_Liste_Port_Etat_Action");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, fixed_Liste_Port_Etat_Action, "fixed_Liste_Port_Etat_Action");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, entry_Liste_Port_Etat_Action_Email, "entry_Liste_Port_Etat_Action_Email");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, checkbutton_Liste_Port_Etat_Action_Message, "checkbutton_Liste_Port_Etat_Action_Message");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, checkbutton_Liste_Port_Etat_Action_Email, "checkbutton_Liste_Port_Etat_Action_Email");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Etat, "label_Liste_Port_Etat");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, checkbutton_Liste_Port_Etat_Close, "checkbutton_Liste_Port_Etat_Close");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, checkbutton_Liste_Port_Etat, "checkbutton_Liste_Port_Etat");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Open, "label_Liste_Port_Open");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Close, "label_Liste_Port_Close");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, checkbutton_Liste_Port_Marque, "checkbutton_Liste_Port_Marque");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, iconentry_Liste_Port_Ouvert, "iconentry_Liste_Port_Ouvert");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, iconentry_Liste_Port_Fermer, "iconentry_Liste_Port_Fermer");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Modif, "label_Liste_Port_Modif");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, button_Liste_Port_Ajouter, "button_Liste_Port_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, alignment_Liste_Port_Ajouter, "alignment_Liste_Port_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, hbox_Liste_Port_Ajouter, "hbox_Liste_Port_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, image_Liste_Port_Ajouter, "image_Liste_Port_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Ajouter, "label_Liste_Port_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, button_Liste_Port_Enlever, "button_Liste_Port_Enlever");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, alignment_Liste_Port_Enlever, "alignment_Liste_Port_Enlever");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, hbox_Liste_Port_Enlever, "hbox_Liste_Port_Enlever");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, image_Liste_Port_Enlever, "image_Liste_Port_Enlever");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Enlever, "label_Liste_Port_Enlever");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, button_Liste_Port_Annuler, "button_Liste_Port_Annuler");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, alignment_Liste_Port_Annuler, "alignment_Liste_Port_Annuler");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, hbox_Liste_Port_Annuler, "hbox_Liste_Port_Annuler");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, image_Liste_Port_Annuler, "image_Liste_Port_Annuler");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Annuler, "label_Liste_Port_Annuler");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, button_Liste_Port_Ok, "button_Liste_Port_Ok");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, alignment_Liste_Port_Enregistrer, "alignment_Liste_Port_Enregistrer");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, hbox_Liste_Port_Enregistrer, "hbox_Liste_Port_Enregistrer");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, image_Liste_Port_Enregistrer, "image_Liste_Port_Enregistrer");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Enregistrer, "label_Liste_Port_Enregistrer");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, button_Liste_Port_Sauvegarder, "button_Liste_Port_Sauvegarder");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, alignment_Liste_Port_Valider, "alignment_Liste_Port_Valider");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, hbox_Liste_Port_Valider, "hbox_Liste_Port_Valider");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, image_Liste_Port_Valider, "image_Liste_Port_Valider");
	GLADE_HOOKUP_OBJECT (window_Liste_Port, label_Liste_Port_Valider, "label_Liste_Port_Valider");
	GLADE_HOOKUP_OBJECT_NO_REF (window_Liste_Port, tooltips, "tooltips");

	return window_Liste_Port;
}

GtkWidget *create_window_Force_Reseau (void)
{
	window_Force_Reseau = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_Force_Reseau, 340, 115);
	gtk_window_set_title (GTK_WINDOW (window_Force_Reseau), "Add a subnet");
	gtk_window_set_position (GTK_WINDOW (window_Force_Reseau), GTK_WIN_POS_CENTER);
	window_Force_Reseau_icon_pixbuf = create_pixbuf ("/usr/share/pixmaps/AutoScan/AutoScan.png");
	if (window_Force_Reseau_icon_pixbuf) {
		gtk_window_set_icon (GTK_WINDOW (window_Force_Reseau), window_Force_Reseau_icon_pixbuf);
		gdk_pixbuf_unref (window_Force_Reseau_icon_pixbuf);
	}

	fixed_Force_Reseau = gtk_fixed_new ();
	gtk_widget_show (fixed_Force_Reseau);
	gtk_container_add (GTK_CONTAINER (window_Force_Reseau), fixed_Force_Reseau);

	entry_Force_Reseau = gtk_entry_new ();
	gtk_widget_show (entry_Force_Reseau);
	gtk_fixed_put (GTK_FIXED (fixed_Force_Reseau), entry_Force_Reseau, 112, 14);
	gtk_widget_set_size_request (entry_Force_Reseau, 160, 25);

	label_Force_Reseau_Format = gtk_label_new ("Format : a.b.c.255");
	gtk_widget_show (label_Force_Reseau_Format);
	gtk_fixed_put (GTK_FIXED (fixed_Force_Reseau), label_Force_Reseau_Format, 100, 45);
	gtk_widget_set_size_request (label_Force_Reseau_Format, 129, 16);

	label_Force_Reseau = gtk_label_new ("subnet :");
	gtk_widget_show (label_Force_Reseau);
	gtk_fixed_put (GTK_FIXED (fixed_Force_Reseau), label_Force_Reseau, 8, 16);
	gtk_widget_set_size_request (label_Force_Reseau, 96, 16);

	button_Force_Reseau_Ok = gtk_button_new ();
	gtk_widget_show (button_Force_Reseau_Ok);
	gtk_fixed_put (GTK_FIXED (fixed_Force_Reseau), button_Force_Reseau_Ok, 112, 72);
	gtk_widget_set_size_request (button_Force_Reseau_Ok, 104, 32);

	alignment_Force_Reseau_Ok = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Force_Reseau_Ok);
	gtk_container_add (GTK_CONTAINER (button_Force_Reseau_Ok), alignment_Force_Reseau_Ok);

	hbox_Force_Reseau_Ok = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Force_Reseau_Ok);
	gtk_container_add (GTK_CONTAINER (alignment_Force_Reseau_Ok), hbox_Force_Reseau_Ok);

	image_Force_Reseau_Ok = create_pixmap (window_Force_Reseau, "/usr/share/pixmaps/AutoScan/button_ok.png");
	gtk_widget_show (image_Force_Reseau_Ok);
	gtk_box_pack_start (GTK_BOX (hbox_Force_Reseau_Ok), image_Force_Reseau_Ok, FALSE, FALSE, 0);

	label_Force_Reseau_Ok = gtk_label_new_with_mnemonic ("Ok");
	gtk_widget_show (label_Force_Reseau_Ok);
	gtk_box_pack_start (GTK_BOX (hbox_Force_Reseau_Ok), label_Force_Reseau_Ok, FALSE, FALSE, 0);

	button_Force_Reseau_Annuler = gtk_button_new ();
	gtk_widget_show (button_Force_Reseau_Annuler);
	gtk_fixed_put (GTK_FIXED (fixed_Force_Reseau), button_Force_Reseau_Annuler, 224, 72);
	gtk_widget_set_size_request (button_Force_Reseau_Annuler, 104, 32);

	alignment_Force_Reseau_Annuler = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Force_Reseau_Annuler);
	gtk_container_add (GTK_CONTAINER (button_Force_Reseau_Annuler), alignment_Force_Reseau_Annuler);

	hbox_Force_Reseau_Annuler = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Force_Reseau_Annuler);
	gtk_container_add (GTK_CONTAINER (alignment_Force_Reseau_Annuler), hbox_Force_Reseau_Annuler);

	image_Force_Reseau_Annuler = create_pixmap (window_Force_Reseau, "/usr/share/pixmaps/AutoScan/button_cancel.png");
	gtk_widget_show (image_Force_Reseau_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Force_Reseau_Annuler), image_Force_Reseau_Annuler, FALSE, FALSE, 0);

	label_Force_Reseau_Annuler = gtk_label_new_with_mnemonic ("Cancel");
	gtk_widget_show (label_Force_Reseau_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Force_Reseau_Annuler), label_Force_Reseau_Annuler, FALSE, FALSE, 0);

	g_signal_connect ((gpointer) window_Force_Reseau, "destroy", G_CALLBACK (on_window_Force_Reseau_destroy), NULL);
	g_signal_connect ((gpointer) button_Force_Reseau_Ok, "clicked", G_CALLBACK (on_button_Force_Reseau_Appliquer_clicked), NULL);
	g_signal_connect ((gpointer) button_Force_Reseau_Annuler, "clicked", G_CALLBACK (on_button_Force_Reseau_Annuler_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_Force_Reseau, window_Force_Reseau, "window_Force_Reseau");
	GLADE_HOOKUP_OBJECT (window_Force_Reseau, fixed_Force_Reseau, "fixed_Force_Reseau");
	GLADE_HOOKUP_OBJECT (window_Force_Reseau, entry_Force_Reseau, "entry_Force_Reseau");
	GLADE_HOOKUP_OBJECT (window_Force_Reseau, label_Force_Reseau_Format, "label_Force_Reseau_Format");
	GLADE_HOOKUP_OBJECT (window_Force_Reseau, label_Force_Reseau, "label_Force_Reseau");
	GLADE_HOOKUP_OBJECT (window_Force_Reseau, button_Force_Reseau_Ok, "button_Force_Reseau_Ok");
	GLADE_HOOKUP_OBJECT (window_Force_Reseau, alignment_Force_Reseau_Ok, "alignment_Force_Reseau_Ok");
	GLADE_HOOKUP_OBJECT (window_Force_Reseau, hbox_Force_Reseau_Ok, "hbox_Force_Reseau_Ok");
	GLADE_HOOKUP_OBJECT (window_Force_Reseau, image_Force_Reseau_Ok, "image_Force_Reseau_Ok");
	GLADE_HOOKUP_OBJECT (window_Force_Reseau, label_Force_Reseau_Ok, "label_Force_Reseau_Ok");
	GLADE_HOOKUP_OBJECT (window_Force_Reseau, button_Force_Reseau_Annuler, "button_Force_Reseau_Annuler");
	GLADE_HOOKUP_OBJECT (window_Force_Reseau, alignment_Force_Reseau_Annuler, "alignment_Force_Reseau_Annuler");
	GLADE_HOOKUP_OBJECT (window_Force_Reseau, hbox_Force_Reseau_Annuler, "hbox_Force_Reseau_Annuler");
	GLADE_HOOKUP_OBJECT (window_Force_Reseau, image_Force_Reseau_Annuler, "image_Force_Reseau_Annuler");
	GLADE_HOOKUP_OBJECT (window_Force_Reseau, label_Force_Reseau_Annuler, "label_Force_Reseau_Annuler");

	return window_Force_Reseau;
}

GtkWidget *create_window_Ajout_Script (void)
{
	tooltips = gtk_tooltips_new ();

	window_Ajout_Script = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_Ajout_Script, 562, 346);
	gtk_window_set_title (GTK_WINDOW (window_Ajout_Script), "Add a script");
	gtk_window_set_position (GTK_WINDOW (window_Ajout_Script), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window_Ajout_Script), FALSE);
	window_Ajout_Script_icon_pixbuf = create_pixbuf ("/usr/share/pixmaps/AutoScan/AutoScan.png");
	if (window_Ajout_Script_icon_pixbuf) {
		gtk_window_set_icon (GTK_WINDOW (window_Ajout_Script), window_Ajout_Script_icon_pixbuf);
		gdk_pixbuf_unref (window_Ajout_Script_icon_pixbuf);
	}

	fixed_Ajout_Script = gtk_fixed_new ();
	gtk_widget_show (fixed_Ajout_Script);
	gtk_container_add (GTK_CONTAINER (window_Ajout_Script), fixed_Ajout_Script);

	scrolledwindow_Ajout_Script = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Ajout_Script);
	gtk_fixed_put (GTK_FIXED (fixed_Ajout_Script), scrolledwindow_Ajout_Script, 16, 8);
	gtk_widget_set_size_request (scrolledwindow_Ajout_Script, 539, 144);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_Ajout_Script), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	ctree_Ajout_Script = gtk_ctree_new (2, 0);
	gtk_widget_show (ctree_Ajout_Script);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Ajout_Script), ctree_Ajout_Script);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Ajout_Script), 0, 151);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Ajout_Script), 1, 80);
	gtk_clist_column_titles_show (GTK_CLIST (ctree_Ajout_Script));

	label_Ajout_Script = gtk_label_new ("Script name");
	gtk_widget_show (label_Ajout_Script);
	gtk_clist_set_column_widget (GTK_CLIST (ctree_Ajout_Script), 0, label_Ajout_Script);

	label_Ajout_Script = gtk_label_new ("Path");
	gtk_widget_show (label_Ajout_Script);
	gtk_clist_set_column_widget (GTK_CLIST (ctree_Ajout_Script), 1, label_Ajout_Script);

	frame_Ajout_Script = gtk_frame_new (NULL);
	gtk_widget_show (frame_Ajout_Script);
	gtk_fixed_put (GTK_FIXED (fixed_Ajout_Script), frame_Ajout_Script, 16, 176);
	gtk_widget_set_size_request (frame_Ajout_Script, 416, 136);

	fixed_Ajout_Script_Modif = gtk_fixed_new ();
	gtk_widget_show (fixed_Ajout_Script_Modif);
	gtk_container_add (GTK_CONTAINER (frame_Ajout_Script), fixed_Ajout_Script_Modif);

	entry_Ajout_Script_Nom = gtk_entry_new ();
	gtk_widget_show (entry_Ajout_Script_Nom);
	gtk_fixed_put (GTK_FIXED (fixed_Ajout_Script_Modif), entry_Ajout_Script_Nom, 88, 8);
	gtk_widget_set_size_request (entry_Ajout_Script_Nom, 216, 21);

	entry_Ajout_Script_Parametre = gtk_entry_new ();
	gtk_widget_show (entry_Ajout_Script_Parametre);
	gtk_fixed_put (GTK_FIXED (fixed_Ajout_Script_Modif), entry_Ajout_Script_Parametre, 88, 72);
	gtk_widget_set_size_request (entry_Ajout_Script_Parametre, 216, 21);
	gtk_tooltips_set_tip (tooltips, entry_Ajout_Script_Parametre, "Parameters : _IP_ : IP adress, _MAC_ : Adress Mac", NULL);

	fileentry_Ajout_Script = gnome_file_entry_new (NULL, NULL);
	gtk_widget_show (fileentry_Ajout_Script);
	gtk_fixed_put (GTK_FIXED (fixed_Ajout_Script_Modif), fileentry_Ajout_Script, 88, 32);
	gtk_widget_set_size_request (fileentry_Ajout_Script, 316, 31);

	combo_entry_Ajout_Script = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (fileentry_Ajout_Script));
	gtk_widget_show (combo_entry_Ajout_Script);

	label_Ajout_Script_Parametre = gtk_label_new ("Params :");
	gtk_widget_show (label_Ajout_Script_Parametre);
	gtk_fixed_put (GTK_FIXED (fixed_Ajout_Script_Modif), label_Ajout_Script_Parametre, 16, 72);
	gtk_widget_set_size_request (label_Ajout_Script_Parametre, 56, 16);

	label_Ajout_Script_Nom = gtk_label_new ("Script name : ");
	gtk_widget_show (label_Ajout_Script_Nom);
	gtk_fixed_put (GTK_FIXED (fixed_Ajout_Script_Modif), label_Ajout_Script_Nom, 8, 16);
	gtk_widget_set_size_request (label_Ajout_Script_Nom, 72, 16);

	label_Ajout_Script_Chemin = gtk_label_new ("Path : ");
	gtk_widget_show (label_Ajout_Script_Chemin);
	gtk_fixed_put (GTK_FIXED (fixed_Ajout_Script_Modif), label_Ajout_Script_Chemin, 16, 42);
	gtk_widget_set_size_request (label_Ajout_Script_Chemin, 72, 16);

	label_Ajout_Script_Modif = gtk_label_new ("Change, add, delete ");
	gtk_widget_show (label_Ajout_Script_Modif);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Ajout_Script), label_Ajout_Script_Modif);

	button_Ajout_Script_Enlever = gtk_button_new ();
	gtk_widget_show (button_Ajout_Script_Enlever);
	gtk_fixed_put (GTK_FIXED (fixed_Ajout_Script), button_Ajout_Script_Enlever, 448, 197);
	gtk_widget_set_size_request (button_Ajout_Script_Enlever, 104, 36);

	alignment_Ajout_Script_Enlever = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Ajout_Script_Enlever);
	gtk_container_add (GTK_CONTAINER (button_Ajout_Script_Enlever), alignment_Ajout_Script_Enlever);

	hbox_Ajout_Script_Enlever = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Ajout_Script_Enlever);
	gtk_container_add (GTK_CONTAINER (alignment_Ajout_Script_Enlever), hbox_Ajout_Script_Enlever);

	image_Ajout_Script_Enlever = create_pixmap (window_Ajout_Script, "/usr/share/pixmaps/AutoScan/editcut.png");
	gtk_widget_show (image_Ajout_Script_Enlever);
	gtk_box_pack_start (GTK_BOX (hbox_Ajout_Script_Enlever), image_Ajout_Script_Enlever, FALSE, FALSE, 0);

	label_Ajout_Script_Enlever = gtk_label_new_with_mnemonic ("Delete");
	gtk_widget_show (label_Ajout_Script_Enlever);
	gtk_box_pack_start (GTK_BOX (hbox_Ajout_Script_Enlever), label_Ajout_Script_Enlever, FALSE, FALSE, 0);

	button_Ajout_Script_Ok = gtk_button_new ();
	gtk_widget_show (button_Ajout_Script_Ok);
	gtk_fixed_put (GTK_FIXED (fixed_Ajout_Script), button_Ajout_Script_Ok, 448, 232);
	gtk_widget_set_size_request (button_Ajout_Script_Ok, 104, 36);

	alignment_Ajout_Script_Ok = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Ajout_Script_Ok);
	gtk_container_add (GTK_CONTAINER (button_Ajout_Script_Ok), alignment_Ajout_Script_Ok);

	hbox_Ajout_Script_Ok = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Ajout_Script_Ok);
	gtk_container_add (GTK_CONTAINER (alignment_Ajout_Script_Ok), hbox_Ajout_Script_Ok);

	image_Ajout_Script_Ok = create_pixmap (window_Ajout_Script, "/usr/share/pixmaps/AutoScan/button_ok.png");
	gtk_widget_show (image_Ajout_Script_Ok);
	gtk_box_pack_start (GTK_BOX (hbox_Ajout_Script_Ok), image_Ajout_Script_Ok, FALSE, FALSE, 0);

	label_Ajout_Script_Ok = gtk_label_new_with_mnemonic ("Ok");
	gtk_widget_show (label_Ajout_Script_Ok);
	gtk_box_pack_start (GTK_BOX (hbox_Ajout_Script_Ok), label_Ajout_Script_Ok, FALSE, FALSE, 0);

	button_Ajout_Script_Annuler = gtk_button_new ();
	gtk_widget_show (button_Ajout_Script_Annuler);
	gtk_fixed_put (GTK_FIXED (fixed_Ajout_Script), button_Ajout_Script_Annuler, 448, 267);
	gtk_widget_set_size_request (button_Ajout_Script_Annuler, 104, 36);

	alignment_Ajout_Script_Annuler = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Ajout_Script_Annuler);
	gtk_container_add (GTK_CONTAINER (button_Ajout_Script_Annuler), alignment_Ajout_Script_Annuler);

	hbox_Ajout_Script_Annuler = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Ajout_Script_Annuler);
	gtk_container_add (GTK_CONTAINER (alignment_Ajout_Script_Annuler), hbox_Ajout_Script_Annuler);

	image_Ajout_Script_Annuler = create_pixmap (window_Ajout_Script, "/usr/share/pixmaps/AutoScan/button_cancel.png");
	gtk_widget_show (image_Ajout_Script_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Ajout_Script_Annuler), image_Ajout_Script_Annuler, FALSE, FALSE, 0);

	label_Ajout_Script_Annuler = gtk_label_new_with_mnemonic ("Cancel");
	gtk_widget_show (label_Ajout_Script_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Ajout_Script_Annuler), label_Ajout_Script_Annuler, FALSE, FALSE, 0);

	button_Ajout_Script_Ajouter = gtk_button_new ();
	gtk_widget_show (button_Ajout_Script_Ajouter);
	gtk_fixed_put (GTK_FIXED (fixed_Ajout_Script), button_Ajout_Script_Ajouter, 448, 160);
	gtk_widget_set_size_request (button_Ajout_Script_Ajouter, 104, 36);

	alignment_Ajout_Script_Ajouter = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Ajout_Script_Ajouter);
	gtk_container_add (GTK_CONTAINER (button_Ajout_Script_Ajouter), alignment_Ajout_Script_Ajouter);

	hbox_Ajout_Script_Ajouter = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Ajout_Script_Ajouter);
	gtk_container_add (GTK_CONTAINER (alignment_Ajout_Script_Ajouter), hbox_Ajout_Script_Ajouter);

	image_Ajout_Script_Ajouter = create_pixmap (window_Ajout_Script, "/usr/share/pixmaps/AutoScan/filenew.png");
	gtk_widget_show (image_Ajout_Script_Ajouter);
	gtk_box_pack_start (GTK_BOX (hbox_Ajout_Script_Ajouter), image_Ajout_Script_Ajouter, FALSE, FALSE, 0);

	label_Ajout_Script_Ajouter = gtk_label_new_with_mnemonic ("Add");
	gtk_widget_show (label_Ajout_Script_Ajouter);
	gtk_box_pack_start (GTK_BOX (hbox_Ajout_Script_Ajouter), label_Ajout_Script_Ajouter, FALSE, FALSE, 0);

	button_Ajout_Script_Sauvegarder = gtk_button_new ();
	gtk_widget_show (button_Ajout_Script_Sauvegarder);
	gtk_fixed_put (GTK_FIXED (fixed_Ajout_Script), button_Ajout_Script_Sauvegarder, 448, 304);
	gtk_widget_set_size_request (button_Ajout_Script_Sauvegarder, 104, 36);

	alignment_Ajout_Script_Sauvegarder = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Ajout_Script_Sauvegarder);
	gtk_container_add (GTK_CONTAINER (button_Ajout_Script_Sauvegarder), alignment_Ajout_Script_Sauvegarder);

	hbox_Ajout_Script_Sauvegarder = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Ajout_Script_Sauvegarder);
	gtk_container_add (GTK_CONTAINER (alignment_Ajout_Script_Sauvegarder), hbox_Ajout_Script_Sauvegarder);

	image_Ajout_Script_Sauvegarder = create_pixmap (window_Ajout_Script, "/usr/share/pixmaps/AutoScan/filesave.png");
	gtk_widget_show (image_Ajout_Script_Sauvegarder);
	gtk_box_pack_start (GTK_BOX (hbox_Ajout_Script_Sauvegarder), image_Ajout_Script_Sauvegarder, FALSE, FALSE, 0);

	label_Ajout_Script_Sauvegarder = gtk_label_new_with_mnemonic ("Save");
	gtk_widget_show (label_Ajout_Script_Sauvegarder);
	gtk_box_pack_start (GTK_BOX (hbox_Ajout_Script_Sauvegarder), label_Ajout_Script_Sauvegarder, FALSE, FALSE, 0);

	g_signal_connect ((gpointer) window_Ajout_Script, "destroy", G_CALLBACK (on_window_Ajout_Script_destroy), NULL);
	g_signal_connect ((gpointer) ctree_Ajout_Script, "tree_select_row", G_CALLBACK (on_ctree_Ajout_Script_tree_select_row), NULL);
	g_signal_connect ((gpointer) entry_Ajout_Script_Nom, "changed", G_CALLBACK (Liste_Script_changed), NULL);
	g_signal_connect ((gpointer) entry_Ajout_Script_Parametre, "changed", G_CALLBACK (Liste_Script_changed), NULL);
	g_signal_connect ((gpointer) combo_entry_Ajout_Script, "changed", G_CALLBACK (Liste_Script_changed), NULL);
	g_signal_connect ((gpointer) button_Ajout_Script_Enlever, "clicked", G_CALLBACK (on_button_Ajout_Script_Enlever_clicked), NULL);
	g_signal_connect ((gpointer) button_Ajout_Script_Ok, "clicked", G_CALLBACK (on_button_Ajout_Script_Ok_clicked), NULL);
	g_signal_connect ((gpointer) button_Ajout_Script_Annuler, "clicked", G_CALLBACK (on_button_Ajout_Script_Annuler_clicked), NULL);
	g_signal_connect ((gpointer) button_Ajout_Script_Ajouter, "clicked", G_CALLBACK (on_button_Ajout_Script_Ajouter_clicked), NULL);
	g_signal_connect ((gpointer) button_Ajout_Script_Sauvegarder, "clicked", G_CALLBACK (on_button_Ajout_Script_Sauvegarder_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_Ajout_Script, window_Ajout_Script, "window_Ajout_Script");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, fixed_Ajout_Script, "fixed_Ajout_Script");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, scrolledwindow_Ajout_Script, "scrolledwindow_Ajout_Script");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, ctree_Ajout_Script, "ctree_Ajout_Script");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, label_Ajout_Script, "label_Ajout_Script");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, label_Ajout_Script, "label_Ajout_Script");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, frame_Ajout_Script, "frame_Ajout_Script");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, fixed_Ajout_Script_Modif, "fixed_Ajout_Script_Modif");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, entry_Ajout_Script_Nom, "entry_Ajout_Script_Nom");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, entry_Ajout_Script_Parametre, "entry_Ajout_Script_Parametre");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, fileentry_Ajout_Script, "fileentry_Ajout_Script");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, combo_entry_Ajout_Script, "combo_entry_Ajout_Script");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, label_Ajout_Script_Parametre, "label_Ajout_Script_Parametre");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, label_Ajout_Script_Nom, "label_Ajout_Script_Nom");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, label_Ajout_Script_Chemin, "label_Ajout_Script_Chemin");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, label_Ajout_Script_Modif, "label_Ajout_Script_Modif");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, button_Ajout_Script_Enlever, "button_Ajout_Script_Enlever");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, alignment_Ajout_Script_Enlever, "alignment_Ajout_Script_Enlever");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, hbox_Ajout_Script_Enlever, "hbox_Ajout_Script_Enlever");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, image_Ajout_Script_Enlever, "image_Ajout_Script_Enlever");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, label_Ajout_Script_Enlever, "label_Ajout_Script_Enlever");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, button_Ajout_Script_Ok, "button_Ajout_Script_Ok");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, alignment_Ajout_Script_Ok, "alignment_Ajout_Script_Ok");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, hbox_Ajout_Script_Ok, "hbox_Ajout_Script_Ok");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, image_Ajout_Script_Ok, "image_Ajout_Script_Ok");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, label_Ajout_Script_Ok, "label_Ajout_Script_Ok");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, button_Ajout_Script_Annuler, "button_Ajout_Script_Annuler");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, alignment_Ajout_Script_Annuler, "alignment_Ajout_Script_Annuler");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, hbox_Ajout_Script_Annuler, "hbox_Ajout_Script_Annuler");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, image_Ajout_Script_Annuler, "image_Ajout_Script_Annuler");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, label_Ajout_Script_Annuler, "label_Ajout_Script_Annuler");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, button_Ajout_Script_Ajouter, "button_Ajout_Script_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, alignment_Ajout_Script_Ajouter, "alignment_Ajout_Script_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, hbox_Ajout_Script_Ajouter, "hbox_Ajout_Script_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, image_Ajout_Script_Ajouter, "image_Ajout_Script_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, label_Ajout_Script_Ajouter, "label_Ajout_Script_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, button_Ajout_Script_Sauvegarder, "button_Ajout_Script_Sauvegarder");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, alignment_Ajout_Script_Sauvegarder, "alignment_Ajout_Script_Sauvegarder");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, hbox_Ajout_Script_Sauvegarder, "hbox_Ajout_Script_Sauvegarder");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, image_Ajout_Script_Sauvegarder, "image_Ajout_Script_Sauvegarder");
	GLADE_HOOKUP_OBJECT (window_Ajout_Script, label_Ajout_Script_Sauvegarder, "label_Ajout_Script_Sauvegarder");
	GLADE_HOOKUP_OBJECT_NO_REF (window_Ajout_Script, tooltips, "tooltips");

	return window_Ajout_Script;
}


GtkWidget *create_fileselection_Enregistrer (void)
{


	fileselection_Enregistrer = gtk_file_selection_new ("Save");
	gtk_container_set_border_width (GTK_CONTAINER (fileselection_Enregistrer), 10);
	gtk_window_set_position (GTK_WINDOW (fileselection_Enregistrer), GTK_WIN_POS_CENTER);

	ok_button_Enregistrer = GTK_FILE_SELECTION (fileselection_Enregistrer)->ok_button;
	gtk_widget_show (ok_button_Enregistrer);
	GTK_WIDGET_SET_FLAGS (ok_button_Enregistrer, GTK_CAN_DEFAULT);

	cancel_button_Enregistrer = GTK_FILE_SELECTION (fileselection_Enregistrer)->cancel_button;
	gtk_widget_show (cancel_button_Enregistrer);
	GTK_WIDGET_SET_FLAGS (cancel_button_Enregistrer, GTK_CAN_DEFAULT);

	g_signal_connect ((gpointer) fileselection_Enregistrer, "destroy", G_CALLBACK (on_fileselection_Enregistrer_destroy), NULL);
	g_signal_connect ((gpointer) ok_button_Enregistrer, "clicked", G_CALLBACK (on_ok_button_Enregistrer_clicked), NULL);
	g_signal_connect ((gpointer) cancel_button_Enregistrer, "clicked", G_CALLBACK (on_cancel_button_Enregistrer_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (fileselection_Enregistrer, fileselection_Enregistrer, "fileselection_Enregistrer");
	GLADE_HOOKUP_OBJECT_NO_REF (fileselection_Enregistrer, ok_button_Enregistrer, "ok_button_Enregistrer");
	GLADE_HOOKUP_OBJECT_NO_REF (fileselection_Enregistrer, cancel_button_Enregistrer, "cancel_button_Enregistrer");

	return fileselection_Enregistrer;
}

/*GtkWidget *create_fileselection_Enregistrer (void)
{
	fileselection_Enregistrer = gtk_file_selection_new ("Save");
	gtk_container_set_border_width (GTK_CONTAINER (fileselection_Enregistrer), 10);
	gtk_window_set_position (GTK_WINDOW (fileselection_Enregistrer), GTK_WIN_POS_CENTER);
	fileselection_Enregistrer_icon_pixbuf = create_pixbuf ("/usr/share/pixmaps/AutoScan/AutoScan.png");
	if (fileselection_Enregistrer_icon_pixbuf) {
		gtk_window_set_icon (GTK_WINDOW (fileselection_Enregistrer), fileselection_Enregistrer_icon_pixbuf);
		gdk_pixbuf_unref (fileselection_Enregistrer_icon_pixbuf);
	}
	ok_button_Enregistrer = GTK_FILE_SELECTION (fileselection_Enregistrer)->ok_button;
	gtk_widget_show (ok_button_Enregistrer);
	gtk_widget_set_size_request (ok_button_Enregistrer, 104, 32);
	GTK_WIDGET_SET_FLAGS (ok_button_Enregistrer, GTK_CAN_DEFAULT);

	alignment_Enregistrer_Save = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Enregistrer_Save);
	gtk_container_add (GTK_CONTAINER (ok_button_Enregistrer), alignment_Enregistrer_Save);

	hbox_Enregistrer_Save = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Enregistrer_Save);
	gtk_container_add (GTK_CONTAINER (alignment_Enregistrer_Save), hbox_Enregistrer_Save);

	image_Enregistrer_Save = create_pixmap (fileselection_Enregistrer, "/usr/share/pixmaps/AutoScan/filesave.png");
	gtk_widget_show (image_Enregistrer_Save);
	gtk_box_pack_start (GTK_BOX (hbox_Enregistrer_Save), image_Enregistrer_Save, FALSE, FALSE, 0);

	label_Enregistrer_Save = gtk_label_new_with_mnemonic ("Save");
	gtk_widget_show (label_Enregistrer_Save);
	gtk_box_pack_start (GTK_BOX (hbox_Enregistrer_Save), label_Enregistrer_Save, FALSE, FALSE, 0);

	cancel_button_Enregistrer = GTK_FILE_SELECTION (fileselection_Enregistrer)->cancel_button;
	gtk_widget_show (cancel_button_Enregistrer);
	gtk_widget_set_size_request (cancel_button_Enregistrer, 104, 35);
	GTK_WIDGET_SET_FLAGS (cancel_button_Enregistrer, GTK_CAN_DEFAULT);

	g_signal_connect ((gpointer) fileselection_Enregistrer, "destroy", G_CALLBACK (on_fileselection_Enregistrer_destroy), NULL);
	g_signal_connect ((gpointer) ok_button_Enregistrer, "clicked", G_CALLBACK (on_ok_button_Enregistrer_clicked), NULL);
	g_signal_connect ((gpointer) cancel_button_Enregistrer, "clicked", G_CALLBACK (on_cancel_button_Enregistrer_clicked), NULL);

	GLADE_HOOKUP_OBJECT_NO_REF (fileselection_Enregistrer, fileselection_Enregistrer, "fileselection_Enregistrer");
	GLADE_HOOKUP_OBJECT_NO_REF (fileselection_Enregistrer, ok_button_Enregistrer, "ok_button_Enregistrer");
	GLADE_HOOKUP_OBJECT (fileselection_Enregistrer, alignment_Enregistrer_Save, "alignment_Enregistrer_Save");
	GLADE_HOOKUP_OBJECT (fileselection_Enregistrer, hbox_Enregistrer_Save, "hbox_Enregistrer_Save");
	GLADE_HOOKUP_OBJECT (fileselection_Enregistrer, image_Enregistrer_Save, "image_Enregistrer_Save");
	GLADE_HOOKUP_OBJECT (fileselection_Enregistrer, label_Enregistrer_Save, "label_Enregistrer_Save");
	GLADE_HOOKUP_OBJECT_NO_REF (fileselection_Enregistrer, cancel_button_Enregistrer, "cancel_button_Enregistrer");

	return fileselection_Enregistrer;
}
*/
GtkWidget *create_window_Preference (void)
{
	GList    *combo_Preference_Sous_reseau_privee_items = NULL;

	window_Preference = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_Preference, 702, 417);
	gtk_window_set_title (GTK_WINDOW (window_Preference), "Preferences");
	gtk_window_set_position (GTK_WINDOW (window_Preference), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window_Preference), FALSE);
	window_Preference_icon_pixbuf = create_pixbuf ("/usr/share/pixmaps/AutoScan/AutoScan.png");
	if (window_Preference_icon_pixbuf) {
		gtk_window_set_icon (GTK_WINDOW (window_Preference), window_Preference_icon_pixbuf);
		gdk_pixbuf_unref (window_Preference_icon_pixbuf);
	}

	fixed_Preference = gtk_fixed_new ();
	gtk_widget_show (fixed_Preference);
	gtk_container_add (GTK_CONTAINER (window_Preference), fixed_Preference);

	notebook_Preference = gtk_notebook_new ();
	gtk_widget_show (notebook_Preference);
	gtk_fixed_put (GTK_FIXED (fixed_Preference), notebook_Preference, 0, 0);
	gtk_widget_set_size_request (notebook_Preference, 703, 371);
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook_Preference), GTK_POS_LEFT);

	fixed_Preference_Interface = gtk_fixed_new ();
	gtk_widget_show (fixed_Preference_Interface);
	gtk_container_add (GTK_CONTAINER (notebook_Preference), fixed_Preference_Interface);

	spinbutton_Preference_Interface_Max_Thread_Scan_adj = gtk_adjustment_new (50, 10, 1000, 1, 10, 10);
	spinbutton_Preference_Interface_Max_Thread_Scan = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_Preference_Interface_Max_Thread_Scan_adj), 1, 0);
	gtk_widget_show (spinbutton_Preference_Interface_Max_Thread_Scan);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Interface), spinbutton_Preference_Interface_Max_Thread_Scan, 104, 44);
	gtk_widget_set_size_request (spinbutton_Preference_Interface_Max_Thread_Scan, 64, 25);

	label_Preference_Interface_Max_Thread_Scan = gtk_label_new ("Thread max");
	gtk_widget_show (label_Preference_Interface_Max_Thread_Scan);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Interface), label_Preference_Interface_Max_Thread_Scan, 8, 48);
	gtk_widget_set_size_request (label_Preference_Interface_Max_Thread_Scan, 96, 16);

	combo_Preference_Interface_Interface = gtk_combo_new ();
	g_object_set_data (G_OBJECT (GTK_COMBO (combo_Preference_Interface_Interface)->popwin), "GladeParentKey", combo_Preference_Interface_Interface);
	gtk_widget_show (combo_Preference_Interface_Interface);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Interface), combo_Preference_Interface_Interface, 104, 12);
	gtk_widget_set_size_request (combo_Preference_Interface_Interface, 91, 25);

	combo_entry_Preference_Interface = GTK_COMBO (combo_Preference_Interface_Interface)->entry;
	gtk_widget_show (combo_entry_Preference_Interface);

	label_Preference_Interface_Interface = gtk_label_new ("Interface :");
	gtk_widget_show (label_Preference_Interface_Interface);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Interface), label_Preference_Interface_Interface, 8, 16);
	gtk_widget_set_size_request (label_Preference_Interface_Interface, 80, 16);

	frame_Preference_Sous_reseau_privee = gtk_frame_new (NULL);
	gtk_widget_show (frame_Preference_Sous_reseau_privee);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Interface), frame_Preference_Sous_reseau_privee, 8, 80);
	gtk_widget_set_size_request (frame_Preference_Sous_reseau_privee, 560, 272);

	fixed_Preference_Sous_reseau_privee = gtk_fixed_new ();
	gtk_widget_show (fixed_Preference_Sous_reseau_privee);
	gtk_container_add (GTK_CONTAINER (frame_Preference_Sous_reseau_privee), fixed_Preference_Sous_reseau_privee);

	scrolledwindow_Preference_Sous_reseau_privee = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Preference_Sous_reseau_privee);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Sous_reseau_privee), scrolledwindow_Preference_Sous_reseau_privee, 8, 16);
	gtk_widget_set_size_request (scrolledwindow_Preference_Sous_reseau_privee, 533, 124);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_Preference_Sous_reseau_privee), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	ctree_Preference_Sous_reseau_privee = gtk_ctree_new (2, 0);
	gtk_widget_show (ctree_Preference_Sous_reseau_privee);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Preference_Sous_reseau_privee), ctree_Preference_Sous_reseau_privee);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Preference_Sous_reseau_privee), 0, 159);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Preference_Sous_reseau_privee), 1, 326);
	gtk_clist_column_titles_show (GTK_CLIST (ctree_Preference_Sous_reseau_privee));

	label_Preference_Sous_reseau_privee_Mask = gtk_label_new (_("Mask"));
	gtk_widget_show (label_Preference_Sous_reseau_privee_Mask);
	gtk_clist_set_column_widget (GTK_CLIST (ctree_Preference_Sous_reseau_privee), 0, label_Preference_Sous_reseau_privee_Mask);


	label_Preference_Sous_reseau_privee_Sous_reseau = gtk_label_new ("subnet");
	gtk_widget_show (label_Preference_Sous_reseau_privee_Sous_reseau);
	gtk_clist_set_column_widget (GTK_CLIST (ctree_Preference_Sous_reseau_privee), 1, label_Preference_Sous_reseau_privee_Sous_reseau);

	combo_Preference_Sous_reseau_privee = gtk_combo_new ();
	g_object_set_data (G_OBJECT (GTK_COMBO (combo_Preference_Sous_reseau_privee)->popwin), "GladeParentKey", combo_Preference_Sous_reseau_privee);
	gtk_widget_show (combo_Preference_Sous_reseau_privee);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Sous_reseau_privee), combo_Preference_Sous_reseau_privee, 32, 160);
	gtk_widget_set_size_request (combo_Preference_Sous_reseau_privee, 138, 25);
	combo_Preference_Sous_reseau_privee_items = g_list_append (combo_Preference_Sous_reseau_privee_items, (gpointer) _("255.0.0.0"));
	combo_Preference_Sous_reseau_privee_items = g_list_append (combo_Preference_Sous_reseau_privee_items, (gpointer) _("255.255.0.0"));
	combo_Preference_Sous_reseau_privee_items = g_list_append (combo_Preference_Sous_reseau_privee_items, (gpointer) _("255.255.255.0"));
	gtk_combo_set_popdown_strings (GTK_COMBO (combo_Preference_Sous_reseau_privee), combo_Preference_Sous_reseau_privee_items);
	g_list_free (combo_Preference_Sous_reseau_privee_items);


	combo_entry_Preference_Sous_reseau_privee = GTK_COMBO (combo_Preference_Sous_reseau_privee)->entry;
	gtk_widget_show (combo_entry_Preference_Sous_reseau_privee);

	entry_Preference_Sous_reseau_privee_1 = gtk_entry_new ();
	gtk_widget_show (entry_Preference_Sous_reseau_privee_1);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Sous_reseau_privee), entry_Preference_Sous_reseau_privee_1, 232, 160);
	gtk_widget_set_size_request (entry_Preference_Sous_reseau_privee_1, 40, 24);
	gtk_entry_set_max_length (GTK_ENTRY (entry_Preference_Sous_reseau_privee_1), 3);

	entry_Preference_Sous_reseau_privee_3 = gtk_entry_new ();
	gtk_widget_show (entry_Preference_Sous_reseau_privee_3);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Sous_reseau_privee), entry_Preference_Sous_reseau_privee_3, 344, 160);
	gtk_widget_set_size_request (entry_Preference_Sous_reseau_privee_3, 40, 24);
	gtk_entry_set_max_length (GTK_ENTRY (entry_Preference_Sous_reseau_privee_3), 3);

	entry_Preference_Sous_reseau_privee_2 = gtk_entry_new ();
	gtk_widget_show (entry_Preference_Sous_reseau_privee_2);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Sous_reseau_privee), entry_Preference_Sous_reseau_privee_2, 288, 160);
	gtk_widget_set_size_request (entry_Preference_Sous_reseau_privee_2, 40, 24);
	gtk_entry_set_max_length (GTK_ENTRY (entry_Preference_Sous_reseau_privee_2), 3);


	button_Preference_Sous_reseau_privee_Enlever = gtk_button_new ();
	gtk_widget_show (button_Preference_Sous_reseau_privee_Enlever);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Sous_reseau_privee), button_Preference_Sous_reseau_privee_Enlever, 128, 200);
	gtk_widget_set_size_request (button_Preference_Sous_reseau_privee_Enlever, 104, 36);

	alignment_Preference_Sous_reseau_privee_Enlever = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Preference_Sous_reseau_privee_Enlever);
	gtk_container_add (GTK_CONTAINER (button_Preference_Sous_reseau_privee_Enlever), alignment_Preference_Sous_reseau_privee_Enlever);

	hbox_Preference_Sous_reseau_privee_Enlever = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Preference_Sous_reseau_privee_Enlever);
	gtk_container_add (GTK_CONTAINER (alignment_Preference_Sous_reseau_privee_Enlever), hbox_Preference_Sous_reseau_privee_Enlever);

	image_Preference_Sous_reseau_privee_Enlever = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/editcut.png");
	gtk_widget_show (image_Preference_Sous_reseau_privee_Enlever);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Sous_reseau_privee_Enlever), image_Preference_Sous_reseau_privee_Enlever, FALSE, FALSE, 0);

	label_Preference_Sous_reseau_privee_Enlever = gtk_label_new_with_mnemonic ("Delete");
	gtk_widget_show (label_Preference_Sous_reseau_privee_Enlever);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Sous_reseau_privee_Enlever), label_Preference_Sous_reseau_privee_Enlever, FALSE, FALSE, 0);

	button_Preference_Sous_reseau_privee_Ok = gtk_button_new ();
	gtk_widget_show (button_Preference_Sous_reseau_privee_Ok);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Sous_reseau_privee), button_Preference_Sous_reseau_privee_Ok, 328, 200);
	gtk_widget_set_size_request (button_Preference_Sous_reseau_privee_Ok, 104, 36);

	alignment_Preference_Sous_reseau_privee_Ok = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Preference_Sous_reseau_privee_Ok);
	gtk_container_add (GTK_CONTAINER (button_Preference_Sous_reseau_privee_Ok), alignment_Preference_Sous_reseau_privee_Ok);

	hbox_Preference_Sous_reseau_privee_Ok = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Preference_Sous_reseau_privee_Ok);
	gtk_container_add (GTK_CONTAINER (alignment_Preference_Sous_reseau_privee_Ok), hbox_Preference_Sous_reseau_privee_Ok);

	image_Preference_Sous_reseau_privee_Ok = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/button_ok.png");
	gtk_widget_show (image_Preference_Sous_reseau_privee_Ok);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Sous_reseau_privee_Ok), image_Preference_Sous_reseau_privee_Ok, FALSE, FALSE, 0);

	label_Preference_Sous_reseau_privee_Ok = gtk_label_new_with_mnemonic ("Ok");
	gtk_widget_show (label_Preference_Sous_reseau_privee_Ok);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Sous_reseau_privee_Ok), label_Preference_Sous_reseau_privee_Ok, FALSE, FALSE, 0);

	button_Preference_Sous_reseau_privee_Annuler = gtk_button_new ();
	gtk_widget_show (button_Preference_Sous_reseau_privee_Annuler);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Sous_reseau_privee), button_Preference_Sous_reseau_privee_Annuler, 440, 200);
	gtk_widget_set_size_request (button_Preference_Sous_reseau_privee_Annuler, 104, 36);

	alignment_Preference_Sous_reseau_privee_Annuler = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Preference_Sous_reseau_privee_Annuler);
	gtk_container_add (GTK_CONTAINER (button_Preference_Sous_reseau_privee_Annuler), alignment_Preference_Sous_reseau_privee_Annuler);

	hbox_Preference_Sous_reseau_privee_Annuler = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Preference_Sous_reseau_privee_Annuler);
	gtk_container_add (GTK_CONTAINER (alignment_Preference_Sous_reseau_privee_Annuler), hbox_Preference_Sous_reseau_privee_Annuler);

	image_Preference_Sous_reseau_privee_Annuler = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/button_cancel.png");
	gtk_widget_show (image_Preference_Sous_reseau_privee_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Sous_reseau_privee_Annuler), image_Preference_Sous_reseau_privee_Annuler, FALSE, FALSE, 0);

	label_Preference_Sous_reseau_privee_Annuler = gtk_label_new_with_mnemonic ("Cancel");
	gtk_widget_show (label_Preference_Sous_reseau_privee_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Sous_reseau_privee_Annuler), label_Preference_Sous_reseau_privee_Annuler, FALSE, FALSE, 0);

	button_Preference_Sous_reseau_privee_Ajouter = gtk_button_new ();
	gtk_widget_show (button_Preference_Sous_reseau_privee_Ajouter);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Sous_reseau_privee), button_Preference_Sous_reseau_privee_Ajouter, 16, 200);
	gtk_widget_set_size_request (button_Preference_Sous_reseau_privee_Ajouter, 104, 36);

	alignment_Preference_Sous_reseau_privee_Ajouter = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Preference_Sous_reseau_privee_Ajouter);
	gtk_container_add (GTK_CONTAINER (button_Preference_Sous_reseau_privee_Ajouter), alignment_Preference_Sous_reseau_privee_Ajouter);

	hbox_Preference_Sous_reseau_privee_Ajouter = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Preference_Sous_reseau_privee_Ajouter);
	gtk_container_add (GTK_CONTAINER (alignment_Preference_Sous_reseau_privee_Ajouter), hbox_Preference_Sous_reseau_privee_Ajouter);

	image_Preference_Sous_reseau_privee_Ajouter = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/filenew.png");
	gtk_widget_show (image_Preference_Sous_reseau_privee_Ajouter);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Sous_reseau_privee_Ajouter), image_Preference_Sous_reseau_privee_Ajouter, FALSE, FALSE, 0);

	label_Preference_Sous_reseau_privee_Ajouter = gtk_label_new_with_mnemonic ("Add");
	gtk_widget_show (label_Preference_Sous_reseau_privee_Ajouter);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Sous_reseau_privee_Ajouter), label_Preference_Sous_reseau_privee_Ajouter, FALSE, FALSE, 0);

	label_Preference_Interface_Reseau_Prive = gtk_label_new ("Private Subnet");
	gtk_widget_show (label_Preference_Interface_Reseau_Prive);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Preference_Sous_reseau_privee), label_Preference_Interface_Reseau_Prive);

	fixed_Menu_Interfaces = gtk_fixed_new ();
	gtk_widget_show (fixed_Menu_Interfaces);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_Preference), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook_Preference), 0), fixed_Menu_Interfaces);
	gtk_widget_set_size_request (fixed_Menu_Interfaces, 107, 36);

	image_Menu_Interfaces = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/network_32.png");
	gtk_widget_show (image_Menu_Interfaces);
	gtk_fixed_put (GTK_FIXED (fixed_Menu_Interfaces), image_Menu_Interfaces, 0, 0);
	gtk_widget_set_size_request (image_Menu_Interfaces, 32, 40);

	label_Menu_Interfaces = gtk_label_new ("Interfaces");
	gtk_widget_show (label_Menu_Interfaces);
	gtk_fixed_put (GTK_FIXED (fixed_Menu_Interfaces), label_Menu_Interfaces, 39, 11);
	gtk_widget_set_size_request (label_Menu_Interfaces, 63, 16);

	fixed_Preference_Type_equipement = gtk_fixed_new ();
	//gtk_widget_show (fixed_Preference_Type_equipement);
	gtk_container_add (GTK_CONTAINER (notebook_Preference), fixed_Preference_Type_equipement);

	scrolledwindow_Preference_Type_equipement = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Preference_Type_equipement);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement), scrolledwindow_Preference_Type_equipement, 8, 16);
	gtk_widget_set_size_request (scrolledwindow_Preference_Type_equipement, 136, 337);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_Preference_Type_equipement), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	ctree_Preference_Type_equipement = gtk_ctree_new (1, 0);
	gtk_widget_show (ctree_Preference_Type_equipement);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Preference_Type_equipement), ctree_Preference_Type_equipement);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Preference_Type_equipement), 0, 80);
	gtk_clist_column_titles_hide (GTK_CLIST (ctree_Preference_Type_equipement));

	entry_Preference_Type_equipement_Type = gtk_entry_new ();
	gtk_widget_show (entry_Preference_Type_equipement_Type);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement), entry_Preference_Type_equipement_Type, 208, 19);
	gtk_widget_set_size_request (entry_Preference_Type_equipement_Type, 166, 21);

	frame_Preference_Type_equipement_Etat_Port = gtk_frame_new (NULL);
	gtk_widget_show (frame_Preference_Type_equipement_Etat_Port);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement), frame_Preference_Type_equipement_Etat_Port, 160, 208);
	gtk_widget_set_size_request (frame_Preference_Type_equipement_Etat_Port, 304, 153);

	fixed_Preference_Type_equipement_Etat_Port = gtk_fixed_new ();
	gtk_widget_show (fixed_Preference_Type_equipement_Etat_Port);
	gtk_container_add (GTK_CONTAINER (frame_Preference_Type_equipement_Etat_Port), fixed_Preference_Type_equipement_Etat_Port);

	frame_Preference_Type_equipement_Etat_Port_Actions = gtk_frame_new (NULL);
	gtk_widget_show (frame_Preference_Type_equipement_Etat_Port_Actions);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement_Etat_Port), frame_Preference_Type_equipement_Etat_Port_Actions, 16, 22);
	gtk_widget_set_size_request (frame_Preference_Type_equipement_Etat_Port_Actions, 280, 101);

	fixed_Preference_Type_equipement_Etat_Port_Actions = gtk_fixed_new ();
	gtk_widget_show (fixed_Preference_Type_equipement_Etat_Port_Actions);
	gtk_container_add (GTK_CONTAINER (frame_Preference_Type_equipement_Etat_Port_Actions), fixed_Preference_Type_equipement_Etat_Port_Actions);

	entry_Preference_Type_equipement_Etat_Port_Actions_Email = gtk_entry_new ();
	gtk_widget_show (entry_Preference_Type_equipement_Etat_Port_Actions_Email);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement_Etat_Port_Actions), entry_Preference_Type_equipement_Etat_Port_Actions_Email, 80, 0);
	gtk_widget_set_size_request (entry_Preference_Type_equipement_Etat_Port_Actions_Email, 188, 21);

	entry_Preference_Type_equipement_Etat_Port_Actions_Script = gtk_entry_new ();
	gtk_widget_show (entry_Preference_Type_equipement_Etat_Port_Actions_Script);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement_Etat_Port_Actions), entry_Preference_Type_equipement_Etat_Port_Actions_Script, 80, 48);
	gtk_widget_set_size_request (entry_Preference_Type_equipement_Etat_Port_Actions_Script, 188, 21);

	checkbutton_Preference_Type_equipement_Etat_Port_Actions_Email = gtk_check_button_new_with_mnemonic ("Email ");
	gtk_widget_show (checkbutton_Preference_Type_equipement_Etat_Port_Actions_Email);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement_Etat_Port_Actions), checkbutton_Preference_Type_equipement_Etat_Port_Actions_Email, 16, 0);
	gtk_widget_set_size_request (checkbutton_Preference_Type_equipement_Etat_Port_Actions_Email, 136, 24);

	checkbutton_Preference_Type_equipement_Etat_Port_Actions_Script = gtk_check_button_new_with_mnemonic ("Script");
	gtk_widget_show (checkbutton_Preference_Type_equipement_Etat_Port_Actions_Script);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement_Etat_Port_Actions), checkbutton_Preference_Type_equipement_Etat_Port_Actions_Script, 16, 48);
	gtk_widget_set_size_request (checkbutton_Preference_Type_equipement_Etat_Port_Actions_Script, 72, 24);

	checkbutton_Preference_Type_equipement_Etat_Port_Actions_Message = gtk_check_button_new_with_mnemonic ("Message");
	gtk_widget_show (checkbutton_Preference_Type_equipement_Etat_Port_Actions_Message);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement_Etat_Port_Actions), checkbutton_Preference_Type_equipement_Etat_Port_Actions_Message, 16, 24);
	gtk_widget_set_size_request (checkbutton_Preference_Type_equipement_Etat_Port_Actions_Message, 98, 23);

	label_Preference_Type_equipement_Etat_Port_Actions = gtk_label_new ("Actions");
	gtk_widget_show (label_Preference_Type_equipement_Etat_Port_Actions);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Preference_Type_equipement_Etat_Port_Actions), label_Preference_Type_equipement_Etat_Port_Actions);

	checkbutton_Preference_Type_equipement_Etat_Port_Down = gtk_check_button_new_with_mnemonic ("Poll even when it is closed");
	gtk_widget_show (checkbutton_Preference_Type_equipement_Etat_Port_Down);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement_Etat_Port), checkbutton_Preference_Type_equipement_Etat_Port_Down, 16, 0);
	gtk_widget_set_size_request (checkbutton_Preference_Type_equipement_Etat_Port_Down, 168, 24);

	checkbutton_Preference_Type_equipement_Etat_Port = gtk_check_button_new_with_mnemonic ("Detect the state change of the selected port");
	gtk_widget_show (checkbutton_Preference_Type_equipement_Etat_Port);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Preference_Type_equipement_Etat_Port), checkbutton_Preference_Type_equipement_Etat_Port);

	frame_Preference_Type_equipement_Etat_Equipement = gtk_frame_new (NULL);
	gtk_widget_show (frame_Preference_Type_equipement_Etat_Equipement);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement), frame_Preference_Type_equipement_Etat_Equipement, 160, 56);
	gtk_widget_set_size_request (frame_Preference_Type_equipement_Etat_Equipement, 304, 153);

	fixed_Preference_Type_equipement_Etat_Equipement = gtk_fixed_new ();
	gtk_widget_show (fixed_Preference_Type_equipement_Etat_Equipement);
	gtk_container_add (GTK_CONTAINER (frame_Preference_Type_equipement_Etat_Equipement), fixed_Preference_Type_equipement_Etat_Equipement);

	frame_Preference_Type_equipement_Etat_Equipement = gtk_frame_new (NULL);
	gtk_widget_show (frame_Preference_Type_equipement_Etat_Equipement);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement_Etat_Equipement), frame_Preference_Type_equipement_Etat_Equipement, 16, 22);
	gtk_widget_set_size_request (frame_Preference_Type_equipement_Etat_Equipement, 280, 101);

	fixed_Preference_Type_equipement_Etat_Equipement_Actions = gtk_fixed_new ();
	gtk_widget_show (fixed_Preference_Type_equipement_Etat_Equipement_Actions);
	gtk_container_add (GTK_CONTAINER (frame_Preference_Type_equipement_Etat_Equipement), fixed_Preference_Type_equipement_Etat_Equipement_Actions);

	entry_Preference_Type_equipement_Etat_Equipement_Actions_Email = gtk_entry_new ();
	gtk_widget_show (entry_Preference_Type_equipement_Etat_Equipement_Actions_Email);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement_Etat_Equipement_Actions), entry_Preference_Type_equipement_Etat_Equipement_Actions_Email, 80, 0);
	gtk_widget_set_size_request (entry_Preference_Type_equipement_Etat_Equipement_Actions_Email, 188, 21);

	entry_Preference_Type_equipement_Etat_Equipement_Actions_Script = gtk_entry_new ();
	gtk_widget_show (entry_Preference_Type_equipement_Etat_Equipement_Actions_Script);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement_Etat_Equipement_Actions), entry_Preference_Type_equipement_Etat_Equipement_Actions_Script, 80, 48);
	gtk_widget_set_size_request (entry_Preference_Type_equipement_Etat_Equipement_Actions_Script, 188, 21);

	checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Email = gtk_check_button_new_with_mnemonic ("Email");
	gtk_widget_show (checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Email);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement_Etat_Equipement_Actions), checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Email, 16, 0);
	gtk_widget_set_size_request (checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Email, 136, 24);

	checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Message = gtk_check_button_new_with_mnemonic ("Message");
	gtk_widget_show (checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Message);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement_Etat_Equipement_Actions), checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Message, 16, 24);
	gtk_widget_set_size_request (checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Message, 98, 23);

	checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Script = gtk_check_button_new_with_mnemonic ("Script");
	gtk_widget_show (checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Script);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement_Etat_Equipement_Actions), checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Script, 16, 48);
	gtk_widget_set_size_request (checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Script, 98, 23);

	label_Preference_Type_equipement_Etat_Equipement_Actions = gtk_label_new ("Actions");
	gtk_widget_show (label_Preference_Type_equipement_Etat_Equipement_Actions);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Preference_Type_equipement_Etat_Equipement), label_Preference_Type_equipement_Etat_Equipement_Actions);

	checkbutton_Preference_Type_equipement_Etat_Equipement_Down = gtk_check_button_new_with_mnemonic ("Poll even when it is down");
	gtk_widget_show (checkbutton_Preference_Type_equipement_Etat_Equipement_Down);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement_Etat_Equipement), checkbutton_Preference_Type_equipement_Etat_Equipement_Down, 16, 0);
	gtk_widget_set_size_request (checkbutton_Preference_Type_equipement_Etat_Equipement_Down, 168, 24);

	checkbutton_Preference_Type_equipement_Etat_Equipement = gtk_check_button_new_with_mnemonic ("Detect the state change of the selected equipment");
	gtk_widget_show (checkbutton_Preference_Type_equipement_Etat_Equipement);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Preference_Type_equipement_Etat_Equipement), checkbutton_Preference_Type_equipement_Etat_Equipement);

	label_Preference_Type_equipement_Type = gtk_label_new ("Type :");
	gtk_widget_show (label_Preference_Type_equipement_Type);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement), label_Preference_Type_equipement_Type, 160, 20);
	gtk_widget_set_size_request (label_Preference_Type_equipement_Type, 40, 16);

	iconentry_Preference_Type_equipement = gnome_icon_entry_new (NULL, NULL);
	gtk_widget_show (iconentry_Preference_Type_equipement);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement), iconentry_Preference_Type_equipement, 392, 4);
	gtk_widget_set_size_request (iconentry_Preference_Type_equipement, 72, 50);

	button_Preference_Type_equipement_Ajouter = gtk_button_new ();
	gtk_widget_show (button_Preference_Type_equipement_Ajouter);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement), button_Preference_Type_equipement_Ajouter, 472, 80);
	gtk_widget_set_size_request (button_Preference_Type_equipement_Ajouter, 104, 32);

	alignment_Preference_Type_equipement_Ajouter = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Preference_Type_equipement_Ajouter);
	gtk_container_add (GTK_CONTAINER (button_Preference_Type_equipement_Ajouter), alignment_Preference_Type_equipement_Ajouter);

	hbox_Preference_Type_equipement_Ajouter = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Preference_Type_equipement_Ajouter);
	gtk_container_add (GTK_CONTAINER (alignment_Preference_Type_equipement_Ajouter), hbox_Preference_Type_equipement_Ajouter);

	image_Preference_Type_equipement_Ajouter = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/filenew.png");
	gtk_widget_show (image_Preference_Type_equipement_Ajouter);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Type_equipement_Ajouter), image_Preference_Type_equipement_Ajouter, FALSE, FALSE, 0);

	label_Preference_Type_equipement_Ajouter = gtk_label_new_with_mnemonic ("Add");
	gtk_widget_show (label_Preference_Type_equipement_Ajouter);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Type_equipement_Ajouter), label_Preference_Type_equipement_Ajouter, FALSE, FALSE, 0);

	button_Preference_Type_equipement_Enlever = gtk_button_new ();
	gtk_widget_show (button_Preference_Type_equipement_Enlever);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement), button_Preference_Type_equipement_Enlever, 472, 120);
	gtk_widget_set_size_request (button_Preference_Type_equipement_Enlever, 104, 32);

	alignment_Preference_Type_equipement_Enlever = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Preference_Type_equipement_Enlever);
	gtk_container_add (GTK_CONTAINER (button_Preference_Type_equipement_Enlever), alignment_Preference_Type_equipement_Enlever);

	hbox_Preference_Type_equipement_Enlever = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Preference_Type_equipement_Enlever);
	gtk_container_add (GTK_CONTAINER (alignment_Preference_Type_equipement_Enlever), hbox_Preference_Type_equipement_Enlever);

	image_Preference_Type_equipement_Enlever = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/editcut.png");
	gtk_widget_show (image_Preference_Type_equipement_Enlever);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Type_equipement_Enlever), image_Preference_Type_equipement_Enlever, FALSE, FALSE, 0);

	label_Preference_Type_equipement_Enlever = gtk_label_new_with_mnemonic ("Delete");
	gtk_widget_show (label_Preference_Type_equipement_Enlever);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Type_equipement_Enlever), label_Preference_Type_equipement_Enlever, FALSE, FALSE, 0);

	button_Preference_Type_equipement_Valider = gtk_button_new ();
	gtk_widget_show (button_Preference_Type_equipement_Valider);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement), button_Preference_Type_equipement_Valider, 472, 240);
	gtk_widget_set_size_request (button_Preference_Type_equipement_Valider, 104, 32);

	alignment_Preference_Type_equipement_Valider = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Preference_Type_equipement_Valider);
	gtk_container_add (GTK_CONTAINER (button_Preference_Type_equipement_Valider), alignment_Preference_Type_equipement_Valider);

	hbox_Preference_Type_equipement_Valider = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Preference_Type_equipement_Valider);
	gtk_container_add (GTK_CONTAINER (alignment_Preference_Type_equipement_Valider), hbox_Preference_Type_equipement_Valider);

	image_Preference_Type_equipement_Valider = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/button_ok.png");
	gtk_widget_show (image_Preference_Type_equipement_Valider);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Type_equipement_Valider), image_Preference_Type_equipement_Valider, FALSE, FALSE, 0);

	label_Preference_Type_equipement_Valider = gtk_label_new_with_mnemonic ("Ok");
	gtk_widget_show (label_Preference_Type_equipement_Valider);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Type_equipement_Valider), label_Preference_Type_equipement_Valider, FALSE, FALSE, 0);

	button_Preference_Type_equipement_Annuler = gtk_button_new ();
	gtk_widget_show (button_Preference_Type_equipement_Annuler);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Type_equipement), button_Preference_Type_equipement_Annuler, 472, 280);
	gtk_widget_set_size_request (button_Preference_Type_equipement_Annuler, 104, 32);

	alignment_Preference_Type_equipement_Annuler = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Preference_Type_equipement_Annuler);
	gtk_container_add (GTK_CONTAINER (button_Preference_Type_equipement_Annuler), alignment_Preference_Type_equipement_Annuler);

	hbox_Preference_Type_equipement_Annuler = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Preference_Type_equipement_Annuler);
	gtk_container_add (GTK_CONTAINER (alignment_Preference_Type_equipement_Annuler), hbox_Preference_Type_equipement_Annuler);

	image_Preference_Type_equipement_Annuler = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/button_cancel.png");
	gtk_widget_show (image_Preference_Type_equipement_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Type_equipement_Annuler), image_Preference_Type_equipement_Annuler, FALSE, FALSE, 0);

	label_Preference_Type_equipement_Annuler = gtk_label_new_with_mnemonic ("Cancel");
	gtk_widget_show (label_Preference_Type_equipement_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Type_equipement_Annuler), label_Preference_Type_equipement_Annuler, FALSE, FALSE, 0);

	fixed_Menu_Type_Equipement = gtk_fixed_new ();
	gtk_widget_show (fixed_Menu_Type_Equipement);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_Preference), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook_Preference), 1), fixed_Menu_Type_Equipement);
	gtk_widget_set_size_request (fixed_Menu_Type_Equipement, 107, 36);

	image_Menu_Type_Equipement = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/view_tree.png");
	gtk_widget_show (image_Menu_Type_Equipement);
	gtk_fixed_put (GTK_FIXED (fixed_Menu_Type_Equipement), image_Menu_Type_Equipement, 2, 2);
	gtk_widget_set_size_request (image_Menu_Type_Equipement, 32, 32);

	label_Menu_Type_Equipement = gtk_label_new ("Equipment \n      type");
	gtk_widget_show (label_Menu_Type_Equipement);
	gtk_fixed_put (GTK_FIXED (fixed_Menu_Type_Equipement), label_Menu_Type_Equipement, 32, 2);
	gtk_widget_set_size_request (label_Menu_Type_Equipement, 80, 32);

	fixed_Preference_Notification = gtk_fixed_new ();
	//gtk_widget_show (fixed_Preference_Notification);
	gtk_container_add (GTK_CONTAINER (notebook_Preference), fixed_Preference_Notification);

	frame_Preference_Notification_Nouveau_Equipement = gtk_frame_new (NULL);
	gtk_widget_show (frame_Preference_Notification_Nouveau_Equipement);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification), frame_Preference_Notification_Nouveau_Equipement, 8, 8);
	gtk_widget_set_size_request (frame_Preference_Notification_Nouveau_Equipement, 552, 80);

	fixed_Preference_Notification_Nouveau_Equipement = gtk_fixed_new ();
	gtk_widget_show (fixed_Preference_Notification_Nouveau_Equipement);
	gtk_container_add (GTK_CONTAINER (frame_Preference_Notification_Nouveau_Equipement), fixed_Preference_Notification_Nouveau_Equipement);

	entry_Preference_Notification_Nouveau_Equipement_Email = gtk_entry_new ();
	gtk_widget_show (entry_Preference_Notification_Nouveau_Equipement_Email);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Nouveau_Equipement), entry_Preference_Notification_Nouveau_Equipement_Email, 96, 24);
	gtk_widget_set_size_request (entry_Preference_Notification_Nouveau_Equipement_Email, 158, 21);

	checkbutton_Preference_Notification_Nouveau_Equipement_Message = gtk_check_button_new_with_mnemonic ("Message");
	gtk_widget_show (checkbutton_Preference_Notification_Nouveau_Equipement_Message);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Nouveau_Equipement), checkbutton_Preference_Notification_Nouveau_Equipement_Message, 24, 0);
	gtk_widget_set_size_request (checkbutton_Preference_Notification_Nouveau_Equipement_Message, 103, 23);

	checkbutton_Preference_Notification_Nouveau_Equipement_Email = gtk_check_button_new_with_mnemonic ("Email");
	gtk_widget_show (checkbutton_Preference_Notification_Nouveau_Equipement_Email);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Nouveau_Equipement), checkbutton_Preference_Notification_Nouveau_Equipement_Email, 24, 24);
	gtk_widget_set_size_request (checkbutton_Preference_Notification_Nouveau_Equipement_Email, 72, 24);

	checkbutton_Preference_Notification_Nouveau_Equipement = gtk_check_button_new_with_mnemonic ("Action when a new equipment is discovered");
	gtk_widget_show (checkbutton_Preference_Notification_Nouveau_Equipement);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Preference_Notification_Nouveau_Equipement), checkbutton_Preference_Notification_Nouveau_Equipement);

	frame_Preference_Notification_Etat_Equipement = gtk_frame_new (NULL);
	gtk_widget_show (frame_Preference_Notification_Etat_Equipement);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification), frame_Preference_Notification_Etat_Equipement, 8, 120);
	gtk_widget_set_size_request (frame_Preference_Notification_Etat_Equipement, 552, 80);

	fixed_Preference_Notification_Etat_Equipement = gtk_fixed_new ();
	gtk_widget_show (fixed_Preference_Notification_Etat_Equipement);
	gtk_container_add (GTK_CONTAINER (frame_Preference_Notification_Etat_Equipement), fixed_Preference_Notification_Etat_Equipement);

	spinbutton_Preference_Notification_Etat_Equipement_Up_Secondes_adj = gtk_adjustment_new (30, 1, 1000, 1, 10, 10);
	spinbutton_Preference_Notification_Etat_Equipement_Up_Secondes = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_Preference_Notification_Etat_Equipement_Up_Secondes_adj), 1, 0);
	gtk_widget_show (spinbutton_Preference_Notification_Etat_Equipement_Up_Secondes);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Etat_Equipement), spinbutton_Preference_Notification_Etat_Equipement_Up_Secondes, 144, 8);
	gtk_widget_set_size_request (spinbutton_Preference_Notification_Etat_Equipement_Up_Secondes, 48, 21);

	spinbutton_Preference_Notification_Etat_Equipement_Down_Secondes_adj = gtk_adjustment_new (30, 1, 1000, 1, 10, 10);
	spinbutton_Preference_Notification_Etat_Equipement_Down_Secondes = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_Preference_Notification_Etat_Equipement_Down_Secondes_adj), 1, 0);
	gtk_widget_show (spinbutton_Preference_Notification_Etat_Equipement_Down_Secondes);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Etat_Equipement), spinbutton_Preference_Notification_Etat_Equipement_Down_Secondes, 144, 32);
	gtk_widget_set_size_request (spinbutton_Preference_Notification_Etat_Equipement_Down_Secondes, 48, 21);

	label_Preference_Notification_Etat_Equipement_Test_Up_Secondes = gtk_label_new ("secondes");
	gtk_widget_show (label_Preference_Notification_Etat_Equipement_Test_Up_Secondes);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Etat_Equipement), label_Preference_Notification_Etat_Equipement_Test_Up_Secondes, 208, 8);
	gtk_widget_set_size_request (label_Preference_Notification_Etat_Equipement_Test_Up_Secondes, 72, 16);

	label_Preference_Notification_Etat_Equipement_Test_Up_Secondes = gtk_label_new ("secondes");
	gtk_widget_show (label_Preference_Notification_Etat_Equipement_Test_Up_Secondes);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Etat_Equipement), label_Preference_Notification_Etat_Equipement_Test_Up_Secondes, 216, 32);
	gtk_widget_set_size_request (label_Preference_Notification_Etat_Equipement_Test_Up_Secondes, 56, 16);

	label_Preference_Notification_Etat_Equipement_Test_Up = gtk_label_new ("When it is up :");
	gtk_widget_show (label_Preference_Notification_Etat_Equipement_Test_Up);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Etat_Equipement), label_Preference_Notification_Etat_Equipement_Test_Up, 24, 8);
	gtk_widget_set_size_request (label_Preference_Notification_Etat_Equipement_Test_Up, 96, 16);

	label_Preference_Notification_Etat_Equipement_Down = gtk_label_new ("When it is down");
	gtk_widget_show (label_Preference_Notification_Etat_Equipement_Down);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Etat_Equipement), label_Preference_Notification_Etat_Equipement_Down, 24, 32);
	gtk_widget_set_size_request (label_Preference_Notification_Etat_Equipement_Down, 104, 16);

	label_Preference_Notification_Etat_Equipement = gtk_label_new ("Poll the state change of the equipments");
	gtk_widget_show (label_Preference_Notification_Etat_Equipement);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Preference_Notification_Etat_Equipement), label_Preference_Notification_Etat_Equipement);

	frame_Preference_Notification_Etat_Port = gtk_frame_new (NULL);
	gtk_widget_show (frame_Preference_Notification_Etat_Port);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification), frame_Preference_Notification_Etat_Port, 8, 208);
	gtk_widget_set_size_request (frame_Preference_Notification_Etat_Port, 552, 88);

	fixed_Preference_Notification_Etat_Port = gtk_fixed_new ();
	gtk_widget_show (fixed_Preference_Notification_Etat_Port);
	gtk_container_add (GTK_CONTAINER (frame_Preference_Notification_Etat_Port), fixed_Preference_Notification_Etat_Port);

	spinbutton_Preference_Notification_Etat_Port_Up_Secondes_adj = gtk_adjustment_new (120, 1, 1000, 1, 10, 10);
	spinbutton_Preference_Notification_Etat_Port_Up_Secondes = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_Preference_Notification_Etat_Port_Up_Secondes_adj), 1, 0);
	gtk_widget_show (spinbutton_Preference_Notification_Etat_Port_Up_Secondes);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Etat_Port), spinbutton_Preference_Notification_Etat_Port_Up_Secondes, 144, 8);
	gtk_widget_set_size_request (spinbutton_Preference_Notification_Etat_Port_Up_Secondes, 48, 21);

	spinbutton_Preference_Notification_Etat_Port_Down_Secondes_adj = gtk_adjustment_new (120, 1, 1000, 1, 10, 10);
	spinbutton_Preference_Notification_Etat_Port_Down_Secondes = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_Preference_Notification_Etat_Port_Down_Secondes_adj), 1, 0);
	gtk_widget_show (spinbutton_Preference_Notification_Etat_Port_Down_Secondes);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Etat_Port), spinbutton_Preference_Notification_Etat_Port_Down_Secondes, 144, 32);
	gtk_widget_set_size_request (spinbutton_Preference_Notification_Etat_Port_Down_Secondes, 48, 21);

	label_Preference_Notification_Etat_Port_Test_Up = gtk_label_new ("secondes");
	gtk_widget_show (label_Preference_Notification_Etat_Port_Test_Up);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Etat_Port), label_Preference_Notification_Etat_Port_Test_Up, 208, 8);
	gtk_widget_set_size_request (label_Preference_Notification_Etat_Port_Test_Up, 72, 16);

	label_Preference_Notification_Etat_Port_Test_Down = gtk_label_new ("secondes");
	gtk_widget_show (label_Preference_Notification_Etat_Port_Test_Down);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Etat_Port), label_Preference_Notification_Etat_Port_Test_Down, 216, 32);
	gtk_widget_set_size_request (label_Preference_Notification_Etat_Port_Test_Down, 56, 16);

	label_Preference_Notification_Etat_Port_Test_Up = gtk_label_new ("When it is open :");
	gtk_widget_show (label_Preference_Notification_Etat_Port_Test_Up);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Etat_Port), label_Preference_Notification_Etat_Port_Test_Up, 16, 8);
	gtk_widget_set_size_request (label_Preference_Notification_Etat_Port_Test_Up, 112, 16);

	label_Preference_Notification_Etat_Port_Test_Down = gtk_label_new ("When it is close");
	gtk_widget_show (label_Preference_Notification_Etat_Port_Test_Down);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification_Etat_Port), label_Preference_Notification_Etat_Port_Test_Down, 16, 32);
	gtk_widget_set_size_request (label_Preference_Notification_Etat_Port_Test_Down, 112, 16);

	label_Preference_Notification_Etat_Port = gtk_label_new ("Poll the state change of the ports");
	gtk_widget_show (label_Preference_Notification_Etat_Port);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Preference_Notification_Etat_Port), label_Preference_Notification_Etat_Port);

	checkbutton_Preference_Notification_Etat_NBNS = gtk_check_button_new_with_mnemonic ("Takes into account the messages \"NBNS Release\"");
	gtk_widget_show (checkbutton_Preference_Notification_Etat_NBNS);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Notification), checkbutton_Preference_Notification_Etat_NBNS, 8, 96);
	gtk_widget_set_size_request (checkbutton_Preference_Notification_Etat_NBNS, 305, 16);

	fixed_Menu_Notifications = gtk_fixed_new ();
	gtk_widget_show (fixed_Menu_Notifications);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_Preference), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook_Preference), 2), fixed_Menu_Notifications);
	gtk_widget_set_size_request (fixed_Menu_Notifications, 107, 36);

	image_Menu_Notifications = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/Notifications.png");
	gtk_widget_show (image_Menu_Notifications);
	gtk_fixed_put (GTK_FIXED (fixed_Menu_Notifications), image_Menu_Notifications, 0, 0);
	gtk_widget_set_size_request (image_Menu_Notifications, 32, 32);

	label_Menu_Notifications = gtk_label_new ("Notifications");
	gtk_widget_show (label_Menu_Notifications);
	gtk_fixed_put (GTK_FIXED (fixed_Menu_Notifications), label_Menu_Notifications, 32, 9);
	gtk_widget_set_size_request (label_Menu_Notifications, 72, 16);

	fixed_Preference_Preference = gtk_fixed_new ();
	//gtk_widget_show (fixed_Preference_Preference);
	gtk_container_add (GTK_CONTAINER (notebook_Preference), fixed_Preference_Preference);

	entry_Preference_Preference_Shell = gtk_entry_new ();
	gtk_widget_show (entry_Preference_Preference_Shell);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Preference), entry_Preference_Preference_Shell, 104, 16);
	gtk_widget_set_size_request (entry_Preference_Preference_Shell, 344, 21);

	entry_Preference_Preference_Samba = gtk_entry_new ();
	gtk_widget_show (entry_Preference_Preference_Samba);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Preference), entry_Preference_Preference_Samba, 104, 48);
	gtk_widget_set_size_request (entry_Preference_Preference_Samba, 344, 21);

	entry_Preference_Preference_Web = gtk_entry_new ();
	gtk_widget_show (entry_Preference_Preference_Web);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Preference), entry_Preference_Preference_Web, 104, 80);
	gtk_widget_set_size_request (entry_Preference_Preference_Web, 344, 21);

	label_Preference_Preference_Samba = gtk_label_new ("Samba browser :");
	gtk_widget_show (label_Preference_Preference_Samba);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Preference), label_Preference_Preference_Samba, 8, 48);
	gtk_widget_set_size_request (label_Preference_Preference_Samba, 96, 16);

	label_Preference_Preference_Shell = gtk_label_new ("Shell : ");
	gtk_widget_show (label_Preference_Preference_Shell);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Preference), label_Preference_Preference_Shell, 56, 16);
	gtk_widget_set_size_request (label_Preference_Preference_Shell, 40, 16);

	label_Preference_Preference_Web = gtk_label_new ("web browser :");
	gtk_widget_show (label_Preference_Preference_Web);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Preference), label_Preference_Preference_Web, 24, 80);
	gtk_widget_set_size_request (label_Preference_Preference_Web, 88, 16);

	label_Preference_Preference_Smtp = gtk_label_new ("Smtp :");
	gtk_widget_show (label_Preference_Preference_Smtp);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Preference), label_Preference_Preference_Smtp, 56, 112);
	gtk_widget_set_size_request (label_Preference_Preference_Smtp, 41, 16);

	entry_Preference_Preference_Smtp = gtk_entry_new ();
	gtk_widget_show (entry_Preference_Preference_Smtp);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Preference), entry_Preference_Preference_Smtp, 104, 112);
	gtk_widget_set_size_request (entry_Preference_Preference_Smtp, 344, 21);

	fixed_Menu_Preferences = gtk_fixed_new ();
	gtk_widget_show (fixed_Menu_Preferences);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_Preference), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook_Preference), 3), fixed_Menu_Preferences);
	gtk_widget_set_size_request (fixed_Menu_Preferences, 107, 36);

	image_Menu_Preferences = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/kcontrol.png");
	gtk_widget_show (image_Menu_Preferences);
	gtk_fixed_put (GTK_FIXED (fixed_Menu_Preferences), image_Menu_Preferences, 0, 0);
	gtk_widget_set_size_request (image_Menu_Preferences, 32, 32);

	label_Menu_Preferences = gtk_label_new ("Preferences");
	gtk_widget_show (label_Menu_Preferences);
	gtk_fixed_put (GTK_FIXED (fixed_Menu_Preferences), label_Menu_Preferences, 40, 9);
	gtk_widget_set_size_request (label_Menu_Preferences, 64, 16);

	fixed_Preference_Extensions = gtk_fixed_new ();
	//gtk_widget_show (fixed_Preference_Extensions);
	gtk_container_add (GTK_CONTAINER (notebook_Preference), fixed_Preference_Extensions);

	ctree_Preference_Extensions = gtk_ctree_new (1, 0);
	gtk_widget_show (ctree_Preference_Extensions);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Extensions), ctree_Preference_Extensions, 40, 40);
	gtk_widget_set_size_request (ctree_Preference_Extensions, 504, 280);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Preference_Extensions), 0, 80);
	gtk_clist_column_titles_hide (GTK_CLIST (ctree_Preference_Extensions));

	label_Preference_Extensions = gtk_label_new ("The following Extensions are installed :");
	gtk_widget_show (label_Preference_Extensions);
	gtk_fixed_put (GTK_FIXED (fixed_Preference_Extensions), label_Preference_Extensions, 16, 16);
	gtk_widget_set_size_request (label_Preference_Extensions, 208, 16);

	fixed_Menu_Extensions = gtk_fixed_new ();
	gtk_widget_show (fixed_Menu_Extensions);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_Preference), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook_Preference), 4), fixed_Menu_Extensions);
	gtk_widget_set_size_request (fixed_Menu_Extensions, 107, 36);

	image_Menu_Extensions = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/Extensions.png");
	gtk_widget_show (image_Menu_Extensions);
	gtk_fixed_put (GTK_FIXED (fixed_Menu_Extensions), image_Menu_Extensions, 0, 0);
	gtk_widget_set_size_request (image_Menu_Extensions, 32, 32);

	label_Menu_Extensions = gtk_label_new ("Extensions");
	gtk_widget_show (label_Menu_Extensions);
	gtk_fixed_put (GTK_FIXED (fixed_Menu_Extensions), label_Menu_Extensions, 40, 9);
	gtk_widget_set_size_request (label_Menu_Extensions, 60, 16);

	button_Preference_Enregistrer = gtk_button_new ();
	gtk_widget_show (button_Preference_Enregistrer);
	gtk_fixed_put (GTK_FIXED (fixed_Preference), button_Preference_Enregistrer, 448, 376);
	gtk_widget_set_size_request (button_Preference_Enregistrer, 104, 36);

	alignment_Preference_Enregistrer = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Preference_Enregistrer);
	gtk_container_add (GTK_CONTAINER (button_Preference_Enregistrer), alignment_Preference_Enregistrer);

	hbox_Preference_Enregistrer = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Preference_Enregistrer);
	gtk_container_add (GTK_CONTAINER (alignment_Preference_Enregistrer), hbox_Preference_Enregistrer);

	image_Preference_Enregistrer = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/filesave.png");
	gtk_widget_show (image_Preference_Enregistrer);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Enregistrer), image_Preference_Enregistrer, FALSE, FALSE, 0);

	label_Preference_Enregistrer = gtk_label_new_with_mnemonic ("Save");
	gtk_widget_show (label_Preference_Enregistrer);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Enregistrer), label_Preference_Enregistrer, FALSE, FALSE, 0);

	button_Preference_Annuler = gtk_button_new ();
	gtk_widget_show (button_Preference_Annuler);
	gtk_fixed_put (GTK_FIXED (fixed_Preference), button_Preference_Annuler, 568, 376);
	gtk_widget_set_size_request (button_Preference_Annuler, 104, 36);

	alignment_Preference_Annuler = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Preference_Annuler);
	gtk_container_add (GTK_CONTAINER (button_Preference_Annuler), alignment_Preference_Annuler);

	hbox_Preference_Annuler = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Preference_Annuler);
	gtk_container_add (GTK_CONTAINER (alignment_Preference_Annuler), hbox_Preference_Annuler);

	image_Preference_Annuler = create_pixmap (window_Preference, "/usr/share/pixmaps/AutoScan/button_cancel.png");
	gtk_widget_show (image_Preference_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Annuler), image_Preference_Annuler, FALSE, FALSE, 0);

	label_Preference_Annuler = gtk_label_new_with_mnemonic ("Cancel");
	gtk_widget_show (label_Preference_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Preference_Annuler), label_Preference_Annuler, FALSE, FALSE, 0);

	g_signal_connect ((gpointer) window_Preference, "destroy", G_CALLBACK (on_window_Preference_destroy), NULL);
	g_signal_connect ((gpointer) ctree_Preference_Sous_reseau_privee, "tree_select_row", G_CALLBACK (on_ctree_Preference_Sous_reseau_privee_tree_select_row), NULL);
	g_signal_connect ((gpointer) combo_entry_Preference_Sous_reseau_privee, "changed", G_CALLBACK (Preference_Sous_reseau_privee_changed), NULL);
	g_signal_connect ((gpointer) combo_entry_Preference_Sous_reseau_privee, "changed", G_CALLBACK (on_combo_entry_Preference_Sous_reseau_privee_changed), NULL);
	g_signal_connect ((gpointer) entry_Preference_Sous_reseau_privee_1, "changed", G_CALLBACK (Preference_Sous_reseau_privee_changed), NULL);
	g_signal_connect ((gpointer) entry_Preference_Sous_reseau_privee_2, "changed", G_CALLBACK (Preference_Sous_reseau_privee_changed), NULL);
	g_signal_connect ((gpointer) entry_Preference_Sous_reseau_privee_3, "changed", G_CALLBACK (Preference_Sous_reseau_privee_changed), NULL);
	g_signal_connect ((gpointer) button_Preference_Sous_reseau_privee_Enlever, "clicked", G_CALLBACK (on_button_Preference_Sous_reseau_privee_Enlever_clicked), NULL);
	g_signal_connect ((gpointer) button_Preference_Sous_reseau_privee_Ok, "clicked", G_CALLBACK (on_button_Preference_Sous_reseau_privee_Ok_clicked), NULL);
	g_signal_connect ((gpointer) button_Preference_Sous_reseau_privee_Annuler, "clicked", G_CALLBACK (on_button_Preference_Sous_reseau_privee_Annuler_clicked), NULL);
	g_signal_connect ((gpointer) button_Preference_Sous_reseau_privee_Ajouter, "clicked", G_CALLBACK (on_button_Preference_Sous_reseau_privee_Ajouter_clicked), NULL);
	g_signal_connect ((gpointer) ctree_Preference_Type_equipement, "tree_select_row", G_CALLBACK (on_ctree_Preference_Type_equipement_tree_select_row), NULL);
	g_signal_connect ((gpointer) entry_Preference_Type_equipement_Type, "changed", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) entry_Preference_Type_equipement_Etat_Port_Actions_Email, "changed", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) entry_Preference_Type_equipement_Etat_Port_Actions_Script, "changed", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Port_Actions_Email, "clicked", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Port_Actions_Email, "clicked", G_CALLBACK (on_checkbutton_Preference_Type_equipement_Etat_Port_Actions_Email_clicked), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Port_Actions_Script, "clicked", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Port_Actions_Script, "clicked", G_CALLBACK (on_checkbutton_Preference_Type_equipement_Etat_Port_Actions_Script_clicked), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Port_Actions_Message, "clicked", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Port_Down, "clicked", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Port, "clicked", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Port, "clicked", G_CALLBACK (on_checkbutton_Preference_Type_equipement_Etat_Port_clicked), NULL);
	g_signal_connect ((gpointer) entry_Preference_Type_equipement_Etat_Equipement_Actions_Email, "changed", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) entry_Preference_Type_equipement_Etat_Equipement_Actions_Script, "changed", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Email, "clicked", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Email, "clicked", G_CALLBACK (on_checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Email_clicked), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Message, "clicked", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Script, "clicked", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Script, "clicked", G_CALLBACK (on_checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Script_clicked), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Equipement_Down, "clicked", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Equipement, "clicked", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Type_equipement_Etat_Equipement, "clicked", G_CALLBACK (on_checkbutton_Preference_Type_equipement_Etat_Equipement_clicked), NULL);
	g_signal_connect ((gpointer) iconentry_Preference_Type_equipement, "changed", G_CALLBACK (Liste_Type_Equipement_changed), NULL);
	g_signal_connect ((gpointer) button_Preference_Type_equipement_Ajouter, "clicked", G_CALLBACK (on_button_Preference_Type_equipement_Ajouter_clicked), NULL);
	g_signal_connect ((gpointer) button_Preference_Type_equipement_Enlever, "clicked", G_CALLBACK (on_button_Preference_Type_equipement_Enlever_clicked), NULL);
	g_signal_connect ((gpointer) button_Preference_Type_equipement_Valider, "clicked", G_CALLBACK (on_button_Preference_Type_equipement_Valider_clicked), NULL);
	g_signal_connect ((gpointer) button_Preference_Type_equipement_Annuler, "clicked", G_CALLBACK (on_button_Preference_Type_equipement_Annuler_clicked), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Notification_Nouveau_Equipement_Email, "clicked", G_CALLBACK (on_checkbutton_Preference_Notification_Nouveau_Equipement_Email_clicked), NULL);
	g_signal_connect ((gpointer) checkbutton_Preference_Notification_Nouveau_Equipement, "clicked", G_CALLBACK (on_checkbutton_Preference_Notification_Nouveau_Equipement_clicked), NULL);
	g_signal_connect ((gpointer) button_Preference_Enregistrer, "clicked", G_CALLBACK (on_button_Preference_Enregistrer_clicked), NULL);
	g_signal_connect ((gpointer) button_Preference_Annuler, "clicked", G_CALLBACK (on_button_Preference_Annuler_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_Preference, window_Preference, "window_Preference");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Preference, "fixed_Preference");
	GLADE_HOOKUP_OBJECT (window_Preference, notebook_Preference, "notebook_Preference");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Preference_Interface, "fixed_Preference_Interface");
	GLADE_HOOKUP_OBJECT (window_Preference, spinbutton_Preference_Interface_Max_Thread_Scan, "spinbutton_Preference_Interface_Max_Thread_Scan");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Interface_Max_Thread_Scan, "label_Preference_Interface_Max_Thread_Scan");
	GLADE_HOOKUP_OBJECT (window_Preference, combo_Preference_Interface_Interface, "combo_Preference_Interface_Interface");
	GLADE_HOOKUP_OBJECT (window_Preference, combo_entry_Preference_Interface, "combo_entry_Preference_Interface");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Interface_Interface, "label_Preference_Interface_Interface");
	GLADE_HOOKUP_OBJECT (window_Preference, frame_Preference_Sous_reseau_privee, "frame_Preference_Sous_reseau_privee");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Preference_Sous_reseau_privee, "fixed_Preference_Sous_reseau_privee");
	GLADE_HOOKUP_OBJECT (window_Preference, scrolledwindow_Preference_Sous_reseau_privee, "scrolledwindow_Preference_Sous_reseau_privee");
	GLADE_HOOKUP_OBJECT (window_Preference, ctree_Preference_Sous_reseau_privee, "ctree_Preference_Sous_reseau_privee");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Sous_reseau_privee_Mask, "label_Preference_Sous_reseau_privee_Mask");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Sous_reseau_privee_Sous_reseau, "label_Preference_Sous_reseau_privee_Sous_reseau");
	GLADE_HOOKUP_OBJECT (window_Preference, combo_Preference_Sous_reseau_privee, "combo_Preference_Sous_reseau_privee");
	GLADE_HOOKUP_OBJECT (window_Preference, combo_entry_Preference_Sous_reseau_privee, "combo_entry_Preference_Sous_reseau_privee");
	GLADE_HOOKUP_OBJECT (window_Preference, entry_Preference_Sous_reseau_privee_1, "entry_Preference_Sous_reseau_privee_1");
	GLADE_HOOKUP_OBJECT (window_Preference, entry_Preference_Sous_reseau_privee_2, "entry_Preference_Sous_reseau_privee_2");
	GLADE_HOOKUP_OBJECT (window_Preference, entry_Preference_Sous_reseau_privee_3, "entry_Preference_Sous_reseau_privee_3");
	GLADE_HOOKUP_OBJECT (window_Preference, button_Preference_Sous_reseau_privee_Enlever, "button_Preference_Sous_reseau_privee_Enlever");
	GLADE_HOOKUP_OBJECT (window_Preference, alignment_Preference_Sous_reseau_privee_Enlever, "alignment_Preference_Sous_reseau_privee_Enlever");
	GLADE_HOOKUP_OBJECT (window_Preference, hbox_Preference_Sous_reseau_privee_Enlever, "hbox_Preference_Sous_reseau_privee_Enlever");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Preference_Sous_reseau_privee_Enlever, "image_Preference_Sous_reseau_privee_Enlever");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Sous_reseau_privee_Enlever, "label_Preference_Sous_reseau_privee_Enlever");
	GLADE_HOOKUP_OBJECT (window_Preference, button_Preference_Sous_reseau_privee_Ok, "button_Preference_Sous_reseau_privee_Ok");
	GLADE_HOOKUP_OBJECT (window_Preference, alignment_Preference_Sous_reseau_privee_Ok, "alignment_Preference_Sous_reseau_privee_Ok");
	GLADE_HOOKUP_OBJECT (window_Preference, hbox_Preference_Sous_reseau_privee_Ok, "hbox_Preference_Sous_reseau_privee_Ok");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Preference_Sous_reseau_privee_Ok, "image_Preference_Sous_reseau_privee_Ok");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Sous_reseau_privee_Ok, "label_Preference_Sous_reseau_privee_Ok");
	GLADE_HOOKUP_OBJECT (window_Preference, button_Preference_Sous_reseau_privee_Annuler, "button_Preference_Sous_reseau_privee_Annuler");
	GLADE_HOOKUP_OBJECT (window_Preference, alignment_Preference_Sous_reseau_privee_Annuler, "alignment_Preference_Sous_reseau_privee_Annuler");
	GLADE_HOOKUP_OBJECT (window_Preference, hbox_Preference_Sous_reseau_privee_Annuler, "hbox_Preference_Sous_reseau_privee_Annuler");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Preference_Sous_reseau_privee_Annuler, "image_Preference_Sous_reseau_privee_Annuler");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Sous_reseau_privee_Annuler, "label_Preference_Sous_reseau_privee_Annuler");
	GLADE_HOOKUP_OBJECT (window_Preference, button_Preference_Sous_reseau_privee_Ajouter, "button_Preference_Sous_reseau_privee_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Preference, alignment_Preference_Sous_reseau_privee_Ajouter, "alignment_Preference_Sous_reseau_privee_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Preference, hbox_Preference_Sous_reseau_privee_Ajouter, "hbox_Preference_Sous_reseau_privee_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Preference_Sous_reseau_privee_Ajouter, "image_Preference_Sous_reseau_privee_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Sous_reseau_privee_Ajouter, "label_Preference_Sous_reseau_privee_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Interface_Reseau_Prive, "label_Preference_Interface_Reseau_Prive");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Menu_Interfaces, "fixed_Menu_Interfaces");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Menu_Interfaces, "image_Menu_Interfaces");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Menu_Interfaces, "label_Menu_Interfaces");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Preference_Type_equipement, "fixed_Preference_Type_equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, scrolledwindow_Preference_Type_equipement, "scrolledwindow_Preference_Type_equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, ctree_Preference_Type_equipement, "ctree_Preference_Type_equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, entry_Preference_Type_equipement_Type, "entry_Preference_Type_equipement_Type");
	GLADE_HOOKUP_OBJECT (window_Preference, frame_Preference_Type_equipement_Etat_Port, "frame_Preference_Type_equipement_Etat_Port");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Preference_Type_equipement_Etat_Port, "fixed_Preference_Type_equipement_Etat_Port");
	GLADE_HOOKUP_OBJECT (window_Preference, frame_Preference_Type_equipement_Etat_Port_Actions, "frame_Preference_Type_equipement_Etat_Port_Actions");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Preference_Type_equipement_Etat_Port_Actions, "fixed_Preference_Type_equipement_Etat_Port_Actions");
	GLADE_HOOKUP_OBJECT (window_Preference, entry_Preference_Type_equipement_Etat_Port_Actions_Email, "entry_Preference_Type_equipement_Etat_Port_Actions_Email");
	GLADE_HOOKUP_OBJECT (window_Preference, entry_Preference_Type_equipement_Etat_Port_Actions_Script, "entry_Preference_Type_equipement_Etat_Port_Actions_Script");
	GLADE_HOOKUP_OBJECT (window_Preference, checkbutton_Preference_Type_equipement_Etat_Port_Actions_Email, "checkbutton_Preference_Type_equipement_Etat_Port_Actions_Email");
	GLADE_HOOKUP_OBJECT (window_Preference, checkbutton_Preference_Type_equipement_Etat_Port_Actions_Script, "checkbutton_Preference_Type_equipement_Etat_Port_Actions_Script");
	GLADE_HOOKUP_OBJECT (window_Preference, checkbutton_Preference_Type_equipement_Etat_Port_Actions_Message, "checkbutton_Preference_Type_equipement_Etat_Port_Actions_Message");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Type_equipement_Etat_Port_Actions, "label_Preference_Type_equipement_Etat_Port_Actions");
	GLADE_HOOKUP_OBJECT (window_Preference, checkbutton_Preference_Type_equipement_Etat_Port_Down, "checkbutton_Preference_Type_equipement_Etat_Port_Down");
	GLADE_HOOKUP_OBJECT (window_Preference, checkbutton_Preference_Type_equipement_Etat_Port, "checkbutton_Preference_Type_equipement_Etat_Port");
	GLADE_HOOKUP_OBJECT (window_Preference, frame_Preference_Type_equipement_Etat_Equipement, "frame_Preference_Type_equipement_Etat_Equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Preference_Type_equipement_Etat_Equipement, "fixed_Preference_Type_equipement_Etat_Equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, frame_Preference_Type_equipement_Etat_Equipement, "frame_Preference_Type_equipement_Etat_Equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Preference_Type_equipement_Etat_Equipement_Actions, "fixed_Preference_Type_equipement_Etat_Equipement_Actions");
	GLADE_HOOKUP_OBJECT (window_Preference, entry_Preference_Type_equipement_Etat_Equipement_Actions_Email, "entry_Preference_Type_equipement_Etat_Equipement_Actions_Email");
	GLADE_HOOKUP_OBJECT (window_Preference, entry_Preference_Type_equipement_Etat_Equipement_Actions_Script, "entry_Preference_Type_equipement_Etat_Equipement_Actions_Script");
	GLADE_HOOKUP_OBJECT (window_Preference, checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Email, "checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Email");
	GLADE_HOOKUP_OBJECT (window_Preference, checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Message, "checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Message");
	GLADE_HOOKUP_OBJECT (window_Preference, checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Script, "checkbutton_Preference_Type_equipement_Etat_Equipement_Actions_Script");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Type_equipement_Etat_Equipement_Actions, "label_Preference_Type_equipement_Etat_Equipement_Actions");
	GLADE_HOOKUP_OBJECT (window_Preference, checkbutton_Preference_Type_equipement_Etat_Equipement_Down, "checkbutton_Preference_Type_equipement_Etat_Equipement_Down");
	GLADE_HOOKUP_OBJECT (window_Preference, checkbutton_Preference_Type_equipement_Etat_Equipement, "checkbutton_Preference_Type_equipement_Etat_Equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Type_equipement_Type, "label_Preference_Type_equipement_Type");
	GLADE_HOOKUP_OBJECT (window_Preference, iconentry_Preference_Type_equipement, "iconentry_Preference_Type_equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, button_Preference_Type_equipement_Ajouter, "button_Preference_Type_equipement_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Preference, alignment_Preference_Type_equipement_Ajouter, "alignment_Preference_Type_equipement_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Preference, hbox_Preference_Type_equipement_Ajouter, "hbox_Preference_Type_equipement_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Preference_Type_equipement_Ajouter, "image_Preference_Type_equipement_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Type_equipement_Ajouter, "label_Preference_Type_equipement_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Preference, button_Preference_Type_equipement_Enlever, "button_Preference_Type_equipement_Enlever");
	GLADE_HOOKUP_OBJECT (window_Preference, alignment_Preference_Type_equipement_Enlever, "alignment_Preference_Type_equipement_Enlever");
	GLADE_HOOKUP_OBJECT (window_Preference, hbox_Preference_Type_equipement_Enlever, "hbox_Preference_Type_equipement_Enlever");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Preference_Type_equipement_Enlever, "image_Preference_Type_equipement_Enlever");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Type_equipement_Enlever, "label_Preference_Type_equipement_Enlever");
	GLADE_HOOKUP_OBJECT (window_Preference, button_Preference_Type_equipement_Valider, "button_Preference_Type_equipement_Valider");
	GLADE_HOOKUP_OBJECT (window_Preference, alignment_Preference_Type_equipement_Valider, "alignment_Preference_Type_equipement_Valider");
	GLADE_HOOKUP_OBJECT (window_Preference, hbox_Preference_Type_equipement_Valider, "hbox_Preference_Type_equipement_Valider");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Preference_Type_equipement_Valider, "image_Preference_Type_equipement_Valider");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Type_equipement_Valider, "label_Preference_Type_equipement_Valider");
	GLADE_HOOKUP_OBJECT (window_Preference, button_Preference_Type_equipement_Annuler, "button_Preference_Type_equipement_Annuler");
	GLADE_HOOKUP_OBJECT (window_Preference, alignment_Preference_Type_equipement_Annuler, "alignment_Preference_Type_equipement_Annuler");
	GLADE_HOOKUP_OBJECT (window_Preference, hbox_Preference_Type_equipement_Annuler, "hbox_Preference_Type_equipement_Annuler");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Preference_Type_equipement_Annuler, "image_Preference_Type_equipement_Annuler");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Type_equipement_Annuler, "label_Preference_Type_equipement_Annuler");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Menu_Type_Equipement, "fixed_Menu_Type_Equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Menu_Type_Equipement, "image_Menu_Type_Equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Menu_Type_Equipement, "label_Menu_Type_Equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Preference_Notification, "fixed_Preference_Notification");
	GLADE_HOOKUP_OBJECT (window_Preference, frame_Preference_Notification_Nouveau_Equipement, "frame_Preference_Notification_Nouveau_Equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Preference_Notification_Nouveau_Equipement, "fixed_Preference_Notification_Nouveau_Equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, entry_Preference_Notification_Nouveau_Equipement_Email, "entry_Preference_Notification_Nouveau_Equipement_Email");
	GLADE_HOOKUP_OBJECT (window_Preference, checkbutton_Preference_Notification_Nouveau_Equipement_Message, "checkbutton_Preference_Notification_Nouveau_Equipement_Message");
	GLADE_HOOKUP_OBJECT (window_Preference, checkbutton_Preference_Notification_Nouveau_Equipement_Email, "checkbutton_Preference_Notification_Nouveau_Equipement_Email");
	GLADE_HOOKUP_OBJECT (window_Preference, checkbutton_Preference_Notification_Nouveau_Equipement, "checkbutton_Preference_Notification_Nouveau_Equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, frame_Preference_Notification_Etat_Equipement, "frame_Preference_Notification_Etat_Equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Preference_Notification_Etat_Equipement, "fixed_Preference_Notification_Etat_Equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, spinbutton_Preference_Notification_Etat_Equipement_Up_Secondes, "spinbutton_Preference_Notification_Etat_Equipement_Up_Secondes");
	GLADE_HOOKUP_OBJECT (window_Preference, spinbutton_Preference_Notification_Etat_Equipement_Down_Secondes, "spinbutton_Preference_Notification_Etat_Equipement_Down_Secondes");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Notification_Etat_Equipement_Test_Up_Secondes, "label_Preference_Notification_Etat_Equipement_Test_Up_Secondes");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Notification_Etat_Equipement_Test_Up_Secondes, "label_Preference_Notification_Etat_Equipement_Test_Up_Secondes");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Notification_Etat_Equipement_Test_Up, "label_Preference_Notification_Etat_Equipement_Test_Up");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Notification_Etat_Equipement_Down, "label_Preference_Notification_Etat_Equipement_Down");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Notification_Etat_Equipement, "label_Preference_Notification_Etat_Equipement");
	GLADE_HOOKUP_OBJECT (window_Preference, frame_Preference_Notification_Etat_Port, "frame_Preference_Notification_Etat_Port");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Preference_Notification_Etat_Port, "fixed_Preference_Notification_Etat_Port");
	GLADE_HOOKUP_OBJECT (window_Preference, spinbutton_Preference_Notification_Etat_Port_Up_Secondes, "spinbutton_Preference_Notification_Etat_Port_Up_Secondes");
	GLADE_HOOKUP_OBJECT (window_Preference, spinbutton_Preference_Notification_Etat_Port_Down_Secondes, "spinbutton_Preference_Notification_Etat_Port_Down_Secondes");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Notification_Etat_Port_Test_Up, "label_Preference_Notification_Etat_Port_Test_Up");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Notification_Etat_Port_Test_Down, "label_Preference_Notification_Etat_Port_Test_Down");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Notification_Etat_Port_Test_Up, "label_Preference_Notification_Etat_Port_Test_Up");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Notification_Etat_Port_Test_Down, "label_Preference_Notification_Etat_Port_Test_Down");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Notification_Etat_Port, "label_Preference_Notification_Etat_Port");
	GLADE_HOOKUP_OBJECT (window_Preference, checkbutton_Preference_Notification_Etat_NBNS, "checkbutton_Preference_Notification_Etat_NBNS");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Menu_Notifications, "fixed_Menu_Notifications");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Menu_Notifications, "image_Menu_Notifications");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Menu_Notifications, "label_Menu_Notifications");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Preference_Preference, "fixed_Preference_Preference");
	GLADE_HOOKUP_OBJECT (window_Preference, entry_Preference_Preference_Shell, "entry_Preference_Preference_Shell");
	GLADE_HOOKUP_OBJECT (window_Preference, entry_Preference_Preference_Samba, "entry_Preference_Preference_Samba");
	GLADE_HOOKUP_OBJECT (window_Preference, entry_Preference_Preference_Web, "entry_Preference_Preference_Web");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Preference_Samba, "label_Preference_Preference_Samba");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Preference_Shell, "label_Preference_Preference_Shell");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Preference_Web, "label_Preference_Preference_Web");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Preference_Smtp, "label_Preference_Preference_Smtp");
	GLADE_HOOKUP_OBJECT (window_Preference, entry_Preference_Preference_Smtp, "entry_Preference_Preference_Smtp");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Menu_Preferences, "fixed_Menu_Preferences");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Menu_Preferences, "image_Menu_Preferences");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Menu_Preferences, "label_Menu_Preferences");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Preference_Extensions, "fixed_Preference_Extensions");
	GLADE_HOOKUP_OBJECT (window_Preference, ctree_Preference_Extensions, "ctree_Preference_Extensions");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Extensions, "label_Preference_Extensions");
	GLADE_HOOKUP_OBJECT (window_Preference, fixed_Menu_Extensions, "fixed_Menu_Extensions");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Menu_Extensions, "image_Menu_Extensions");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Menu_Extensions, "label_Menu_Extensions");
	GLADE_HOOKUP_OBJECT (window_Preference, button_Preference_Enregistrer, "button_Preference_Enregistrer");
	GLADE_HOOKUP_OBJECT (window_Preference, alignment_Preference_Enregistrer, "alignment_Preference_Enregistrer");
	GLADE_HOOKUP_OBJECT (window_Preference, hbox_Preference_Enregistrer, "hbox_Preference_Enregistrer");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Preference_Enregistrer, "image_Preference_Enregistrer");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Enregistrer, "label_Preference_Enregistrer");
	GLADE_HOOKUP_OBJECT (window_Preference, button_Preference_Annuler, "button_Preference_Annuler");
	GLADE_HOOKUP_OBJECT (window_Preference, alignment_Preference_Annuler, "alignment_Preference_Annuler");
	GLADE_HOOKUP_OBJECT (window_Preference, hbox_Preference_Annuler, "hbox_Preference_Annuler");
	GLADE_HOOKUP_OBJECT (window_Preference, image_Preference_Annuler, "image_Preference_Annuler");
	GLADE_HOOKUP_OBJECT (window_Preference, label_Preference_Annuler, "label_Preference_Annuler");

	return window_Preference;
}

GtkWidget *create_window_Liste_Signatures (void)
{
	GList    *combo_Liste_Signatures_Type_items = NULL;
	
	
    tooltips = gtk_tooltips_new ();
	window_Liste_Signatures = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_Liste_Signatures, 702, 495);
	gtk_window_set_title (GTK_WINDOW (window_Liste_Signatures), "OS fingerprinting list");
	gtk_window_set_position (GTK_WINDOW (window_Liste_Signatures), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window_Liste_Signatures), FALSE);
	window_Liste_Signatures_icon_pixbuf = create_pixbuf ("/usr/share/pixmaps/AutoScan/AutoScan.png");
	if (window_Liste_Signatures_icon_pixbuf) {
		gtk_window_set_icon (GTK_WINDOW (window_Liste_Signatures), window_Liste_Signatures_icon_pixbuf);
		gdk_pixbuf_unref (window_Liste_Signatures_icon_pixbuf);
	}

	fixed_Liste_Signatures = gtk_fixed_new ();
	gtk_widget_show (fixed_Liste_Signatures);
	gtk_container_add (GTK_CONTAINER (window_Liste_Signatures), fixed_Liste_Signatures);

	scrolledwindow_Liste_Signatures = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Liste_Signatures);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures), scrolledwindow_Liste_Signatures, 16, 8);
	gtk_widget_set_size_request (scrolledwindow_Liste_Signatures, 665, 216);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_Liste_Signatures), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	ctree_Liste_Signatures = gtk_ctree_new (2, 0);
	gtk_widget_show (ctree_Liste_Signatures);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Liste_Signatures), ctree_Liste_Signatures);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Liste_Signatures), 0, 314);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Liste_Signatures), 1, 80);
	gtk_clist_column_titles_show (GTK_CLIST (ctree_Liste_Signatures));

	label_Liste_Signatures_Nom_Systeme = gtk_label_new ("Sytem name");
	gtk_widget_show (label_Liste_Signatures_Nom_Systeme);
	gtk_clist_set_column_widget (GTK_CLIST (ctree_Liste_Signatures), 0, label_Liste_Signatures_Nom_Systeme);

	label_Liste_Signatures_md5 = gtk_label_new ("MD5 signatures");
	gtk_widget_show (label_Liste_Signatures_md5);
	gtk_clist_set_column_widget (GTK_CLIST (ctree_Liste_Signatures), 1, label_Liste_Signatures_md5);

	frame_Liste_Signatures = gtk_frame_new (NULL);
	gtk_widget_show (frame_Liste_Signatures);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures), frame_Liste_Signatures, 16, 232);
	gtk_widget_set_size_request (frame_Liste_Signatures, 560, 254);

	fixed_Liste_Signatures_Ajout = gtk_fixed_new ();
	gtk_widget_show (fixed_Liste_Signatures_Ajout);
	gtk_container_add (GTK_CONTAINER (frame_Liste_Signatures), fixed_Liste_Signatures_Ajout);

	entry_Liste_Signatures_Nom = gtk_entry_new ();
	gtk_widget_show (entry_Liste_Signatures_Nom);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Ajout), entry_Liste_Signatures_Nom, 144, 11);
	gtk_widget_set_size_request (entry_Liste_Signatures_Nom, 232, 21);
    gtk_tooltips_set_tip (tooltips, entry_Liste_Signatures_Nom, _("Example1 : Linux kernel 2.4 (X86) from Debian \nExample2 : Microsoft Windows XP Home (French) SP1\nExample3 : Apple Mac OS X 10.2 (Jaguar) \nExample4 : Cisco 4500 router running IOS 10.1"), NULL);
	
	
	entry_Liste_Signatures_md5 = gtk_entry_new ();
	gtk_widget_show (entry_Liste_Signatures_md5);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Ajout), entry_Liste_Signatures_md5, 144, 37);
	gtk_widget_set_size_request (entry_Liste_Signatures_md5, 232, 21);

	combo_Liste_Signatures_Type = gtk_combo_new ();
	g_object_set_data (G_OBJECT (GTK_COMBO (combo_Liste_Signatures_Type)->popwin), "GladeParentKey", combo_Liste_Signatures_Type);
	gtk_widget_show (combo_Liste_Signatures_Type);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Ajout), combo_Liste_Signatures_Type, 143, 63);
	gtk_widget_set_size_request (combo_Liste_Signatures_Type, 183, 24);
	combo_Liste_Signatures_Type_items = g_list_append (combo_Liste_Signatures_Type_items, (gpointer) "WorkStation");
	combo_Liste_Signatures_Type_items = g_list_append (combo_Liste_Signatures_Type_items, (gpointer) "Switch");
	combo_Liste_Signatures_Type_items = g_list_append (combo_Liste_Signatures_Type_items, (gpointer) "Printer");
	combo_Liste_Signatures_Type_items = g_list_append (combo_Liste_Signatures_Type_items, (gpointer) "Router");
	combo_Liste_Signatures_Type_items = g_list_append (combo_Liste_Signatures_Type_items, (gpointer) "Router-Switch");
	combo_Liste_Signatures_Type_items = g_list_append (combo_Liste_Signatures_Type_items, (gpointer) "Wifi");
	gtk_combo_set_popdown_strings (GTK_COMBO (combo_Liste_Signatures_Type), combo_Liste_Signatures_Type_items);
	g_list_free (combo_Liste_Signatures_Type_items);

	entry_Liste_Signatures_Type = GTK_COMBO (combo_Liste_Signatures_Type)->entry;
	gtk_widget_show (entry_Liste_Signatures_Type);

	label_Liste_Signatures_Type = gtk_label_new ("Type : ");
	gtk_widget_show (label_Liste_Signatures_Type);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Ajout), label_Liste_Signatures_Type, 72, 64);
	gtk_widget_set_size_request (label_Liste_Signatures_Type, 47, 17);

	label_Liste_Signatures_Up = gtk_label_new ("Up");
	gtk_widget_show (label_Liste_Signatures_Up);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Ajout), label_Liste_Signatures_Up, 416, 72);
	gtk_widget_set_size_request (label_Liste_Signatures_Up, 32, 16);

	label_Liste_Signatures_Down = gtk_label_new ("Down");
	gtk_widget_show (label_Liste_Signatures_Down);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Ajout), label_Liste_Signatures_Down, 488, 72);
	gtk_widget_set_size_request (label_Liste_Signatures_Down, 39, 16);

	label_Liste_Signatures_Nom = gtk_label_new ("System name :");
	gtk_widget_show (label_Liste_Signatures_Nom);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Ajout), label_Liste_Signatures_Nom, 16, 13);
	gtk_widget_set_size_request (label_Liste_Signatures_Nom, 120, 16);

	label_Liste_Signatures_md5 = gtk_label_new ("Md5 signature :");
	gtk_widget_show (label_Liste_Signatures_md5);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Ajout), label_Liste_Signatures_md5, 8, 38);
	gtk_widget_set_size_request (label_Liste_Signatures_md5, 120, 16);

	frame_Liste_Signatures_Etat = gtk_frame_new (NULL);
	//gtk_widget_show (frame_Liste_Signatures_Etat);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Ajout), frame_Liste_Signatures_Etat, 136, 104);
	gtk_widget_set_size_request (frame_Liste_Signatures_Etat, 360, 120);

	fixed_Liste_Signatures_Etat = gtk_fixed_new ();
	gtk_widget_show (fixed_Liste_Signatures_Etat);
	gtk_container_add (GTK_CONTAINER (frame_Liste_Signatures_Etat), fixed_Liste_Signatures_Etat);

	frame_Liste_Signatures_Etat_Action = gtk_frame_new (NULL);
	gtk_widget_show (frame_Liste_Signatures_Etat_Action);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Etat), frame_Liste_Signatures_Etat_Action, 32, 24);
	gtk_widget_set_size_request (frame_Liste_Signatures_Etat_Action, 280, 64);

	fixed_Liste_Signatures_Etat_Action = gtk_fixed_new ();
	gtk_widget_show (fixed_Liste_Signatures_Etat_Action);
	gtk_container_add (GTK_CONTAINER (frame_Liste_Signatures_Etat_Action), fixed_Liste_Signatures_Etat_Action);

	entry_Liste_Signatures_Etat_Action_Email = gtk_entry_new ();
	gtk_widget_show (entry_Liste_Signatures_Etat_Action_Email);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Etat_Action), entry_Liste_Signatures_Etat_Action_Email, 80, 3);
	gtk_widget_set_size_request (entry_Liste_Signatures_Etat_Action_Email, 188, 21);

	checkbutton_Liste_Signatures_Etat_Action_Message = gtk_check_button_new_with_mnemonic ("Message");
	gtk_widget_show (checkbutton_Liste_Signatures_Etat_Action_Message);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Etat_Action), checkbutton_Liste_Signatures_Etat_Action_Message, 16, 24);
	gtk_widget_set_size_request (checkbutton_Liste_Signatures_Etat_Action_Message, 98, 23);

	checkbutton_Liste_Signatures_Etat_Action_Email = gtk_check_button_new_with_mnemonic ("Email");
	gtk_widget_show (checkbutton_Liste_Signatures_Etat_Action_Email);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Etat_Action), checkbutton_Liste_Signatures_Etat_Action_Email, 16, 0);
	gtk_widget_set_size_request (checkbutton_Liste_Signatures_Etat_Action_Email, 136, 24);

	label_Liste_Signatures_Etat_Action = gtk_label_new ("Actions");
	gtk_widget_show (label_Liste_Signatures_Etat_Action);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Liste_Signatures_Etat_Action), label_Liste_Signatures_Etat_Action);

	checkbutton_Liste_Signatures_Etat_Down = gtk_check_button_new_with_mnemonic ("Poll even when it is down");
	gtk_widget_show (checkbutton_Liste_Signatures_Etat_Down);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Etat), checkbutton_Liste_Signatures_Etat_Down, 16, 0);
	gtk_widget_set_size_request (checkbutton_Liste_Signatures_Etat_Down, 168, 24);

	checkbutton_Liste_Signatures_Etat = gtk_check_button_new_with_mnemonic ("Detect the state change of the equipment");
	gtk_widget_show (checkbutton_Liste_Signatures_Etat);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Liste_Signatures_Etat), checkbutton_Liste_Signatures_Etat);

	checkbutton_Liste_Signatures_NETBIOS = gtk_check_button_new_with_mnemonic ("Netbios request");
	gtk_widget_show (checkbutton_Liste_Signatures_NETBIOS);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Ajout), checkbutton_Liste_Signatures_NETBIOS, 16, 112);
	gtk_widget_set_size_request (checkbutton_Liste_Signatures_NETBIOS, 120, 24);

	checkbutton_Liste_Signatures_Routeur = gtk_check_button_new_with_mnemonic ("");
	//gtk_widget_show (checkbutton_Liste_Signatures_Routeur);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Ajout), checkbutton_Liste_Signatures_Routeur, 16, 136);
	gtk_widget_set_size_request (checkbutton_Liste_Signatures_Routeur, 98, 23);

	iconentry_Liste_Signature_Allumer = gnome_icon_entry_new (NULL, NULL);
	gtk_widget_show (iconentry_Liste_Signature_Allumer);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Ajout), iconentry_Liste_Signature_Allumer, 400, 8);
	gtk_widget_set_size_request (iconentry_Liste_Signature_Allumer, 60, 60);

	iconentry_Liste_Signature_Eteint = gnome_icon_entry_new (NULL, NULL);
	gtk_widget_show (iconentry_Liste_Signature_Eteint);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures_Ajout), iconentry_Liste_Signature_Eteint, 480, 8);
	gtk_widget_set_size_request (iconentry_Liste_Signature_Eteint, 60, 60);

	label_Liste_Signatures_Ajout = gtk_label_new ("change, add, delete");
	gtk_widget_show (label_Liste_Signatures_Ajout);
	gtk_frame_set_label_widget (GTK_FRAME (frame_Liste_Signatures), label_Liste_Signatures_Ajout);

	button_Liste_Signatures_Annuler = gtk_button_new ();
	gtk_widget_show (button_Liste_Signatures_Annuler);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures), button_Liste_Signatures_Annuler, 584, 384);
	gtk_widget_set_size_request (button_Liste_Signatures_Annuler, 104, 36);

	alignment_Liste_Signatures_Annuler = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Liste_Signatures_Annuler);
	gtk_container_add (GTK_CONTAINER (button_Liste_Signatures_Annuler), alignment_Liste_Signatures_Annuler);

	hbox_Liste_Signatures_Annuler = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Liste_Signatures_Annuler);
	gtk_container_add (GTK_CONTAINER (alignment_Liste_Signatures_Annuler), hbox_Liste_Signatures_Annuler);

	image_Liste_Signatures_Annuler = create_pixmap (window_Liste_Signatures, "/usr/share/pixmaps/AutoScan/button_cancel.png");
	gtk_widget_show (image_Liste_Signatures_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Signatures_Annuler), image_Liste_Signatures_Annuler, FALSE, FALSE, 0);

	label_Liste_Signatures_Annuler = gtk_label_new_with_mnemonic ("Cancel");
	gtk_widget_show (label_Liste_Signatures_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Signatures_Annuler), label_Liste_Signatures_Annuler, FALSE, FALSE, 0);

	button_Liste_Signatures_Ok = gtk_button_new ();
	gtk_widget_show (button_Liste_Signatures_Ok);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures), button_Liste_Signatures_Ok, 584, 344);
	gtk_widget_set_size_request (button_Liste_Signatures_Ok, 104, 36);

	alignment_Liste_Signatures_Enregistrer = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Liste_Signatures_Enregistrer);
	gtk_container_add (GTK_CONTAINER (button_Liste_Signatures_Ok), alignment_Liste_Signatures_Enregistrer);

	hbox_Liste_Signatures_Enregistrer = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Liste_Signatures_Enregistrer);
	gtk_container_add (GTK_CONTAINER (alignment_Liste_Signatures_Enregistrer), hbox_Liste_Signatures_Enregistrer);

	image_Liste_Signatures_Enregistrer = create_pixmap (window_Liste_Signatures, "/usr/share/pixmaps/AutoScan/button_ok.png");
	gtk_widget_show (image_Liste_Signatures_Enregistrer);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Signatures_Enregistrer), image_Liste_Signatures_Enregistrer, FALSE, FALSE, 0);

	label_Liste_Signatures_Enregistrer = gtk_label_new_with_mnemonic ("Ok");
	gtk_widget_show (label_Liste_Signatures_Enregistrer);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Signatures_Enregistrer), label_Liste_Signatures_Enregistrer, FALSE, FALSE, 0);

	button_Liste_Signatures_Enlever = gtk_button_new ();
	gtk_widget_show (button_Liste_Signatures_Enlever);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures), button_Liste_Signatures_Enlever, 584, 288);
	gtk_widget_set_size_request (button_Liste_Signatures_Enlever, 104, 36);

	alignment_Liste_Signatures_Enlever = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Liste_Signatures_Enlever);
	gtk_container_add (GTK_CONTAINER (button_Liste_Signatures_Enlever), alignment_Liste_Signatures_Enlever);

	hbox_Liste_Signatures_Enlever = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Liste_Signatures_Enlever);
	gtk_container_add (GTK_CONTAINER (alignment_Liste_Signatures_Enlever), hbox_Liste_Signatures_Enlever);

	image_Liste_Signatures_Enlever = create_pixmap (window_Liste_Signatures, "/usr/share/pixmaps/AutoScan/editcut.png");
	gtk_widget_show (image_Liste_Signatures_Enlever);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Signatures_Enlever), image_Liste_Signatures_Enlever, FALSE, FALSE, 0);

	label_Liste_Signatures_Enlever = gtk_label_new_with_mnemonic ("Delete");
	gtk_widget_show (label_Liste_Signatures_Enlever);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Signatures_Enlever), label_Liste_Signatures_Enlever, FALSE, FALSE, 0);

	button_Liste_Signatures_Ajouter = gtk_button_new ();
	gtk_widget_show (button_Liste_Signatures_Ajouter);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures), button_Liste_Signatures_Ajouter, 584, 248);
	gtk_widget_set_size_request (button_Liste_Signatures_Ajouter, 104, 36);

	alignment_Liste_Signatures_Ajouter = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Liste_Signatures_Ajouter);
	gtk_container_add (GTK_CONTAINER (button_Liste_Signatures_Ajouter), alignment_Liste_Signatures_Ajouter);

	hbox_Liste_Signatures_Ajouter = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Liste_Signatures_Ajouter);
	gtk_container_add (GTK_CONTAINER (alignment_Liste_Signatures_Ajouter), hbox_Liste_Signatures_Ajouter);

	image_Liste_Signatures_Ajouter = create_pixmap (window_Liste_Signatures, "/usr/share/pixmaps/AutoScan/filenew.png");
	gtk_widget_show (image_Liste_Signatures_Ajouter);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Signatures_Ajouter), image_Liste_Signatures_Ajouter, FALSE, FALSE, 0);

	label_Liste_Signatures_Ajouter = gtk_label_new_with_mnemonic ("Add");
	gtk_widget_show (label_Liste_Signatures_Ajouter);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Signatures_Ajouter), label_Liste_Signatures_Ajouter, FALSE, FALSE, 0);

	button_Liste_Signatures_Sauvegarder = gtk_button_new ();
	gtk_widget_show (button_Liste_Signatures_Sauvegarder);
	gtk_fixed_put (GTK_FIXED (fixed_Liste_Signatures), button_Liste_Signatures_Sauvegarder, 584, 440);
	gtk_widget_set_size_request (button_Liste_Signatures_Sauvegarder, 104, 36);

	alignment_Liste_Signatures_Sauvegarder = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Liste_Signatures_Sauvegarder);
	gtk_container_add (GTK_CONTAINER (button_Liste_Signatures_Sauvegarder), alignment_Liste_Signatures_Sauvegarder);

	hbox_Liste_Signatures_Sauvegarder = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Liste_Signatures_Sauvegarder);
	gtk_container_add (GTK_CONTAINER (alignment_Liste_Signatures_Sauvegarder), hbox_Liste_Signatures_Sauvegarder);

	image_Liste_Signatures_Sauvegarder = create_pixmap (window_Liste_Signatures, "/usr/share/pixmaps/AutoScan/filesave.png");
	gtk_widget_show (image_Liste_Signatures_Sauvegarder);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Signatures_Sauvegarder), image_Liste_Signatures_Sauvegarder, FALSE, FALSE, 0);

	label_Liste_Signatures_Sauvegarder = gtk_label_new_with_mnemonic ("Save");
	gtk_widget_show (label_Liste_Signatures_Sauvegarder);
	gtk_box_pack_start (GTK_BOX (hbox_Liste_Signatures_Sauvegarder), label_Liste_Signatures_Sauvegarder, FALSE, FALSE, 0);

	g_signal_connect ((gpointer) window_Liste_Signatures, "destroy", G_CALLBACK (on_window_Liste_Signatures_destroy), NULL);
	g_signal_connect ((gpointer) ctree_Liste_Signatures, "tree_select_row", G_CALLBACK (on_ctree_Liste_Signatures_tree_select_row), NULL);
	g_signal_connect ((gpointer) entry_Liste_Signatures_Nom, "changed", G_CALLBACK (Liste_Signatures_changed), NULL);
	g_signal_connect ((gpointer) entry_Liste_Signatures_md5, "changed", G_CALLBACK (Liste_Signatures_changed), NULL);
	g_signal_connect ((gpointer) entry_Liste_Signatures_Type, "changed", G_CALLBACK (Liste_Signatures_changed), NULL);
	g_signal_connect ((gpointer) entry_Liste_Signatures_Etat_Action_Email, "changed", G_CALLBACK (Liste_Signatures_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Signatures_Etat_Action_Message, "clicked", G_CALLBACK (Liste_Signatures_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Signatures_Etat_Action_Email, "clicked", G_CALLBACK (Liste_Signatures_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Signatures_Etat_Action_Email, "clicked", G_CALLBACK (on_checkbutton_Liste_Signatures_Etat_Action_Email_clicked), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Signatures_Etat_Down, "clicked", G_CALLBACK (Liste_Signatures_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Signatures_Etat, "clicked", G_CALLBACK (Liste_Signatures_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Signatures_Etat, "clicked", G_CALLBACK (on_checkbutton_Liste_Signatures_Etat_clicked), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Signatures_NETBIOS, "clicked", G_CALLBACK (Liste_Signatures_changed), NULL);
	g_signal_connect ((gpointer) checkbutton_Liste_Signatures_Routeur, "clicked", G_CALLBACK (Liste_Signatures_changed), NULL);
	g_signal_connect ((gpointer) iconentry_Liste_Signature_Allumer, "changed", G_CALLBACK (Liste_Signatures_changed), NULL);
	g_signal_connect ((gpointer) iconentry_Liste_Signature_Eteint, "changed", G_CALLBACK (Liste_Signatures_changed), NULL);
	g_signal_connect ((gpointer) button_Liste_Signatures_Annuler, "clicked", G_CALLBACK (on_button_Liste_Signatures_Annuler_clicked), NULL);
	g_signal_connect ((gpointer) button_Liste_Signatures_Ok, "clicked", G_CALLBACK (on_button_Liste_Signatures_Ok_clicked), NULL);
	g_signal_connect ((gpointer) button_Liste_Signatures_Enlever, "clicked", G_CALLBACK (on_button_Liste_Signatures_Enlever_clicked), NULL);
	g_signal_connect ((gpointer) button_Liste_Signatures_Ajouter, "clicked", G_CALLBACK (on_button_Liste_Signatures_Ajouter_clicked), NULL);
	g_signal_connect ((gpointer) button_Liste_Signatures_Sauvegarder, "clicked", G_CALLBACK (on_button_Liste_Signatures_Sauvegarder_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_Liste_Signatures, window_Liste_Signatures, "window_Liste_Signatures");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, fixed_Liste_Signatures, "fixed_Liste_Signatures");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, scrolledwindow_Liste_Signatures, "scrolledwindow_Liste_Signatures");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, ctree_Liste_Signatures, "ctree_Liste_Signatures");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, label_Liste_Signatures_Nom_Systeme, "label_Liste_Signatures_Nom_Systeme");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, label_Liste_Signatures_md5, "label_Liste_Signatures_md5");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, frame_Liste_Signatures, "frame_Liste_Signatures");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, fixed_Liste_Signatures_Ajout, "fixed_Liste_Signatures_Ajout");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, entry_Liste_Signatures_Nom, "entry_Liste_Signatures_Nom");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, entry_Liste_Signatures_md5, "entry_Liste_Signatures_md5");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, combo_Liste_Signatures_Type, "combo_Liste_Signatures_Type");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, entry_Liste_Signatures_Type, "entry_Liste_Signatures_Type");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, label_Liste_Signatures_Type, "label_Liste_Signatures_Type");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, label_Liste_Signatures_Up, "label_Liste_Signatures_Up");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, label_Liste_Signatures_Down, "label_Liste_Signatures_Down");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, label_Liste_Signatures_Nom, "label_Liste_Signatures_Nom");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, label_Liste_Signatures_md5, "label_Liste_Signatures_md5");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, frame_Liste_Signatures_Etat, "frame_Liste_Signatures_Etat");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, fixed_Liste_Signatures_Etat, "fixed_Liste_Signatures_Etat");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, frame_Liste_Signatures_Etat_Action, "frame_Liste_Signatures_Etat_Action");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, fixed_Liste_Signatures_Etat_Action, "fixed_Liste_Signatures_Etat_Action");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, entry_Liste_Signatures_Etat_Action_Email, "entry_Liste_Signatures_Etat_Action_Email");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, checkbutton_Liste_Signatures_Etat_Action_Message, "checkbutton_Liste_Signatures_Etat_Action_Message");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, checkbutton_Liste_Signatures_Etat_Action_Email, "checkbutton_Liste_Signatures_Etat_Action_Email");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, label_Liste_Signatures_Etat_Action, "label_Liste_Signatures_Etat_Action");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, checkbutton_Liste_Signatures_Etat_Down, "checkbutton_Liste_Signatures_Etat_Down");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, checkbutton_Liste_Signatures_Etat, "checkbutton_Liste_Signatures_Etat");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, checkbutton_Liste_Signatures_NETBIOS, "checkbutton_Liste_Signatures_NETBIOS");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, checkbutton_Liste_Signatures_Routeur, "checkbutton_Liste_Signatures_Routeur");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, iconentry_Liste_Signature_Allumer, "iconentry_Liste_Signature_Allumer");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, iconentry_Liste_Signature_Eteint, "iconentry_Liste_Signature_Eteint");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, label_Liste_Signatures_Ajout, "label_Liste_Signatures_Ajout");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, button_Liste_Signatures_Annuler, "button_Liste_Signatures_Annuler");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, alignment_Liste_Signatures_Annuler, "alignment_Liste_Signatures_Annuler");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, hbox_Liste_Signatures_Annuler, "hbox_Liste_Signatures_Annuler");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, image_Liste_Signatures_Annuler, "image_Liste_Signatures_Annuler");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, label_Liste_Signatures_Annuler, "label_Liste_Signatures_Annuler");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, button_Liste_Signatures_Ok, "button_Liste_Signatures_Ok");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, alignment_Liste_Signatures_Enregistrer, "alignment_Liste_Signatures_Enregistrer");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, hbox_Liste_Signatures_Enregistrer, "hbox_Liste_Signatures_Enregistrer");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, image_Liste_Signatures_Enregistrer, "image_Liste_Signatures_Enregistrer");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, label_Liste_Signatures_Enregistrer, "label_Liste_Signatures_Enregistrer");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, button_Liste_Signatures_Enlever, "button_Liste_Signatures_Enlever");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, alignment_Liste_Signatures_Enlever, "alignment_Liste_Signatures_Enlever");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, hbox_Liste_Signatures_Enlever, "hbox_Liste_Signatures_Enlever");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, image_Liste_Signatures_Enlever, "image_Liste_Signatures_Enlever");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, label_Liste_Signatures_Enlever, "label_Liste_Signatures_Enlever");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, button_Liste_Signatures_Ajouter, "button_Liste_Signatures_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, alignment_Liste_Signatures_Ajouter, "alignment_Liste_Signatures_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, hbox_Liste_Signatures_Ajouter, "hbox_Liste_Signatures_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, image_Liste_Signatures_Ajouter, "image_Liste_Signatures_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, label_Liste_Signatures_Ajouter, "label_Liste_Signatures_Ajouter");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, button_Liste_Signatures_Sauvegarder, "button_Liste_Signatures_Sauvegarder");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, alignment_Liste_Signatures_Sauvegarder, "alignment_Liste_Signatures_Sauvegarder");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, hbox_Liste_Signatures_Sauvegarder, "hbox_Liste_Signatures_Sauvegarder");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, image_Liste_Signatures_Sauvegarder, "image_Liste_Signatures_Sauvegarder");
	GLADE_HOOKUP_OBJECT (window_Liste_Signatures, label_Liste_Signatures_Sauvegarder, "label_Liste_Signatures_Sauvegarder");

	return window_Liste_Signatures;
}

GtkWidget *create_window_Debug_Scan (void)
{
	window_Debug_Scan = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_Debug_Scan, 561, 480);
	gtk_window_set_title (GTK_WINDOW (window_Debug_Scan), "Debug Scan");
	gtk_window_set_position (GTK_WINDOW (window_Debug_Scan), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window_Debug_Scan), FALSE);

	fixed_Debug_Scan = gtk_fixed_new ();
	gtk_widget_show (fixed_Debug_Scan);
	gtk_container_add (GTK_CONTAINER (window_Debug_Scan), fixed_Debug_Scan);

	scrolledwindow_Debug_Scan = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Debug_Scan);
	gtk_fixed_put (GTK_FIXED (fixed_Debug_Scan), scrolledwindow_Debug_Scan, 8, 8);
	gtk_widget_set_size_request (scrolledwindow_Debug_Scan, 544, 424);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_Debug_Scan), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	textview_Debug_Scan = gtk_text_new (NULL, NULL);
	gtk_widget_show (textview_Debug_Scan);
	gtk_text_set_editable (textview_Debug_Scan, FALSE);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Debug_Scan), textview_Debug_Scan);

	button_Debug_Scan_Fermer = gtk_button_new_from_stock ("gtk-close");
	gtk_widget_show (button_Debug_Scan_Fermer);
	gtk_fixed_put (GTK_FIXED (fixed_Debug_Scan), button_Debug_Scan_Fermer, 440, 440);
	gtk_widget_set_size_request (button_Debug_Scan_Fermer, 104, 32);

	g_signal_connect ((gpointer) window_Debug_Scan, "destroy", G_CALLBACK (on_window_Debug_Scan_destroy), NULL);
	g_signal_connect ((gpointer) button_Debug_Scan_Fermer, "clicked", G_CALLBACK (on_button_Debug_Scan_Fermer_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_Debug_Scan, window_Debug_Scan, "window_Debug_Scan");
	GLADE_HOOKUP_OBJECT (window_Debug_Scan, fixed_Debug_Scan, "fixed_Debug_Scan");
	GLADE_HOOKUP_OBJECT (window_Debug_Scan, scrolledwindow_Debug_Scan, "scrolledwindow_Debug_Scan");
	GLADE_HOOKUP_OBJECT (window_Debug_Scan, textview_Debug_Scan, "textview_Debug_Scan");
	GLADE_HOOKUP_OBJECT (window_Debug_Scan, button_Debug_Scan_Fermer, "button_Debug_Scan_Fermer");

	return window_Debug_Scan;
}

GtkWidget *create_window_Partage (void)
{
	accel_group = gtk_accel_group_new ();

	window_Partage = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_Partage, 547, 153);
	gtk_window_set_title (GTK_WINDOW (window_Partage), "file sharing");
	gtk_window_set_position (GTK_WINDOW (window_Partage), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window_Partage), FALSE);

	fixed_Partage = gtk_fixed_new ();
	gtk_widget_show (fixed_Partage);
	gtk_container_add (GTK_CONTAINER (window_Partage), fixed_Partage);

	entry_Partage_Utilisateur = gtk_entry_new ();
	gtk_widget_show (entry_Partage_Utilisateur);
	gtk_fixed_put (GTK_FIXED (fixed_Partage), entry_Partage_Utilisateur, 216, 8);
	gtk_widget_set_size_request (entry_Partage_Utilisateur, 158, 25);

	entry_Partage_Mot_Passe = gtk_entry_new ();
	gtk_widget_show (entry_Partage_Mot_Passe);
	gtk_fixed_put (GTK_FIXED (fixed_Partage), entry_Partage_Mot_Passe, 216, 48);
	gtk_widget_set_size_request (entry_Partage_Mot_Passe, 158, 25);
	gtk_entry_set_visibility (GTK_ENTRY (entry_Partage_Mot_Passe), FALSE);

	entry_Partage_Domaine = gtk_entry_new ();
	gtk_widget_show (entry_Partage_Domaine);
	gtk_fixed_put (GTK_FIXED (fixed_Partage), entry_Partage_Domaine, 216, 88);
	gtk_widget_set_size_request (entry_Partage_Domaine, 158, 25);

	label_Partage_Mot_Passe = gtk_label_new ("Password :");
	gtk_widget_show (label_Partage_Mot_Passe);
	gtk_fixed_put (GTK_FIXED (fixed_Partage), label_Partage_Mot_Passe, 104, 56);
	gtk_widget_set_size_request (label_Partage_Mot_Passe, 112, 16);

	label_Partage_Domaine = gtk_label_new ("Domaine :");
	gtk_widget_show (label_Partage_Domaine);
	gtk_fixed_put (GTK_FIXED (fixed_Partage), label_Partage_Domaine, 128, 96);
	gtk_widget_set_size_request (label_Partage_Domaine, 80, 16);

	button_Partage_Fermer = gtk_button_new_from_stock ("gtk-close");
	gtk_widget_show (button_Partage_Fermer);
	gtk_fixed_put (GTK_FIXED (fixed_Partage), button_Partage_Fermer, 416, 72);
	gtk_widget_set_size_request (button_Partage_Fermer, 107, 32);

	button_Partage_Valider = gtk_button_new_from_stock ("gtk-ok");
	gtk_widget_show (button_Partage_Valider);
	gtk_fixed_put (GTK_FIXED (fixed_Partage), button_Partage_Valider, 416, 24);
	gtk_widget_set_size_request (button_Partage_Valider, 104, 32);
	gtk_widget_add_accelerator (button_Partage_Valider, "clicked", accel_group, GDK_Return, 0, GTK_ACCEL_VISIBLE);

	checkbutton_Partage_Password = gtk_check_button_new_with_mnemonic ("Always use this password");
	gtk_widget_show (checkbutton_Partage_Password);
	gtk_fixed_put (GTK_FIXED (fixed_Partage), checkbutton_Partage_Password, 280, 120);
	gtk_widget_set_size_request (checkbutton_Partage_Password, 184, 24);

	label_Partage_Utilisateur = gtk_label_new ("User :");
	gtk_widget_show (label_Partage_Utilisateur);
	gtk_fixed_put (GTK_FIXED (fixed_Partage), label_Partage_Utilisateur, 120, 16);
	gtk_widget_set_size_request (label_Partage_Utilisateur, 96, 16);

	image_Partage_Cle = create_pixmap (window_Partage, "/usr/share/pixmaps/AutoScan/password.png");
	gtk_widget_show (image_Partage_Cle);
	gtk_fixed_put (GTK_FIXED (fixed_Partage), image_Partage_Cle, 8, 8);
	gtk_widget_set_size_request (image_Partage_Cle, 136, 136);

	g_signal_connect ((gpointer) window_Partage, "destroy", G_CALLBACK (on_window_Partage_destroy), NULL);
	g_signal_connect ((gpointer) button_Partage_Fermer, "clicked", G_CALLBACK (on_button_Partage_Fermer_clicked), NULL);
	g_signal_connect ((gpointer) button_Partage_Valider, "clicked", G_CALLBACK (on_button_Partage_Valider_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_Partage, window_Partage, "window_Partage");
	GLADE_HOOKUP_OBJECT (window_Partage, fixed_Partage, "fixed_Partage");
	GLADE_HOOKUP_OBJECT (window_Partage, entry_Partage_Utilisateur, "entry_Partage_Utilisateur");
	GLADE_HOOKUP_OBJECT (window_Partage, entry_Partage_Mot_Passe, "entry_Partage_Mot_Passe");
	GLADE_HOOKUP_OBJECT (window_Partage, entry_Partage_Domaine, "entry_Partage_Domaine");
	GLADE_HOOKUP_OBJECT (window_Partage, label_Partage_Mot_Passe, "label_Partage_Mot_Passe");
	GLADE_HOOKUP_OBJECT (window_Partage, label_Partage_Domaine, "label_Partage_Domaine");
	GLADE_HOOKUP_OBJECT (window_Partage, button_Partage_Fermer, "button_Partage_Fermer");
	GLADE_HOOKUP_OBJECT (window_Partage, button_Partage_Valider, "button_Partage_Valider");
	GLADE_HOOKUP_OBJECT (window_Partage, checkbutton_Partage_Password, "checkbutton_Partage_Password");
	GLADE_HOOKUP_OBJECT (window_Partage, label_Partage_Utilisateur, "label_Partage_Utilisateur");
	GLADE_HOOKUP_OBJECT (window_Partage, image_Partage_Cle, "image_Partage_Cle");

	gtk_window_add_accel_group (GTK_WINDOW (window_Partage), accel_group);

	return window_Partage;
}

GtkWidget *create_window_Dns (void)
{
	window_Dns = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_Dns, 570, 502);
	gtk_window_set_title (GTK_WINDOW (window_Dns), "Dns");
	gtk_window_set_resizable (GTK_WINDOW (window_Dns), FALSE);

	hpaned_Dns = gtk_hpaned_new ();
	gtk_widget_show (hpaned_Dns);
	gtk_container_add (GTK_CONTAINER (window_Dns), hpaned_Dns);
	gtk_paned_set_position (GTK_PANED (hpaned_Dns), 171);

	scrolledwindow_Dns_Liste = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Dns_Liste);
	gtk_paned_pack1 (GTK_PANED (hpaned_Dns), scrolledwindow_Dns_Liste, FALSE, TRUE);

	ctree_Dns_Liste = gtk_ctree_new (1, 0);
	gtk_widget_show (ctree_Dns_Liste);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Dns_Liste), ctree_Dns_Liste);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Dns_Liste), 0, 80);
	gtk_clist_column_titles_hide (GTK_CLIST (ctree_Dns_Liste));

	scrolledwindow_Dns_Info = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Dns_Info);
	gtk_paned_pack2 (GTK_PANED (hpaned_Dns), scrolledwindow_Dns_Info, TRUE, TRUE);

	ctree_Dns_Info = gtk_ctree_new (3, 0);
	gtk_widget_show (ctree_Dns_Info);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Dns_Info), ctree_Dns_Info);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Dns_Info), 0, 80);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Dns_Info), 1, 80);
	gtk_clist_set_column_width (GTK_CLIST (ctree_Dns_Info), 2, 80);
	gtk_clist_column_titles_show (GTK_CLIST (ctree_Dns_Info));

	label_Dns_Info_Nom = gtk_label_new ("Nom");
	gtk_widget_show (label_Dns_Info_Nom);
	gtk_clist_set_column_widget (GTK_CLIST (ctree_Dns_Info), 0, label_Dns_Info_Nom);

	label_Dns_Info_Type = gtk_label_new ("Type");
	gtk_widget_show (label_Dns_Info_Type);
	gtk_clist_set_column_widget (GTK_CLIST (ctree_Dns_Info), 1, label_Dns_Info_Type);

	label_Dns_Info_Donnees = gtk_label_new ("Données");
	gtk_widget_show (label_Dns_Info_Donnees);
	gtk_clist_set_column_widget (GTK_CLIST (ctree_Dns_Info), 2, label_Dns_Info_Donnees);

	g_signal_connect ((gpointer) window_Dns, "destroy", G_CALLBACK (on_window_Dns_destroy), NULL);
	g_signal_connect ((gpointer) ctree_Dns_Liste, "tree_select_row", G_CALLBACK (on_ctree_Dns_Liste_tree_select_row), NULL);
	g_signal_connect ((gpointer) ctree_Dns_Info, "tree_select_row", G_CALLBACK (on_ctree_Dns_Info_tree_select_row), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_Dns, window_Dns, "window_Dns");
	GLADE_HOOKUP_OBJECT (window_Dns, hpaned_Dns, "hpaned_Dns");
	GLADE_HOOKUP_OBJECT (window_Dns, scrolledwindow_Dns_Liste, "scrolledwindow_Dns_Liste");
	GLADE_HOOKUP_OBJECT (window_Dns, ctree_Dns_Liste, "ctree_Dns_Liste");
	GLADE_HOOKUP_OBJECT (window_Dns, scrolledwindow_Dns_Info, "scrolledwindow_Dns_Info");
	GLADE_HOOKUP_OBJECT (window_Dns, ctree_Dns_Info, "ctree_Dns_Info");
	GLADE_HOOKUP_OBJECT (window_Dns, label_Dns_Info_Nom, "label_Dns_Info_Nom");
	GLADE_HOOKUP_OBJECT (window_Dns, label_Dns_Info_Type, "label_Dns_Info_Type");
	GLADE_HOOKUP_OBJECT (window_Dns, label_Dns_Info_Donnees, "label_Dns_Info_Donnees");

	return window_Dns;
}

GtkWidget *create_messagebox (void)
{
	GtkWidget *messagebox;
	GtkWidget *dialog_vbox;
	GtkWidget *dialog_action_area;
	GtkWidget *button_Message;

	messagebox = gnome_message_box_new ("Message", GNOME_MESSAGE_BOX_WARNING, NULL);
	gtk_window_set_title (GTK_WINDOW (messagebox), "Information");
	gtk_window_set_position (GTK_WINDOW (messagebox), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (messagebox), FALSE);

	dialog_vbox = GNOME_DIALOG (messagebox)->vbox;
	gtk_widget_show (dialog_vbox);

	dialog_action_area = GNOME_DIALOG (messagebox)->action_area;
	gtk_widget_show (dialog_action_area);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area), GTK_BUTTONBOX_SPREAD);

	gnome_dialog_append_button (GNOME_DIALOG (messagebox), "gtk-ok");
	button_Message = GTK_WIDGET (g_list_last (GNOME_DIALOG (messagebox)->buttons)->data);
	gtk_widget_show (button_Message);
	GTK_WIDGET_SET_FLAGS (button_Message, GTK_CAN_DEFAULT);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (messagebox, messagebox, "messagebox");
	GLADE_HOOKUP_OBJECT_NO_REF (messagebox, dialog_vbox, "dialog_vbox");
	GLADE_HOOKUP_OBJECT_NO_REF (messagebox, dialog_action_area, "dialog_action_area");
	GLADE_HOOKUP_OBJECT (messagebox, button_Message, "button_Message");

	return messagebox;
}

GtkWidget *create_window_Recherche_Machine (void)
{
	GSList   *radiobutton_Recherche_Adresse_Mac_group = NULL;

	accel_group = gtk_accel_group_new ();

	window_Recherche_Machine = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_Recherche_Machine, 322, 238);
	gtk_window_set_title (GTK_WINDOW (window_Recherche_Machine), "Search a host");
	gtk_window_set_position (GTK_WINDOW (window_Recherche_Machine), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window_Recherche_Machine), FALSE);
	window_Recherche_Machine_icon_pixbuf = create_pixbuf ("/usr/share/pixmaps/AutoScan/AutoScan.png");
	if (window_Recherche_Machine_icon_pixbuf) {
		gtk_window_set_icon (GTK_WINDOW (window_Recherche_Machine), window_Recherche_Machine_icon_pixbuf);
		gdk_pixbuf_unref (window_Recherche_Machine_icon_pixbuf);
	}

	fixed_Recherche = gtk_fixed_new ();
	gtk_widget_show (fixed_Recherche);
	gtk_container_add (GTK_CONTAINER (window_Recherche_Machine), fixed_Recherche);

	entry_Recherche_Adresse_Mac = gtk_entry_new ();
	gtk_widget_show (entry_Recherche_Adresse_Mac);
	gtk_fixed_put (GTK_FIXED (fixed_Recherche), entry_Recherche_Adresse_Mac, 64, 40);
	gtk_widget_set_size_request (entry_Recherche_Adresse_Mac, 232, 24);

	entry_Recherche_Adresse_Ip = gtk_entry_new ();
	gtk_widget_show (entry_Recherche_Adresse_Ip);
	gtk_fixed_put (GTK_FIXED (fixed_Recherche), entry_Recherche_Adresse_Ip, 64, 96);
	gtk_widget_set_size_request (entry_Recherche_Adresse_Ip, 232, 24);

	entry_Recherche_Nom_NetBios = gtk_entry_new ();
	gtk_widget_show (entry_Recherche_Nom_NetBios);
	gtk_fixed_put (GTK_FIXED (fixed_Recherche), entry_Recherche_Nom_NetBios, 64, 152);
	gtk_widget_set_size_request (entry_Recherche_Nom_NetBios, 232, 24);

	radiobutton_Recherche_Adresse_Mac = gtk_radio_button_new_with_mnemonic (NULL, "by the Mac address");
	gtk_widget_show (radiobutton_Recherche_Adresse_Mac);
	gtk_fixed_put (GTK_FIXED (fixed_Recherche), radiobutton_Recherche_Adresse_Mac, 32, 16);
	gtk_widget_set_size_request (radiobutton_Recherche_Adresse_Mac, 168, 24);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_Recherche_Adresse_Mac), radiobutton_Recherche_Adresse_Mac_group);
	radiobutton_Recherche_Adresse_Mac_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_Recherche_Adresse_Mac));

	radiobutton_Recherche_Nom_NetBios = gtk_radio_button_new_with_mnemonic (NULL, "by the hostname");
	gtk_widget_show (radiobutton_Recherche_Nom_NetBios);
	gtk_fixed_put (GTK_FIXED (fixed_Recherche), radiobutton_Recherche_Nom_NetBios, 32, 128);
	gtk_widget_set_size_request (radiobutton_Recherche_Nom_NetBios, 168, 24);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_Recherche_Nom_NetBios), radiobutton_Recherche_Adresse_Mac_group);
	radiobutton_Recherche_Adresse_Mac_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_Recherche_Nom_NetBios));

	radiobutton_Recherche_Adresse_Ip = gtk_radio_button_new_with_mnemonic (NULL, "by the IP address");
	gtk_widget_show (radiobutton_Recherche_Adresse_Ip);
	gtk_fixed_put (GTK_FIXED (fixed_Recherche), radiobutton_Recherche_Adresse_Ip, 32, 72);
	gtk_widget_set_size_request (radiobutton_Recherche_Adresse_Ip, 168, 24);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_Recherche_Adresse_Ip), radiobutton_Recherche_Adresse_Mac_group);
	radiobutton_Recherche_Adresse_Mac_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_Recherche_Adresse_Ip));

	button_Recherche_Annuler = gtk_button_new ();
	gtk_widget_show (button_Recherche_Annuler);
	gtk_fixed_put (GTK_FIXED (fixed_Recherche), button_Recherche_Annuler, 184, 192);
	gtk_widget_set_size_request (button_Recherche_Annuler, 96, 32);

	alignment_Recherche_Annuler = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Recherche_Annuler);
	gtk_container_add (GTK_CONTAINER (button_Recherche_Annuler), alignment_Recherche_Annuler);

	hbox_Recherche_Annuler = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Recherche_Annuler);
	gtk_container_add (GTK_CONTAINER (alignment_Recherche_Annuler), hbox_Recherche_Annuler);

	image_Recherche_Annuler = create_pixmap (window_Recherche_Machine, "/usr/share/pixmaps/AutoScan/button_cancel.png");
	gtk_widget_show (image_Recherche_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Recherche_Annuler), image_Recherche_Annuler, FALSE, FALSE, 0);

	label_Recherche_Annuler = gtk_label_new_with_mnemonic ("Cancel");
	gtk_widget_show (label_Recherche_Annuler);
	gtk_box_pack_start (GTK_BOX (hbox_Recherche_Annuler), label_Recherche_Annuler, FALSE, FALSE, 0);

	button_Recherche_Valider = gtk_button_new ();
	gtk_widget_show (button_Recherche_Valider);
	gtk_fixed_put (GTK_FIXED (fixed_Recherche), button_Recherche_Valider, 56, 192);
	gtk_widget_set_size_request (button_Recherche_Valider, 104, 32);
	gtk_widget_add_accelerator (button_Recherche_Valider, "clicked", accel_group, GDK_Return, 0, GTK_ACCEL_VISIBLE);

	alignment_Recherche_Valider = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Recherche_Valider);
	gtk_container_add (GTK_CONTAINER (button_Recherche_Valider), alignment_Recherche_Valider);

	hbox_Recherche_Valider = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Recherche_Valider);
	gtk_container_add (GTK_CONTAINER (alignment_Recherche_Valider), hbox_Recherche_Valider);

	image_Recherche_Valider = create_pixmap (window_Recherche_Machine, "/usr/share/pixmaps/AutoScan/button_ok.png");
	gtk_widget_show (image_Recherche_Valider);
	gtk_box_pack_start (GTK_BOX (hbox_Recherche_Valider), image_Recherche_Valider, FALSE, FALSE, 0);

	label_Recherche_Valider = gtk_label_new_with_mnemonic ("Ok");
	gtk_widget_show (label_Recherche_Valider);
	gtk_box_pack_start (GTK_BOX (hbox_Recherche_Valider), label_Recherche_Valider, FALSE, FALSE, 0);

	g_signal_connect ((gpointer) window_Recherche_Machine, "destroy", G_CALLBACK (on_window_Recherche_Machine_destroy), NULL);
	g_signal_connect ((gpointer) radiobutton_Recherche_Adresse_Mac, "toggled", G_CALLBACK (on_radiobutton_Recherche_Adresse_Mac_toggled), NULL);
	g_signal_connect ((gpointer) radiobutton_Recherche_Nom_NetBios, "toggled", G_CALLBACK (on_radiobutton_Recherche_Nom_NetBios_toggled), NULL);
	g_signal_connect ((gpointer) radiobutton_Recherche_Adresse_Ip, "toggled", G_CALLBACK (on_radiobutton_Recherche_Adresse_Ip_toggled), NULL);
	g_signal_connect ((gpointer) button_Recherche_Annuler, "clicked", G_CALLBACK (on_button_Recherche_Annuler_clicked), NULL);
	g_signal_connect ((gpointer) button_Recherche_Valider, "clicked", G_CALLBACK (on_button_Recherche_Valider_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_Recherche_Machine, window_Recherche_Machine, "window_Recherche_Machine");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, fixed_Recherche, "fixed_Recherche");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, entry_Recherche_Adresse_Mac, "entry_Recherche_Adresse_Mac");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, entry_Recherche_Adresse_Ip, "entry_Recherche_Adresse_Ip");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, entry_Recherche_Nom_NetBios, "entry_Recherche_Nom_NetBios");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, radiobutton_Recherche_Adresse_Mac, "radiobutton_Recherche_Adresse_Mac");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, radiobutton_Recherche_Nom_NetBios, "radiobutton_Recherche_Nom_NetBios");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, radiobutton_Recherche_Adresse_Ip, "radiobutton_Recherche_Adresse_Ip");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, button_Recherche_Annuler, "button_Recherche_Annuler");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, alignment_Recherche_Annuler, "alignment_Recherche_Annuler");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, hbox_Recherche_Annuler, "hbox_Recherche_Annuler");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, image_Recherche_Annuler, "image_Recherche_Annuler");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, label_Recherche_Annuler, "label_Recherche_Annuler");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, button_Recherche_Valider, "button_Recherche_Valider");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, alignment_Recherche_Valider, "alignment_Recherche_Valider");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, hbox_Recherche_Valider, "hbox_Recherche_Valider");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, image_Recherche_Valider, "image_Recherche_Valider");
	GLADE_HOOKUP_OBJECT (window_Recherche_Machine, label_Recherche_Valider, "label_Recherche_Valider");

	gtk_window_add_accel_group (GTK_WINDOW (window_Recherche_Machine), accel_group);

	return window_Recherche_Machine;
}

GtkWidget *create_window_Resultat_Script (void)
{
	window_Resultat_Script = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_Resultat_Script, 582, 508);
	gtk_window_set_title (GTK_WINDOW (window_Resultat_Script), "result of the scripts");
	gtk_window_set_position (GTK_WINDOW (window_Resultat_Script), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window_Resultat_Script), FALSE);

	fixed_Resultat_Script = gtk_fixed_new ();
	gtk_widget_show (fixed_Resultat_Script);
	gtk_container_add (GTK_CONTAINER (window_Resultat_Script), fixed_Resultat_Script);

	scrolledwindow_Resultat_Script = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Resultat_Script);
	gtk_fixed_put (GTK_FIXED (fixed_Resultat_Script), scrolledwindow_Resultat_Script, 8, 8);
	gtk_widget_set_size_request (scrolledwindow_Resultat_Script, 568, 448);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_Resultat_Script), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	textview_Resultat_Script = gtk_text_new (NULL, NULL);
	gtk_widget_show (textview_Resultat_Script);
	gtk_text_set_editable (textview_Resultat_Script, FALSE);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Resultat_Script), textview_Resultat_Script);

	button_Resultat_Script_Fermer = gtk_button_new_from_stock ("gtk-close");
	gtk_widget_show (button_Resultat_Script_Fermer);
	gtk_fixed_put (GTK_FIXED (fixed_Resultat_Script), button_Resultat_Script_Fermer, 424, 464);
	gtk_widget_set_size_request (button_Resultat_Script_Fermer, 104, 32);

	g_signal_connect ((gpointer) window_Resultat_Script, "destroy", G_CALLBACK (on_window_Resultat_Script_destroy), NULL);
	g_signal_connect ((gpointer) button_Resultat_Script_Fermer, "clicked", G_CALLBACK (on_button_Resultat_Script_Fermer_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_Resultat_Script, window_Resultat_Script, "window_Resultat_Script");
	GLADE_HOOKUP_OBJECT (window_Resultat_Script, fixed_Resultat_Script, "fixed_Resultat_Script");
	GLADE_HOOKUP_OBJECT (window_Resultat_Script, scrolledwindow_Resultat_Script, "scrolledwindow_Resultat_Script");
	GLADE_HOOKUP_OBJECT (window_Resultat_Script, textview_Resultat_Script, "textview_Resultat_Script");
	GLADE_HOOKUP_OBJECT (window_Resultat_Script, button_Resultat_Script_Fermer, "button_Resultat_Script_Fermer");

	return window_Resultat_Script;
}

GtkWidget *create_window_demarrage (void)
{
	window_demarrage = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_demarrage, 602, 328);
	gtk_window_set_title (GTK_WINDOW (window_demarrage), "Start");
	gtk_window_set_position (GTK_WINDOW (window_demarrage), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window_demarrage), FALSE);
	window_demarrage_icon_pixbuf = create_pixbuf ("/usr/share/pixmaps/AutoScan/AutoScan.png");
	if (window_demarrage_icon_pixbuf) {
		gtk_window_set_icon (GTK_WINDOW (window_demarrage), window_demarrage_icon_pixbuf);
		gdk_pixbuf_unref (window_demarrage_icon_pixbuf);
	}

	fixed_Demarrage = gtk_fixed_new ();
	gtk_widget_show (fixed_Demarrage);
	gtk_container_add (GTK_CONTAINER (window_demarrage), fixed_Demarrage);

	viewport_Demarrage_Titre = gtk_viewport_new (NULL, NULL);
	gtk_widget_show (viewport_Demarrage_Titre);
	gtk_fixed_put (GTK_FIXED (fixed_Demarrage), viewport_Demarrage_Titre, 5, 16);
	gtk_widget_set_size_request (viewport_Demarrage_Titre, 595, 24);

	fixed_Demarrage_Titre = gtk_fixed_new ();
	gtk_widget_show (fixed_Demarrage_Titre);
	gtk_container_add (GTK_CONTAINER (viewport_Demarrage_Titre), fixed_Demarrage_Titre);

	label_Demarrage_Titre = gtk_label_new ("Select the desired network");
	gtk_widget_show (label_Demarrage_Titre);
	gtk_fixed_put (GTK_FIXED (fixed_Demarrage_Titre), label_Demarrage_Titre, 184, 0);
	gtk_widget_set_size_request (label_Demarrage_Titre, 208, 16);

	iconlist_Demarrage_Fichier = gnome_icon_list_new (78, NULL, 0);
	gtk_widget_show (iconlist_Demarrage_Fichier);
	gtk_fixed_put (GTK_FIXED (fixed_Demarrage), iconlist_Demarrage_Fichier, 8, 48);
	gtk_widget_set_size_request (iconlist_Demarrage_Fichier, 584, 224);

	button_Demarrage_Preferences = gtk_button_new ();
	gtk_widget_show (button_Demarrage_Preferences);
	gtk_fixed_put (GTK_FIXED (fixed_Demarrage), button_Demarrage_Preferences, 256, 280);
	gtk_widget_set_size_request (button_Demarrage_Preferences, 120, 36);

	alignment_Demarrage_Preferences = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Demarrage_Preferences);
	gtk_container_add (GTK_CONTAINER (button_Demarrage_Preferences), alignment_Demarrage_Preferences);

	hbox_Demarrage_Preferences = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Demarrage_Preferences);
	gtk_container_add (GTK_CONTAINER (alignment_Demarrage_Preferences), hbox_Demarrage_Preferences);

	image_Demarrage_Preferences = create_pixmap (window_demarrage, "/usr/share/pixmaps/AutoScan/configure.png");
	gtk_widget_show (image_Demarrage_Preferences);
	gtk_box_pack_start (GTK_BOX (hbox_Demarrage_Preferences), image_Demarrage_Preferences, FALSE, FALSE, 0);

	label_Demarrage_Preferences = gtk_label_new_with_mnemonic ("Preferences");
	gtk_widget_show (label_Demarrage_Preferences);
	gtk_box_pack_start (GTK_BOX (hbox_Demarrage_Preferences), label_Demarrage_Preferences, FALSE, FALSE, 0);

	checkbutton_Demarrage_Ajouter = gtk_check_button_new_with_mnemonic ("To add the new found machines");
	gtk_widget_show (checkbutton_Demarrage_Ajouter);
	gtk_fixed_put (GTK_FIXED (fixed_Demarrage), checkbutton_Demarrage_Ajouter, 8, 280);
	gtk_widget_set_size_request (checkbutton_Demarrage_Ajouter, 224, 16);

	checkbutton_Demarrage_Mise_A_Jour = gtk_check_button_new_with_mnemonic ("Updated state of the machines");
	//gtk_widget_show (checkbutton_Demarrage_Mise_A_Jour);
	gtk_fixed_put (GTK_FIXED (fixed_Demarrage), checkbutton_Demarrage_Mise_A_Jour, 8, 296);
	gtk_widget_set_size_request (checkbutton_Demarrage_Mise_A_Jour, 192, 22);

	button_Demarrage_Annuler = gtk_button_new ();
	gtk_widget_show (button_Demarrage_Annuler);
	gtk_fixed_put (GTK_FIXED (fixed_Demarrage), button_Demarrage_Annuler, 384, 280);
	gtk_widget_set_size_request (button_Demarrage_Annuler, 96, 36);

	alignment_Demarrage_Valider = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Demarrage_Valider);
	gtk_container_add (GTK_CONTAINER (button_Demarrage_Annuler), alignment_Demarrage_Valider);

	hbox_Demarrage_Valider = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Demarrage_Valider);
	gtk_container_add (GTK_CONTAINER (alignment_Demarrage_Valider), hbox_Demarrage_Valider);

	image_Demarrage_Valider = create_pixmap (window_demarrage, "/usr/share/pixmaps/AutoScan/button_cancel.png");
	gtk_widget_show (image_Demarrage_Valider);
	gtk_box_pack_start (GTK_BOX (hbox_Demarrage_Valider), image_Demarrage_Valider, FALSE, FALSE, 0);

	label_Demarrage_Valider = gtk_label_new_with_mnemonic ("Cancel");
	gtk_widget_show (label_Demarrage_Valider);
	gtk_box_pack_start (GTK_BOX (hbox_Demarrage_Valider), label_Demarrage_Valider, FALSE, FALSE, 0);

	button_Demarrage_Valider = gtk_button_new ();
	gtk_widget_show (button_Demarrage_Valider);
	gtk_fixed_put (GTK_FIXED (fixed_Demarrage), button_Demarrage_Valider, 488, 280);
	gtk_widget_set_size_request (button_Demarrage_Valider, 104, 36);

	alignment_Demarrage_Valider = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Demarrage_Valider);
	gtk_container_add (GTK_CONTAINER (button_Demarrage_Valider), alignment_Demarrage_Valider);

	hbox_Demarrage_Valider = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Demarrage_Valider);
	gtk_container_add (GTK_CONTAINER (alignment_Demarrage_Valider), hbox_Demarrage_Valider);

	image_Demarrage_Valider = create_pixmap (window_demarrage, "/usr/share/pixmaps/AutoScan/button_ok.png");
	gtk_widget_show (image_Demarrage_Valider);
	gtk_box_pack_start (GTK_BOX (hbox_Demarrage_Valider), image_Demarrage_Valider, FALSE, FALSE, 0);

	label_Demarrage_Valider = gtk_label_new_with_mnemonic ("Ok");
	gtk_widget_show (label_Demarrage_Valider);
	gtk_box_pack_start (GTK_BOX (hbox_Demarrage_Valider), label_Demarrage_Valider, FALSE, FALSE, 0);

	g_signal_connect ((gpointer) window_demarrage, "destroy", G_CALLBACK (on_window_demarrage_destroy), NULL);
	g_signal_connect ((gpointer) iconlist_Demarrage_Fichier, "select_icon", G_CALLBACK (on_iconlist_Demarrage_Fichier_select_icon), NULL);
	g_signal_connect ((gpointer) button_Demarrage_Preferences, "clicked", G_CALLBACK (on_button_Demarrage_Preferences_clicked), NULL);
	g_signal_connect ((gpointer) button_Demarrage_Annuler, "clicked", G_CALLBACK (on_button_Demarrage_Annuler_clicked), NULL);
	g_signal_connect ((gpointer) button_Demarrage_Valider, "clicked", G_CALLBACK (on_button_Demarrage_Valider_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_demarrage, window_demarrage, "window_demarrage");
	GLADE_HOOKUP_OBJECT (window_demarrage, fixed_Demarrage, "fixed_Demarrage");
	GLADE_HOOKUP_OBJECT (window_demarrage, viewport_Demarrage_Titre, "viewport_Demarrage_Titre");
	GLADE_HOOKUP_OBJECT (window_demarrage, fixed_Demarrage_Titre, "fixed_Demarrage_Titre");
	GLADE_HOOKUP_OBJECT (window_demarrage, label_Demarrage_Titre, "label_Demarrage_Titre");
	GLADE_HOOKUP_OBJECT (window_demarrage, iconlist_Demarrage_Fichier, "iconlist_Demarrage_Fichier");
	GLADE_HOOKUP_OBJECT (window_demarrage, button_Demarrage_Preferences, "button_Demarrage_Preferences");
	GLADE_HOOKUP_OBJECT (window_demarrage, alignment_Demarrage_Preferences, "alignment_Demarrage_Preferences");
	GLADE_HOOKUP_OBJECT (window_demarrage, hbox_Demarrage_Preferences, "hbox_Demarrage_Preferences");
	GLADE_HOOKUP_OBJECT (window_demarrage, image_Demarrage_Preferences, "image_Demarrage_Preferences");
	GLADE_HOOKUP_OBJECT (window_demarrage, label_Demarrage_Preferences, "label_Demarrage_Preferences");
	GLADE_HOOKUP_OBJECT (window_demarrage, checkbutton_Demarrage_Ajouter, "checkbutton_Demarrage_Ajouter");
	GLADE_HOOKUP_OBJECT (window_demarrage, checkbutton_Demarrage_Mise_A_Jour, "checkbutton_Demarrage_Mise_A_Jour");
	GLADE_HOOKUP_OBJECT (window_demarrage, button_Demarrage_Annuler, "button_Demarrage_Annuler");
	GLADE_HOOKUP_OBJECT (window_demarrage, alignment_Demarrage_Valider, "alignment_Demarrage_Valider");
	GLADE_HOOKUP_OBJECT (window_demarrage, hbox_Demarrage_Valider, "hbox_Demarrage_Valider");
	GLADE_HOOKUP_OBJECT (window_demarrage, image_Demarrage_Valider, "image_Demarrage_Valider");
	GLADE_HOOKUP_OBJECT (window_demarrage, label_Demarrage_Valider, "label_Demarrage_Valider");
	GLADE_HOOKUP_OBJECT (window_demarrage, button_Demarrage_Valider, "button_Demarrage_Valider");
	GLADE_HOOKUP_OBJECT (window_demarrage, alignment_Demarrage_Valider, "alignment_Demarrage_Valider");
	GLADE_HOOKUP_OBJECT (window_demarrage, hbox_Demarrage_Valider, "hbox_Demarrage_Valider");
	GLADE_HOOKUP_OBJECT (window_demarrage, image_Demarrage_Valider, "image_Demarrage_Valider");
	GLADE_HOOKUP_OBJECT (window_demarrage, label_Demarrage_Valider, "label_Demarrage_Valider");

	return window_demarrage;
}

GtkWidget *create_window_Contributions (void)
{
	window_Contributions = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_Contributions, 412, 232);
	gtk_window_set_title (GTK_WINDOW (window_Contributions), "Contributions");
	gtk_window_set_position (GTK_WINDOW (window_Contributions), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window_Contributions), FALSE);
	window_Contributions_icon_pixbuf = create_pixbuf ("/usr/share/pixmaps/AutoScan/AutoScan.png");
	if (window_Contributions_icon_pixbuf) {
		gtk_window_set_icon (GTK_WINDOW (window_Contributions), window_Contributions_icon_pixbuf);
		gdk_pixbuf_unref (window_Contributions_icon_pixbuf);
	}

	fixed_Contributions = gtk_fixed_new ();
	gtk_widget_show (fixed_Contributions);
	gtk_container_add (GTK_CONTAINER (window_Contributions), fixed_Contributions);

	scrolledwindow_Contributions = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Contributions);
	gtk_fixed_put (GTK_FIXED (fixed_Contributions), scrolledwindow_Contributions, 16, 64);
	gtk_widget_set_size_request (scrolledwindow_Contributions, 384, 110);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_Contributions), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	textview_Contributions = gtk_text_new (NULL, NULL);
	gtk_widget_show (textview_Contributions);
	gtk_text_set_editable (textview_Contributions, FALSE);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Contributions), textview_Contributions);
	//gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_Contributions)), "Subject : Unknown Equipment\n\nSystem Name : HP Printer\nMD5 signature : A6B5B035489CE54F400DB93A4495251C\nType : Printer", -1);

	label_Contributions = gtk_label_new ("Contributions to this \ndatabase are welcome. Please send this email \nto tlagarde@users.sourceforge.net.");
	gtk_widget_show (label_Contributions);
	gtk_fixed_put (GTK_FIXED (fixed_Contributions), label_Contributions, 72, 0);
	gtk_widget_set_size_request (label_Contributions, 280, 56);
	gtk_label_set_justify (GTK_LABEL (label_Contributions), GTK_JUSTIFY_CENTER);

	button_Contributions = gtk_button_new_with_mnemonic ("Thanks You");
	gtk_widget_show (button_Contributions);
	gtk_fixed_put (GTK_FIXED (fixed_Contributions), button_Contributions, 288, 184);
	gtk_widget_set_size_request (button_Contributions, 104, 32);

	g_signal_connect ((gpointer) window_Contributions, "destroy", G_CALLBACK (on_window_Contributions_destroy), NULL);
	g_signal_connect ((gpointer) button_Contributions, "clicked", G_CALLBACK (on_button_Contributions_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_Contributions, window_Contributions, "window_Contributions");
	GLADE_HOOKUP_OBJECT (window_Contributions, fixed_Contributions, "fixed_Contributions");
	GLADE_HOOKUP_OBJECT (window_Contributions, scrolledwindow_Contributions, "scrolledwindow_Contributions");
	GLADE_HOOKUP_OBJECT (window_Contributions, textview_Contributions, "textview_Contributions");
	GLADE_HOOKUP_OBJECT (window_Contributions, label_Contributions, "label_Contributions");
	GLADE_HOOKUP_OBJECT (window_Contributions, button_Contributions, "button_Contributions");

	return window_Contributions;
}

GtkWidget *create_window_Notification (void)
{
	window_Notification = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window_Notification, 325, 181);
	gtk_window_set_title (GTK_WINDOW (window_Notification), "Information");
	gtk_window_set_position (GTK_WINDOW (window_Notification), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (window_Notification), FALSE);
	window_Notification_icon_pixbuf = create_pixbuf ("/usr/share/pixmaps/AutoScan/AutoScan.png");
	if (window_Notification_icon_pixbuf) {
		gtk_window_set_icon (GTK_WINDOW (window_Notification), window_Notification_icon_pixbuf);
		gdk_pixbuf_unref (window_Notification_icon_pixbuf);
	}

	fixed_Notifications = gtk_fixed_new ();
	gtk_widget_show (fixed_Notifications);
	gtk_container_add (GTK_CONTAINER (window_Notification), fixed_Notifications);

	label_Notifications = gtk_label_new ("To use Autoscan, right-click on the Autoscan tray icon \nand select the \"Show/Hide\" command");
	gtk_widget_show (label_Notifications);
	gtk_fixed_put (GTK_FIXED (fixed_Notifications), label_Notifications, 16, 8);
	gtk_widget_set_size_request (label_Notifications, 296, 32);

	checkbutton_Notifications = gtk_check_button_new_with_mnemonic ("Don't show this dialog again.");
	gtk_widget_show (checkbutton_Notifications);
	gtk_fixed_put (GTK_FIXED (fixed_Notifications), checkbutton_Notifications, 16, 120);
	gtk_widget_set_size_request (checkbutton_Notifications, 181, 24);

	image_Notifications = create_pixmap (window_Notification, "/usr/share/pixmaps/AutoScan/Notification.png");
	gtk_widget_show (image_Notifications);
	gtk_fixed_put (GTK_FIXED (fixed_Notifications), image_Notifications, 72, 56);
	gtk_widget_set_size_request (image_Notifications, 176, 48);

	button_Notifications_Fermer = gtk_button_new ();
	gtk_widget_show (button_Notifications_Fermer);
	gtk_fixed_put (GTK_FIXED (fixed_Notifications), button_Notifications_Fermer, 216, 136);
	gtk_widget_set_size_request (button_Notifications_Fermer, 104, 32);

	alignment_Notifications_Fermer = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment_Notifications_Fermer);
	gtk_container_add (GTK_CONTAINER (button_Notifications_Fermer), alignment_Notifications_Fermer);

	hbox_Notifications_Fermer = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox_Notifications_Fermer);
	gtk_container_add (GTK_CONTAINER (alignment_Notifications_Fermer), hbox_Notifications_Fermer);

	image_Notifications_Fermer = create_pixmap (window_Notification, "/usr/share/pixmaps/AutoScan/close.png");
	gtk_widget_show (image_Notifications_Fermer);
	gtk_box_pack_start (GTK_BOX (hbox_Notifications_Fermer), image_Notifications_Fermer, FALSE, FALSE, 0);

	label_Notifications_Fermer = gtk_label_new_with_mnemonic ("Close");
	gtk_widget_show (label_Notifications_Fermer);
	gtk_box_pack_start (GTK_BOX (hbox_Notifications_Fermer), label_Notifications_Fermer, FALSE, FALSE, 0);

	g_signal_connect ((gpointer) window_Notification, "destroy", G_CALLBACK (on_window_Notification_destroy), NULL);
	g_signal_connect ((gpointer) button_Notifications_Fermer, "clicked", G_CALLBACK (on_button_Notifications_Fermer_clicked), NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window_Notification, window_Notification, "window_Notification");
	GLADE_HOOKUP_OBJECT (window_Notification, fixed_Notifications, "fixed_Notifications");
	GLADE_HOOKUP_OBJECT (window_Notification, label_Notifications, "label_Notifications");
	GLADE_HOOKUP_OBJECT (window_Notification, checkbutton_Notifications, "checkbutton_Notifications");
	GLADE_HOOKUP_OBJECT (window_Notification, image_Notifications, "image_Notifications");
	GLADE_HOOKUP_OBJECT (window_Notification, button_Notifications_Fermer, "button_Notifications_Fermer");
	GLADE_HOOKUP_OBJECT (window_Notification, alignment_Notifications_Fermer, "alignment_Notifications_Fermer");
	GLADE_HOOKUP_OBJECT (window_Notification, hbox_Notifications_Fermer, "hbox_Notifications_Fermer");
	GLADE_HOOKUP_OBJECT (window_Notification, image_Notifications_Fermer, "image_Notifications_Fermer");
	GLADE_HOOKUP_OBJECT (window_Notification, label_Notifications_Fermer, "label_Notifications_Fermer");

	return window_Notification;
}
